;/* 
; * Demo showing the interaction with a canvas object.   
; *
; *  This file is part of xforms package
; *  T.C. Zhao and M. Overmars  (1997)
; *
; */

(require 'forms)
(require 'struct)

(define KeyPress      2)  ; JL : from X.h
(define KeyRelease    3)
(define ButtonPress   4)
(define ButtonRelease 5)
(define EnterNotify   7) 
(define LeaveNotify   8)
(define Expose       12)

;/**** Forms and Objects ****/

(define-record fdui
  (canvasform
   vdata
   cdata
   ldata
   canvas
   br
   keyboard
   mouse
   done
   misc
   menu))

(define ui (make-fdui 0 0 0 0 0 0 0 0 0 0 0))

(define canvasGC `())

(define (canvas_expose ob win w h ev d)
  (let ((dui d))
    ;XFillRectangle
    (fl_addto_browser (fdui->br dui) "Expose")
    0))

(define (canvas_key ob win w h ev d)
  (let ((dui d)
	; JL : display keycode instead of keysym
	(buf (string-append
	      "KeyPress: keycode="
	      (number->string 
	       (fl_get_xev_member ev xev_keycode)))))
    (fl_addto_browser (fdui->br dui) buf)
    0))

(define (canvas_but ob win w h ev d)
  (let ((dui d)
	(buf (string-append 
	      "Button"
	      (if (= (fl_get_xev_member ev xev_type) ButtonPress)
		  "Press"
		  "Release")
	      ":"
	      (number->string (fl_get_xev_member ev xev_button)))))
    (fl_addto_browser (fdui->br dui) buf) 
    0))

(define (canvas_misc ob win w h ev d)
  (let ((dui d))
    (fl_addto_browser (fdui->br dui) 
		      (if (= (fl_get_xev_member ev xev_type) EnterNotify)
			  "Enter canvas" 
			  "Leave canvas"))
    0))

(define (init_canvas fdui)
  (fl_add_canvas_handler (fdui->canvas ui) Expose  canvas_expose  fdui)
  (fl_add_canvas_handler (fdui->canvas ui) KeyPress  canvas_key  fdui)
  (fl_add_canvas_handler (fdui->canvas ui) ButtonPress canvas_but fdui)
  (fl_add_canvas_handler (fdui->canvas ui) ButtonRelease canvas_but fdui)

  (fl_set_button (fdui->mouse ui) 1)
  (fl_set_button (fdui->keyboard ui) 1)
  ; XCreateGC
  (set! canvasGC (fl_display))
  ;XSetForeground(fl_display()  canvasGC  fl_get_flcolor(FL_BLACK));
  ;(fl_set_foreground (fl_get_gc) (fl_get_flcolor FL_BLACK)) 
  )

;/* callbacks */
(define (sensitive_setting ob event)
  (if (= (fl_get_button ob) FL_TRUE)
      (fl_add_canvas_handler (fdui->canvas ui) event 
			     (if (= event KeyPress)
				 canvas_key canvas_but)
			     ui)
      (fl_remove_canvas_handler (fdui->canvas ui) event
				(if (= event KeyPress)
				    canvas_key canvas_but))
      )
  )

(define (disable_it ob data)
  (if (= (fl_get_button ob) FL_TRUE)
      (fl_deactivate_object (fdui->canvas ui))
      (fl_activate_object (fdui->canvas ui))))

(define (hide_it ob all)
  (if (= all FL_TRUE)
      (begin
	(fl_hide_form (fdui->canvasform ui))
	(fl_show_form (fdui->canvasform ui)
                      FL_PLACE_CENTER  FL_TRANSIENT  "canvas")
	(if #f
	    (fl_remove_selected_xevent (fl_get_object_window (fdui->canvas ui))
				       OwnerGrabButtonMask)
	    )
	)
      (if (= (fl_get_object_visible (fdui->canvas ui)) FL_TRUE)
	  (begin
            (fl_hide_object (fdui->canvas ui))
            (fl_set_object_label ob "ShowCanvas"))
	  (begin
            (fl_show_object (fdui->canvas ui))
            (fl_set_object_label ob "HideCanvas")))))

(define (misc_cb ob data)
  (if (= (fl_get_button ob) FL_TRUE)
      (begin
	(fl_add_canvas_handler (fdui->canvas ui)
			       EnterNotify  
			       canvas_misc ui) 
	(fl_add_canvas_handler (fdui->canvas ui)
			       LeaveNotify 
			       canvas_misc ui))
      (begin
	(fl_remove_canvas_handler (fdui->canvas ui) 
				  EnterNotify  canvas_misc)
	(fl_remove_canvas_handler (fdui->canvas ui)
				  LeaveNotify  canvas_misc))))

(define (canvas . args)
  (fl_set_border_width -2)
  (fl_initialize args)
  (set-fdui-canvasform! ui (create_form_canvasform))
  ;/* fill-in form initialization code */
  (init_canvas ui)
  (fl_addto_menu (fdui->menu ui) "Item1|Item2|Item3|Item4")
  (fl_show_form (fdui->canvasform ui) 
                FL_PLACE_FREE FL_FULLBORDER "canvasform")
  (if #f
      (fl_remove_selected_xevent (fl_get_object_window 
				  (fdui->canvas ui))
				 OwnerGrabButtonMask)
      )
  (letrec ((loop (lambda ()
		   (if (not (equal? (fdui->done ui) (fl_do_forms)))
		       (loop)))))
    (loop))
  (exit 0))

;/*******************/
(define (create_form_canvasform)
  (let ((obj '())
	(form '()))
    (set! form (fl_bgn_form FL_NO_BOX  450  280))
    (set! obj (fl_add_box FL_UP_BOX 0 0 450 280 ""))
    (set! obj (fl_add_canvas FL_SCROLLED_CANVAS 20 40 155 187 ""))
    (set-fdui-canvas! ui obj)
    (set! obj (fl_add_browser FL_NORMAL_BROWSER 188 40 152 187 ""))
    (set-fdui-br! ui obj)
    (set! obj (fl_add_button FL_PUSH_BUTTON 30 236 90 27 "Deactivate"))
    (fl_set_object_callback obj disable_it 0)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 120 236 90 27 "HideCanvas"))
    (fl_set_object_callback obj  hide_it  0)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 210 236 90 27 "HideForm"))
    (fl_set_object_callback obj  hide_it  1)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 300 236 90 27 "Done"))
    (set-fdui-done! ui obj)
    (set! obj (fl_add_text FL_NORMAL_TEXT 130 10 120 20 "Canvas"))
    (fl_set_object_lsize obj FL_MEDIUM_SIZE)
    (fl_set_object_lalign obj FL_ALIGN_CENTER)
    (fl_set_object_lstyle obj FL_BOLD_STYLE)
    (set! obj (fl_add_menu FL_PULLDOWN_MENU  20 10  45 22 "Menu"))
    (set-fdui-menu! ui obj)
    (fl_set_object_shortcut obj "#m"  1)
    (fl_set_object_lstyle obj  FL_BOLD_STYLE)
    (set! obj (fl_add_checkbutton FL_PUSH_BUTTON 345 40 76 26 "Keyboard"))
    (set-fdui-keyboard! ui obj)
    (fl_set_object_color obj FL_COL1 FL_BLUE)
    (fl_set_object_callback obj sensitive_setting KeyPress)
    (set! obj (fl_add_checkbutton FL_PUSH_BUTTON 345 70 76 26 "Mouse"))
    (set-fdui-mouse! ui obj)
    (fl_set_object_color obj FL_COL1 FL_BLUE)
    (fl_set_object_callback obj sensitive_setting ButtonPress)
    (set! obj (fl_add_checkbutton FL_PUSH_BUTTON 345 100 74 26 "Enter/Leave"))
    (set-fdui-misc! ui obj)
    (fl_set_object_color obj FL_COL1 FL_BLUE)
    (fl_set_object_callback obj misc_cb ButtonPress)

    (fl_end_form)
    form))


(canvas)






