;/*
; * Test browser/slider and color handling. Select a colorname,
; * the program will show the color. Change the sliders, the
; * program will pick a colorname that has the closest color.
; *
; *  This file is part of xforms package.
; *  T.C. Zhao and M. Overmars  (1997)
; */

(require 'forms)
(require 'scanf)
(require 'logical)

(define MAX_RGB 3000)

(define cl '())
(define rescol '())
(define dbobj '())
(define colbr '())
(define rs '())
(define gs '())
(define bs '())
(define dbname "")

;/* the RGB data file does not have a standard location on unix. */
(define rgbfile
  (if (equal? (software-type) 'vms)
      "SYS$MANAGER:DECW$RGB.DAT"
      (if (or (equal? (software-type) 'emx) ; JL : ??
	      (equal? (software-type) 'os/2))
	  "/XFree86/lib/X11/rgb.txt"
	  "/usr/lib/X11/rgb.txt"))) ; JL : 'unix

(define rgbdb '())

(define (colbrowser . args)
  (fl_initialize args)
  
  (create_form_cl)
  (set! dbname rgbfile)
  
  (if (= (load_browser dbname) FL_TRUE)
      (fl_set_object_label dbobj dbname)
      (fl_set_object_label dbobj "None"))
  
  (let* ((clsiz (fl_get_form_size cl))
	 (cl_w (car clsiz))
	 (cl_h (cadr clsiz)))
    (fl_set_form_minsize cl cl_w cl_h)
    (fl_set_form_maxsize cl (* 2 cl_w) (* 2 cl_h))
    (fl_show_form cl FL_PLACE_FREE FL_TRANSIENT "RGB Browser"))
  (letrec ((loop (lambda ()
		   (fl_do_forms)
		   (loop))))
    (loop))
  0)

(define (set_entry i)
  (let* ((db (list-ref rgbdb i))
	 (db_r (car db))
	 (db_g (cadr db))
	 (db_b (caddr db)))
    (fl_freeze_form cl)
    (fl_mapcolor (+ FL_FREE_COL4 i) 
		 (inexact->exact db_r)
		 (inexact->exact db_g)
		 (inexact->exact db_b))
    (fl_mapcolor FL_FREE_COL4 
		 (inexact->exact db_r)
		 (inexact->exact db_g)
		 (inexact->exact db_b))
    (fl_set_slider_value rs db_r)
    (fl_set_slider_value gs db_g)
    (fl_set_slider_value bs db_b)
    (fl_redraw_object rescol)
    (fl_unfreeze_form cl))
  0)

(define (br_cb ob q)
  (let ((r (fl_get_browser ob)))
    (if (<= r 0)
	0
	(set_entry (- r 1)))))

		   
	     
(define (read_entry fp)
  (let* ((buf (read-line fp))
	 (lbuf 0)
	 (r 0)
	 (g 0)
	 (b 0)
	 (n 0)
	 (cs 0))
    (if (eof-object? buf)
	#f
	(if (and (> (string-length buf) 0)
		 (char=? (string-ref buf 0) #\!)) ; JL : first line
	    (read_entry fp)
	    (if (< 3 (sscanf buf " %d %d %d %n" r g b n))
		#f
		(list r 
		      g 
		      b ; JL : ignore whitespace in name
		      (substring buf n (string-length buf))))))))



(define (load_browser fname)
  (let ((fp #f)
	(db 0)
	(dbs MAX_RGB))
    (set! fp (open-file fname "r"))
    (if (equal? fp #f)
	(begin
	  (fl_show_alert "Load" fname "Can't open" 0)
	  0)
	; /* read the items */
	(begin
	  (fl_freeze_form cl)
	  (letrec 
	      ((loop 
		(lambda (db lr lg lb)
		  (if (>= db dbs)
		      '()
		      (let ((entry (read_entry fp)))
			(if (not (equal? entry #f))
			    (let* ((r (car entry))
				   (g (cadr entry))
				   (b (caddr entry))
				   (name (cadddr entry))
				   (buf (string-append
					 "("
					 (number->string r) " "
					 (number->string g) " "
					 (number->string b) ") "
					 name)))
			      ;/* unique the entries on the fly */
			      (if (or (not (= lr r))
				      (not (= lg g))
				      (not (= lb b)))
				  (begin
				    (fl_addto_browser colbr buf)
				    (cons (list r g b)
					  (loop (+ db 1) r g b)))
				  (loop db 
					lr lg lb)))
			    '()))
		      ))))
	    (set! rgbdb (loop 0 -1 -1 -1)))
	
	
	  (close-port fp)
	  
	  ; JL : skip marking end of entries 
	  
	  (fl_set_browser_topline colbr 1)
	  (fl_select_browser_line colbr 1)
	  (set_entry 0)
	  (fl_unfreeze_form cl)
	  1
	  ))))


(define (search_entry r g b)
  (letrec ((loop 
	    (lambda (dbs i j mindiff)
	      (if (null? dbs)
		  j
		  (let* ((dbe (car dbs))
			 (db_r (car dbe))
			 (db_g (cadr dbe))
			 (db_b (caddr dbe))
			 (diffr (- r db_r))
			 (diffg (- g db_g))
			 (diffb (- b db_b))
			 (diff 
			  (if (= FL_LINEAR FL_FALSE)
			      (+ (* 3.0 (fl_abs diffr))
				 (* 5.9 (fl_abs diffg))
				 (* 1.1 (fl_abs diffb)))
			      (+ (* 3.0 (* diffr diffr))
				 (* 5.9 (* diffg diffg))
				 (* 1.1 (* diffb diffb))))))
		    (if (< diff mindiff)
			(loop (cdr dbs) (+ i 1) i diff)
			(loop (cdr dbs) (+ i 1) j mindiff)))))))
    (loop rgbdb 0 0 most-positive-fixnum)))
  

(define (search_rgb ob q)
  (let ((r (inexact->exact (fl_get_slider_value rs)))
	(g (inexact->exact (fl_get_slider_value gs)))
	(b (inexact->exact (fl_get_slider_value bs)))
	(i 0)
	(top (fl_get_browser_topline colbr)))
    (fl_freeze_form cl)
    (fl_mapcolor FL_FREE_COL4 r g b)
    (fl_redraw_object rescol)

    (set! i (search_entry r g b))
    ;/* change topline only if necessary */
    (if (or (< i top)
	    (> i (+ top 15)))
	(fl_set_browser_topline colbr (- i 8)))
    (fl_select_browser_line colbr (+ i 1))
    (fl_unfreeze_form cl)))

;/* change database */
(define (db_cb ob q)
  (let ((p (fl_show_input "Enter New Database Name" dbname)))
    (if (or (string-null? p)
	    (string=? p dbname))
	0
	(if (= (load_browser p) FL_TRUE)
	    (set! dbname p)
	    (fl_set_object_label ob dbname)))))

(define (done_cb ob q)
  (fl_finish)
  (exit 0))

(define (create_form_cl)
  (if (not (null? cl))
      0
      (let ((obj '()))
	(set! cl (fl_bgn_form FL_NO_BOX  330  385))
	(set! obj (fl_add_box FL_UP_BOX  0  0  330  385  ""))
	(fl_set_object_color obj  FL_INDIANRED  FL_COL1)

	(set! obj (fl_add_box FL_NO_BOX  40  10  250  30  "Color Browser"))
	(fl_set_object_lcol obj  FL_RED)
	(fl_set_object_lsize obj  FL_HUGE_SIZE)
	(fl_set_object_lstyle obj  (+ FL_BOLD_STYLE FL_SHADOW_STYLE))
	
	(set! obj (fl_add_button FL_NORMAL_BUTTON  40  50  250  25  ""))
	(set! dbobj obj)
	(fl_set_object_boxtype obj  FL_BORDER_BOX)
	(fl_set_object_color obj 
			     (if (= (fl_get_visual_depth) 1)
				 FL_WHITE FL_INDIANRED)
			     FL_INDIANRED)
	(fl_set_object_callback obj  db_cb  0)
	(set! obj (fl_add_valslider FL_VERT_FILL_SLIDER 225 130 30 200  ""))
	(set! rs obj)
	(fl_set_object_color obj  FL_INDIANRED  FL_RED)
	(fl_set_slider_bounds obj  0  255)
	(fl_set_slider_precision obj  0)
	(fl_set_object_callback obj  search_rgb  0)
	(fl_set_slider_return obj  0)

	(set! obj (fl_add_valslider FL_VERT_FILL_SLIDER 255 130 30 200  ""))
	(set! gs obj)
	(fl_set_object_color obj  FL_INDIANRED  FL_GREEN)
	(fl_set_slider_bounds obj  0  255)
	(fl_set_slider_precision obj  0)
	(fl_set_object_callback obj  search_rgb  1)
	(fl_set_slider_return obj  0)

	(set! obj (fl_add_valslider FL_VERT_FILL_SLIDER 285 130 30 200  ""))
	(set! bs obj)
	(fl_set_object_color obj  FL_INDIANRED  FL_BLUE)
	(fl_set_slider_bounds obj  0  255)
	(fl_set_slider_precision obj  0)
	(fl_set_object_callback obj  search_rgb  2)
	(fl_set_slider_return obj  0)
	
	(set! obj (fl_add_browser FL_HOLD_BROWSER  10  90  205  240  ""))
	(set! colbr obj)
	;/* (fl_set_browser_fontstyle obj  FL_FIXED_STYLE) */
	(fl_set_object_callback obj  br_cb  0)

	(set! obj (fl_add_button FL_NORMAL_BUTTON  135  345  80  30  "Done"))
	(fl_set_object_callback obj  done_cb  0)

	(set! obj (fl_add_box FL_FLAT_BOX  225  90  90  35  ""))
	(set! rescol obj)
	(fl_set_object_color obj  FL_FREE_COL4  FL_FREE_COL4)
	(fl_set_object_boxtype obj  FL_BORDER_BOX)

	(fl_end_form)
	(fl_scale_form cl  1.1  1.0))))

;;;
(colbrowser)

