;/*
; * $Id: demo.c,v 0.20 1994/09/22 19:46:02 zhao Exp $
; *
; * Driver for the demo/test suite
; *
; *  This file is part of xforms package
; *  M. Overmars and T.C. Zhao 
; *
; */
(require 'forms)

(define form '())

(define (demo . args)
  ;/* make sure the distribution is consistent wrt version numbers */
  (let* ((libv (fl_library_version))
	 (libver (cadr libv))
	 (librev (caddr libv)))
  (if (not (= FL_INCLUDE_VERSION (car libv)))
      (begin
	(display "header/library version mismatch\n")
	(display (string-append "   Header: " 
				(number->string FL_VERSION) "."
				(number->string FL_REVISION)))
	(display (string-append "  Library: "
				(number->string libver) "."
				(number->string librev)))
	(exit 1)))
  ;/* to cut down the "bug" reports of "can't run the demos", put . on path */
  (if (not (equal? (software-type) 'VMS))
      (if (not (string? (getenv "PATH")))
	  (putenv (string-append "PATH=" (getenv "PATH") ":.")))
      (begin ; else
	(display 
	 "  -------->WARNING<-----------")
	(display 
	 "  The subdemos will not run unless DCL$PATH has been defined\n")
	(display 
	 "  to include the directory where they live\n")))
  ; (fl_set_border_width 6) 
  (fl_initialize args)
  (fl_clear_command_log)
  ;
  (create_the_forms)
  ;
  (if (= (load_the_menu "demo.menu") FL_FALSE)
      (exit -1))
  ;
  (push_menu "@main")
  ;
  (fl_show_form form FL_PLACE_CENTER FL_TRANSIENT "Demo")
  (fl_do_forms)

  0))


;/* Loads the menu file. Returns whether successful. */
(define (load_the_menu fname)

  ;;; utility routines to read menu description file
  (define (getline ip)
    (let ((rchar (read-char ip)))
      (if (eof-object? rchar)
	  rchar
	  (if (char=? rchar #\newline)
	      ""
	      (string-append (make-string 1 rchar) (getline ip))))))
  
  (define (elim_spaces line)
    (if (eof-object? line)
	line
	(if (string=? line "")
	    ""
	    (if (or (char=? (string-ref line 0) #\space)
		    (char=? (string-ref line 0) #\tab))
		(elim_spaces (substring line 1 (string-length line)))
		line))))
  
  (define (getword line)
    (if (string=? line "")
	""
	(if (or (char=? (string-ref line 0) #\:)
		(char=? (string-ref line 0) #\newline))
	    "" ;(make-string 1 (string-ref line 0))
	    (string-append (substring line 0 1)
			   (getword (substring line 
					       1 (string-length line)))))))

  (define (getpretty pline)
    (if (string=? pline "")
	""
	(if (or (char=? (string-ref pline 0) #\tab)
		(char=? (string-ref pline 0) #\newline)
		(char=? (string-ref pline 0) #\:))
	    (getpretty (substring pline 1 (string-length pline)))
	    (if (and (> (string-length pline) 1)
		     (char=? (string-ref pline 0) #\\)
		     (char=? (string-ref pline 1) #\n))
		(string-append (make-string 1 #\newline)
			       (getpretty 
				(substring pline 2 (string-length pline))))
		(string-append (make-string 1 (string-ref pline 0))
			       (getpretty 
				(substring pline 1 (string-length pline))))))))

  (let ((fin (open-input-file fname))
	(line "")
	(line_i #f)
	(mname "")
	(iname "")
	(cname "")
	)
  (if (not (file-exists? fname))
      (begin
	(fl_show_message "ERROR" "" "Cannot read the menu description file.")
	FL_FALSE)
      (letrec ((loop (lambda ()
		       (set! line (elim_spaces (getline fin)))
		       (if (eof-object? line)
			   FL_TRUE
			   (if (or (string=? line "") 
				   (char=? (string-ref line 0) #\#))
			       (loop)
			       (begin
				 (set! mname (getword line))
				 (set! line 
				       (substring line 
						  (+ (string-length mname) 1)
						  (string-length line)))
				 (set! mname (getpretty mname))
				 ;
				 (set! iname (getword line))
				 (set! line 
				       (substring line 
						  (+ (string-length iname) 1)
						  (string-length line)))
				 (set! iname (getpretty iname))
				 (set! cname (getword line))
				 (set! cname (getpretty cname))
				 ; line)

				 (addtomenu mname iname cname)
				 (loop)))))))

	(loop)
	(close-input-port fin)
	FL_TRUE))))


(define (create_form_form)
  (let ((obj '()))
    (set! form (fl_bgn_form FL_NO_BOX 370 450))
    (set! obj (fl_add_box FL_UP_BOX 0 0 370 450 ""))
    (fl_set_object_color obj FL_SLATEBLUE FL_COL1)
    (set! obj (fl_add_box FL_FRAME_BOX 20 20 330 40 "Forms Demonstration"))
    (fl_set_object_color obj FL_INDIANRED FL_COL1)
    (fl_set_object_lsize obj FL_HUGE_SIZE)
    (fl_set_object_lstyle obj (+ FL_ENGRAVED_STYLE FL_BOLD_STYLE))
    (fl_set_object_lcol obj FL_INDIANRED)
    (set! obj (fl_add_box FL_FRAME_BOX 20 70 330 330 ""))
    (fl_set_object_color obj FL_INDIANRED FL_COL1)
    
    (set! obj (fl_add_button FL_NORMAL_BUTTON 190 410 110 30 "Exit"))
    (fl_set_object_callback obj doexit 0)

    (set! obj (fl_add_button FL_NORMAL_BUTTON 80 410 110 30 "Show Log"))
    (fl_set_object_callback obj showlog 0)

    (set! obj (fl_add_button FL_HIDDEN_BUTTON 20 20 330 380 "Button"))
    (fl_set_object_callback obj doback 0)

    (set! obj (fl_add_button FL_NORMAL_BUTTON 40 90 90 90 "Button"))
    (vector-set! but 0 obj)
    (fl_set_object_color obj FL_SLATEBLUE FL_COL1)
    (fl_set_object_callback obj dobut 0)
    (fl_set_object_lstyle obj FL_BOLD_STYLE)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 140 90 90 90 "Button"))
    (vector-set! but 1 obj)
    (fl_set_object_color obj FL_SLATEBLUE FL_COL1)
    (fl_set_object_callback obj dobut 1)
    (fl_set_object_lstyle obj FL_BOLD_STYLE)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 240 90 90 90 "Button"))
    (vector-set! but 2 obj)
    (fl_set_object_color obj FL_SLATEBLUE FL_COL1)
    (fl_set_object_callback obj dobut 2)
    (fl_set_object_lstyle obj FL_BOLD_STYLE)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 240 190 90 90 "Button"))
    (vector-set! but 5 obj)
    (fl_set_object_color obj FL_SLATEBLUE FL_COL1)
    (fl_set_object_callback obj dobut 5)
    (fl_set_object_lstyle obj FL_BOLD_STYLE)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 140 190 90 90 "Button"))
    (vector-set! but 4 obj)
    (fl_set_object_color obj FL_SLATEBLUE FL_COL1)
    (fl_set_object_callback obj dobut 4)
    (fl_set_object_lstyle obj FL_BOLD_STYLE)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 40 190 90 90 "Button"))
    (vector-set! but 3 obj)
    (fl_set_object_color obj FL_SLATEBLUE FL_COL1)
    (fl_set_object_callback obj dobut 3)
    (fl_set_object_lstyle obj FL_BOLD_STYLE)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 40 290 90 90 "Button"))
    (vector-set! but 6 obj)
    (fl_set_object_color obj FL_SLATEBLUE FL_COL1)
    (fl_set_object_callback obj dobut 6)
    (fl_set_object_lstyle obj FL_BOLD_STYLE)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 140 290 90 90 "Button"))
    (vector-set! but 7 obj)
    (fl_set_object_color obj FL_SLATEBLUE FL_COL1)
    (fl_set_object_callback obj dobut 7)
    (fl_set_object_lstyle obj FL_BOLD_STYLE)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 240 290 90 90 "Button"))
    (vector-set! but 8 obj)
    (fl_set_object_color obj FL_SLATEBLUE FL_COL1)
    (fl_set_object_callback obj dobut 8)
    (fl_set_object_lstyle obj FL_BOLD_STYLE)
    (fl_end_form)
    ))

(define (create_the_forms)
  (create_form_form)
  (fl_set_form_dblbuffer form 1))


;/* Go up a level in the menu tree */
(define (doback obj arg)
  (pop_menu))

;/* Exit the program */
(define (doexit obj arg)
  (fl_hide_form form)
  (fl_finish)
  (exit 0))

(define (showlog obj arg)
  (fl_show_command_log FL_TRANSIENT))



(define but (make-vector 9 '()))

;/* Maintaining and building up the menus. */
(define MAXMENU	32)

(define menu_name (make-vector MAXMENU #f))
(define menu_numb (make-vector MAXMENU #f))
(define menu_iname (make-array #f MAXMENU 9))
(define menu_icommand (make-array #f MAXMENU 9))

(define mennumb 0)

;/* Returns the number of a given menu name. */
(define (find_menu nnn)
  (letrec ((loop (lambda (i)
		   (if (>= i mennumb)
		       -1
		       (if (string-ci=? (vector-ref menu_name i) nnn)
			   i
			   (loop (+ i 1)))))))
    (loop 0)))

;/* Creates a new menu with name nnn */
(define (create_menu nnn)
  (if (= mennumb (- MAXMENU 1))
      #t
      (begin
	(vector-set! menu_name mennumb nnn)
	(vector-set! menu_numb mennumb 0)
	(set! mennumb (+ mennumb 1)))))

;/* Adds an item to a menu */
(define (addtomenu men item comm)
  (let ((n (find_menu men)))
    (if (< n 0)
	(begin
	  (create_menu men)
	  (set! n (find_menu men))))
    (if (= (vector-ref menu_numb n) 9)
	#t
	(begin
	  (array-set! menu_iname item n (vector-ref menu_numb n))
	  (array-set! menu_icommand comm n (vector-ref menu_numb n))
	  (vector-set! menu_numb n (+ (vector-ref menu_numb n) 1))))))

;/* Button to Item conversion and back. */

(define b2n '(
	      (-1 -1 -1 -1 0 -1 -1 -1 -1)
	      (-1 -1 -1 0 -1 1 -1 -1 -1)
	      (0 -1 -1 -1 1 -1 -1 -1 2)
	      (0 -1 1 -1 -1 -1 2 -1 3)
	      (0 -1 1 -1 2 -1 3 -1 4)
	      (0 -1 1 2 -1 3 4 -1 5)
	      (0 -1 1 2 3 4 5 -1 6)
	      (0 1 2 3 -1 4 5 6 7)
	      (0 1 2 3 4 5 6 7 8)
	      ))

(define n2b '(
	      (4 -1 -1 -1 -1 -1 -1 -1 -1)
	      (3 5 -1 -1 -1 -1 -1 -1 -1)
	      (0 4 8 -1 -1 -1 -1 -1 -1)
	      (0 2 6 8 -1 -1 -1 -1 -1)
	      (0 2 4 6 8 -1 -1 -1 -1)
	      (0 2 3 5 6 8 -1 -1 -1)
	      (0 2 3 4 5 6 8 -1 -1)
	      (0 1 2 3 5 6 7 8 -1)
	      (0 1 2 3 4 5 6 7 8)
	      ))

;/* Transforms a button number to an item number when there are
;   maxnumb items in total. -1 if the button should not exist. */
(define (but2numb bnumb maxnumb)
  (list-ref (list-ref b2n maxnumb) bnumb))

;/* Transforms an item number to a button number when there are
;   maxnumb items in total. -1 if the item should not exist. */
(define (numb2but inumb maxnumb)
  (list-ref (list-ref n2b maxnumb) inumb))

;/* Pushing and Popping menus */

(define stack (make-vector 64 (make-vector 32)))
(define stsize 0)

;/* Pushes a menu to be visible */
(define (push_menu nnn)
  (let ((n 0)
	(bn 0)
	(men (find_menu nnn))
	;/* make exceptions for B&W */
	(bc_more (if (= (fl_get_visual_depth) 1) FL_COL1 FL_INDIANRED))
	(bc_term (if (= (fl_get_visual_depth) 1) FL_COL1 FL_SLATEBLUE))
	)
    (if (< men 0)
	#t
	(begin
	  (set! n (vector-ref menu_numb men))
	  (fl_freeze_form form)
	  (letrec ((loop 
		    (lambda (i)
		      (if (< i 9)
			  (begin
			    (fl_hide_object (vector-ref but i))
			    (loop (+ i 1)))))))
	    (loop 0))
	  (letrec ((loop 
		    (lambda (i)
		      (if (< i n)
			  (begin
			    (set! bn (numb2but i (- n 1)))
			    (fl_show_object (vector-ref but bn))
			    (fl_set_object_label
			     (vector-ref but bn) 
			     (array-ref menu_iname men i))
			    (if (and (string? 
				      (array-ref menu_icommand men i))
				     (char=? 
				      (string-ref 
				       (array-ref menu_icommand men i) 0) #\@))
				(fl_set_object_color
				 (vector-ref but bn) bc_more FL_RED)
				(fl_set_object_color
				 (vector-ref but bn) bc_term FL_BLUE))
			    (loop (+ i 1)))))))
	    (loop 0))
	  (fl_unfreeze_form form)
	  (vector-set! stack stsize nnn)
	  (set! stsize (+ stsize 1))
	  ))))

;/* Pops a menu */
(define (pop_menu)
  (if (<= stsize 1)
      0
      (begin
	(set! stsize (- stsize 2))
	(push_menu (vector-ref stack stsize)))))

;/* The callback Routines */
;char **fl_get_cmdline_args(int *); JL undocumented !!!

;/* handles a button push */
(define (dobut obj arg)
  (let* ((men (find_menu (vector-ref stack (- stsize 1))))
	 (n (vector-ref menu_numb men))
	 (bn (but2numb arg (- n 1)))
	 )
    (if (and (string? (array-ref menu_icommand men bn))
	     (char=? 
	      (string-ref (array-ref menu_icommand men bn) 0)
	      #\@))
	(push_menu (array-ref menu_icommand men bn))
	(let ((cmd (string-append "/usr/local/sforms-0.88b/scm "
				  " -b -q -l " 
				  (array-ref menu_icommand men bn)
				  ".scm")))
	  (fl_addto_command_log "running ")
	  (fl_addto_command_log cmd)
	  (fl_addto_command_log "\n")
	  (fl_deactivate_all_forms)
	  (fl_exe_command cmd 1) 
	  (fl_activate_all_forms)
	  ))))

(demo)


