;/*
; * $Id: demo27.c,v 0.20 1994/09/22 19:46:02 zhao Exp $
; *
; * This demo shows the use of event callbacks and form callbacks.
; * purposesly mixed form callback, object callback and event callback
; * to show the flexibility of Forms Library's event handling (and
; * test if they actually work together).
; *
; *  This file is part of xforms package
; *  T.C. Zhao and M. Overmars
; */

(require `forms)
(require `struct) ; SLIB

(define main_win `())

;/*------------ Color Part --------------------*/
(define colorform `())
(define redsl `())
(define greensl `())
(define bluesl `())
(define colorobj `())

(define r 128)
(define g 128)
(define b 128)

;/* color form callback routine */
(define (color_callback obj d)
  (set! r (inexact->exact 
	   (* 255 (fl_get_slider_value redsl))))
  (set! g (inexact->exact 
	   (* 255 (fl_get_slider_value greensl))))
  (set! b (inexact->exact 
	   (* 255 (fl_get_slider_value bluesl))))
  (fl_mapcolor FL_FREE_COL1 r g b)
  (fl_redraw_object colorobj))

(define (create_colorform)
  (let ((obj `()))
    (if (null? colorform)
	(begin
	  (set! colorform (fl_bgn_form FL_NO_BOX 315 190))
	  (set! obj (fl_add_box FL_UP_BOX 0 0 315 190 ""))
	  (set! obj (fl_add_slider FL_HOR_SLIDER 20 25 220 35 ""))
	  (set! bluesl obj)
	  (fl_set_object_color obj FL_COL1 FL_BLUE)
	  (set! obj (fl_add_slider FL_HOR_SLIDER 20 135 220 35 ""))
	  (set! redsl obj)
	  (fl_set_object_color obj FL_COL1 FL_RED)
	  (set! obj (fl_add_slider FL_HOR_SLIDER 20 80 220 35 ""))
	  (set! greensl obj)
	  (fl_set_object_color obj FL_COL1 FL_GREEN)
	  (set! obj (fl_add_box FL_BORDER_BOX 250 25 50 145 ""))
	  (set! colorobj obj)
	  (fl_set_object_color obj FL_FREE_COL1 FL_FREE_COL1)
	  (fl_end_form)))))

;/* initializes the color part */
(define (init_colorpart)
  (create_colorform)
  (fl_set_form_callback colorform color_callback  0)
  (fl_set_form_position colorform  20  (- -300 190))
  (fl_show_form colorform FL_PLACE_GEOMETRY FL_TRANSIENT "Color")
  (fl_mapcolor FL_FREE_COL1 r g b)
  (fl_redraw_object colorobj))

;/*--------------- Control Part -------------------*/
(define controlform `())
(define squareobj `())
(define exitobj `())
(define sizeobj `())

(define curobj 1)
(define cursize 20)

(define (select_object ob  which)
  (set! curobj which))

;/* control form callback routine */
(define (control_callback obj  d)
  (if (equal? obj sizeobj) 
      (set! cursize (* 40 (fl_get_slider_value sizeobj)))
      (if (equal? obj exitobj) 
	  (exit 0)
	  )))

(define (create_controlform)
  (let ((obj `()))
    (set! controlform (fl_bgn_form FL_UP_BOX 260 230))
    (fl_bgn_group)
    (set! obj (fl_add_button FL_RADIO_BUTTON 20 150 60 60 "@square"))
    (set! squareobj obj)
    (fl_set_object_lcol obj FL_YELLOW)
    (fl_set_object_callback obj  select_object  1)
    (set! obj (fl_add_button FL_RADIO_BUTTON 20 90 60 60 "@circle"))
    (fl_set_object_lcol obj FL_YELLOW)
    (fl_set_object_callback obj  select_object  2)
    (set! obj (fl_add_button FL_RADIO_BUTTON 20 30 60 60 "@8>"))
    (fl_set_object_lcol obj FL_YELLOW)
    (fl_set_object_callback obj  select_object  3)
    (fl_end_group)
    (set! exitobj (fl_add_button FL_NORMAL_BUTTON 160 30 80 30 "Exit"))
    (set! obj (fl_add_button FL_NORMAL_BUTTON 160 180 80 30 "Clear"))
    (fl_set_object_callback obj clearit 0)
    (set! sizeobj (fl_add_slider FL_VERT_SLIDER 100 30 40 180 "size"))
    (fl_end_form)))

;/* initializes the control part */
(define (init_controlpart)
  (create_controlform)
  (fl_set_form_callback controlform control_callback  0)
  (fl_set_button squareobj 1)
  (fl_set_initial_placement controlform  20  (- -230 40)  
                            260 230)
  (fl_show_form controlform FL_PLACE_SIZE FL_TRANSIENT "Control"))

;/*--------------- Main part -------------------*/

(define-record s_obj
  (type
   r
   g
   b
   x
   y
   size))

(define ob `())
(define onumb 0)

(define (drawit obj)
  (fl_winset main_win)
  (fl_mapcolor FL_FREE_COL1 
	       (s_obj->r obj)
	       (s_obj->g obj)
	       (s_obj->b obj))
  (let ((obj_type (s_obj->type obj))
	(obj_x    (inexact->exact (s_obj->x obj)))
	(obj_y    (inexact->exact (s_obj->y obj)))
	(obj_size (inexact->exact (s_obj->size obj))))
    (cond ((= obj_type 1)
	   (fl_rectf (- obj_x obj_size) (- obj_y obj_size)
		     (* obj_size 2) (* obj_size 2)  FL_FREE_COL1))
	  ((= obj_type 2)
	   (fl_circf obj_x  obj_y  obj_size  FL_FREE_COL1))
	  ((= obj_type 3)
	   (let* ((p0x (- obj_x obj_size))
		  (p0y (+ obj_y obj_size))
		  (p1x (+ obj_x obj_size))
		  (p1y (+ obj_y obj_size))
		  (p2x (+ obj_x 0))
		  (p2y (- obj_y obj_size))
		  (point (list (list p0x p0y)
			       (list p1x p1y)
			       (list p2x p2y))))
	     (fl_polyf point  3  FL_FREE_COL1)))
	  (else 
	   #t))  
    #t))

;/* draws a particular object */
(define (drawobject)
  (let* ((wm (fl_get_win_mouse main_win) )
	 (x0 (cadr wm))
	 (y0 (caddr wm))
	 (km (cadddr wm))
	 (cur_obj (make-s_obj 0 0 0 0 0 0 0)))
    ;
    (set! ob (append ob (list cur_obj)))
    (set-s_obj-x! cur_obj x0)
    (set-s_obj-y! cur_obj y0)
    (set-s_obj-r! cur_obj r)
    (set-s_obj-g! cur_obj g)
    (set-s_obj-b! cur_obj b)
    (set-s_obj-type! cur_obj curobj)
    (set-s_obj-size! cur_obj cursize)
    (drawit cur_obj)
    (set! onumb (+ onumb 1))))

(define (redrawit)
  (fl_clear_window (fl_display) main_win) ; XClearWindow 
  (map (lambda (o)
	 (drawit o))
       ob)
  #t)

(define (clearit o data)
  (set! ob '())
  (set! onumb 0)
  (redrawit))

;/* event callback routine */
(define Expose 12)
(define ButtonPress 4)

(define (main_callback xev p)
  ; xev as event type (integer)
  (fl_winset main_win)
  (let ((xtype (fl_get_xev_member xev xev_type)))
    (if (= xtype Expose)
	(redrawit)
	(if 
	 ;(and 
	 (= xtype ButtonPress)
	 ;(not (= (fl_get_xev_member xev xev_button) Button2)))
	 (drawobject))) 
    #t))

(define (init_mainpart)
  (fl_pref_wingeometry 400 300 400 400)
  (fl_pref_winsize 400 400)
  (set! main_win  (fl_winopen "Drawing")) ; JL : changed order 
  (fl_winbackground main_win (fl_get_flcolor FL_COL1)) 
  (fl_set_event_callback main_callback  0)
  )

;/*---------------------------------------*/

(define (demo27 . args)
  (fl_initialize args)
  (init_colorpart)
  (init_controlpart)
  (init_mainpart)
  (color_callback colorobj  0)
  (letrec ((loop (lambda ()
		   (fl_do_forms)
		   (loop))))
    (loop)
    0))

(demo27)




