;/*
; * OpenGL/mesa canvas demo
; *
; * On most platforms, the mesa/OpenGL support is not compiled into
; * the shared library (It it in the static library). You can compile 
; * ../FORMS/gl.c and put it in the shared library if you have mesa/OpenGL
; *
; *   Also you might want to make a link libGL.a to libMesaGL.a
; *
; *  T.C. Zhao and M. Overmars
; *
; */

(require 'gl) ; work in progress
(require 'forms)
(require 'struct)

(define Expose 12)
(define ButtonPress 4)

(define None 0)

(define-record ui
  (form
   vdata
   cdata
   ldata
   canvas
   done
   menu
   butgrp))

(define fdui 
  (make-ui #f 0 0 0 #f #f #f #f))

(define prim GL_POLYGON)

(define v3f glVertex3f)

(define (draw_cube)
  (glColor3f 1.0 0.0 0.0)
  (glBegin prim)
  (v3f 1.0  1.0  1.0 )
  (v3f 1.0  -1.0  1.0 )
  (v3f 1.0  -1.0  -1.0)
  (v3f 1.0  1.0  -1.0 )
  (glEnd)
  
  (glBegin prim)
  (v3f -1.0  1.0  1.0 )
  (v3f -1.0  1.0  -1.0 )
  (v3f -1.0  -1.0  -1.0 )
  (v3f -1.0  -1.0  1.0 )
  (glEnd)

  (glColor3f 0.0  1.0  0.0 )
  (glBegin prim)
  (v3f  1.0  1.0   1.0 )
  (v3f  1.0  1.0  -1.0 )
  (v3f -1.0  1.0  -1.0 )
  (v3f -1.0  1.0   1.0 )
  (glEnd)
  
  (glBegin prim)
  (v3f  1.0  -1.0   1.0 )
  (v3f -1.0  -1.0   1.0 )
  (v3f -1.0  -1.0  -1.0 )
  (v3f  1.0  -1.0  -1.0 )
  (glEnd)
  
  (glColor3f 0.0  0.0  1.0 )
  (glBegin prim)
  (v3f  1.0   1.0   1.0 )
  (v3f -1.0   1.0   1.0 )
  (v3f -1.0  -1.0   1.0 )
  (v3f  1.0  -1.0   1.0 )
  (glEnd)
  
  (glBegin prim)
  (v3f  1.0  1.0  -1.0 )
  (v3f  1.0 -1.0  -1.0 )
  (v3f -1.0 -1.0  -1.0 )
  (v3f -1.0  1.0  -1.0 )
  (glEnd)
  )

(define xrot 0)
(define yrot 0)
(define zrot 0)
(define (idle_cb ev data)
  (if (or (equal? (fl_get_form_visible (ui->form fdui)) FL_FALSE)
	  (equal? (fl_get_object_visible (ui->canvas fdui)) FL_FALSE))
      0
      (begin
	(fl_activate_glcanvas (ui->canvas fdui))
	(glClear (logior GL_COLOR_BUFFER_BIT GL_DEPTH_BUFFER_BIT))

	(glPushMatrix)
	
	(glRotatef xrot  1.0  0.0  0.0 )
	(glRotatef yrot  0.0  1.0  0.0 )
	(glRotatef zrot  0.0  0.0  1.0 )
	(glTranslatef -1.0 1.2 -0.5)
	
	(draw_cube)
	
	(glPopMatrix)
	
	(glFinish)
	
	(if (not (equal? ev 0))
	    (begin
	      (set! xrot (+ xrot 10.0))
	      (set! yrot (+ yrot  7.0))
	      (set! zrot (- zrot  3.0)))
	    )
	(glXSwapBuffers (fl_display)  (fl_get_canvas_id (ui->canvas fdui)))
	0)))

(define (canvas_expose ob win w h xev ud)
  (glViewport 0 0 w h)
  (glClearColor 0.0 0.0 0.0 0.0)

  (glMatrixMode GL_PROJECTION )
  (glLoadIdentity)
  (glFrustum -1.0  1.0   -1.0  1.0   1.0  10.0 ) 
  (glTranslatef 0.0  0.0  -5.0 )

  (glMatrixMode GL_MODELVIEW ) 
  (glLoadIdentity)
  (glCullFace GL_BACK )
  (glEnable GL_CULL_FACE )
  
  (glShadeModel GL_FLAT )
  
  ;/* refresh */
  (idle_cb 0  0)
  0)

(define suspended #f)
(define (buttonpress_cb ob win w h xev ud)
 (set! suspended (not suspended))
 (if (not suspended)
     (fl_set_idle_callback idle_cb) 0)
 0)

(define primitive (list GL_POLYGON GL_LINE_LOOP))
(define iprimitive 0)
(define (switch_primitive ob data)
  (set! iprimitive (modulo (+ iprimitive 1) 2))
  (set! prim (list-ref primitive iprimitive)))

;/* switch single/dblbuffer */
(define sbuf '(GLX_RGBA GLX_DEPTH_SIZE 1 
			GLX_RED_SIZE 1 GLX_GREEN_SIZE 1 GLX_BLUE_SIZE 1 None))

(define dbuf '(GLX_RGBA GLX_DEPTH_SIZE 1 
			GLX_RED_SIZE 1 GLX_GREEN_SIZE 1 GLX_BLUE_SIZE 1 
			;/*GLX_DOUBLEBUFFER */ 
			None))

(define is_double FL_TRUE)
(define (buffer_cb ob data)
  ;FD_form *fdui = ob->form->fdui;
  (set! is_double (if (= is_double FL_TRUE) FL_FALSE FL_TRUE))
  (fl_set_object_label ob (if (= is_double FL_TRUE) "Single" "Double"))
  (fl_set_glcanvas_attributes (ui->canvas FDUI)
			      (if (= is_double FL_TRUE) dbuf sbuf)))


(define (menu_cb ob data)
  #f)

(define (gl . args)
  (let ((form '())
	(obj '()))
    (fl_initialize args)
    (set! fdui (create_form_form))

    ;/* fill-in form initialization code */
    (fl_set_object_gravity (ui->butgrp fdui)
			   FL_NorthEast FL_NorthEast)
    (fl_addto_menu (ui->menu fdui) "Item1|Item2|Item3|Item4")

    (fl_add_canvas_handler (ui->canvas fdui) Expose  canvas_expose  0)
    (fl_add_canvas_handler (ui->canvas fdui) ButtonPress  buttonpress_cb  0)
    (fl_set_idle_callback idle_cb  0)

    ;/* geometry stuff */
    (fl_set_form_minsize (ui->form fdui) 340  280)

    (fl_show_form (ui->form fdui) (logior FL_PLACE_CENTER FL_FREE_SIZE)
		  FL_FULLBORDER 
		  "OpenGL Canvas")

    (letrec ((loop (lambda ()
		     (if (not (equal? (ui->done fdui) (fl_do_forms)))
			 (loop)))))
    (loop))

    (fl_finish)
    0))

(define (hide_it ob data)
  (if (= (fl_get_object_visible (ui->canvas fdui)) FL_TRUE)
      (begin
	(fl_hide_object (ui->canvas fdui))
	(fl_set_object_label ob "Show"))
      (begin
	(fl_show_object (ui->canvas fdui))
	(fl_set_object_label ob "Hide"))))

(define (reshow ob data)
  (fl_hide_form (fl_get_object_form ob))
  (fl_show_form (ui->form fdui) (logior FL_PLACE_CENTER FL_FREE_SIZE)
		FL_FULLBORDER 
		"OpenGL Canvas"))

(define (create_form_form)
  (let ((form '())
	(obj '()))
    (set! form (fl_bgn_form FL_NO_BOX  340  280))
    (set! obj (fl_add_box FL_UP_BOX 0 0 340 280 ""))
    (set! obj (fl_add_glcanvas FL_NORMAL_CANVAS 20 45 225 215 ""))
    (set-ui-canvas! fdui obj)
    (fl_set_object_gravity obj  FL_NorthWest  FL_SouthEast)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 255 230 70 30 "Done"))
    (set-ui-done! fdui obj)
    (fl_set_button_shortcut obj "Dd#d" 1)
    (fl_set_object_gravity obj  FL_SouthEast  FL_SouthEast)
    (set! obj (fl_add_menu FL_PULLDOWN_MENU 20 15 54 20 "Canvas"))
    (set-ui-menu! fdui obj)
    (fl_set_object_shortcut obj "#CcC" 1)
    (fl_set_object_boxtype obj FL_FLAT_BOX)
    (fl_set_object_lstyle obj FL_BOLD_STYLE)
    (fl_set_object_gravity obj  FL_NorthWest  FL_NorthWest)
    (fl_set_object_callback obj menu_cb 0)

    (set! obj (fl_bgn_group))
    (set-ui-butgrp! fdui obj)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 255 45 70 30 "Poly/Line"))
    (fl_set_object_gravity obj  FL_NorthEast  FL_NorthEast)
    (fl_set_object_callback obj switch_primitive 0)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 255 75 70 30 "Hide"))
    (fl_set_button_shortcut obj "#HhH" 1)
    (fl_set_object_callback obj hide_it 0)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 255 105 70 30 "ReShow"))
    (fl_set_button_shortcut obj "#RRr" 1)
    (fl_set_object_callback obj reshow 0)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 255 135 70 30 "Single"))
    (fl_set_object_callback obj buffer_cb 0)
    (fl_end_group)
    (fl_end_form)
    
    (set-ui-form! fdui form)
    fdui))
  

(gl)