;/*
; * OpenGL rendering in its own window
; *
; *  T.C. Zhao and M. Overmars
; *
; */

(require 'glx)
(require 'gl)
(require 'forms)
(require 'struct)

(define none 0)
;/**** Forms and Objects ****/
(define-record fdui
  (form
   vdata
   cdata
   ldata
   polybutt))

(define glcontrol (make-fdui #f #f #f #f #f))

(define prim GL_POLYGON) ;/* GL primitive to draw */
(define glwin #f)

(define (glwin . args)
  ;FD_glcontrol *fd_glcontrol;
  ; GLXContext context;
   (let ((config (list GLX_RGBA GLX_DEPTH_SIZE 16 GLX_DOUBLEBUFFER 0)))
     (fl_initialize args)
     (set! glcontrol (create_form_glcontrol))
     
     (if (not (equal? glwin #f))
	 (set! glwin (fl_glwincreate config 250 250))
	 (begin
	   (display "GLWin: Can't create OpenGL window\n")
	   (exit 0)))
     
     (fl_add_event_callback glwin Expose handle_expose 0)
     (fl_winshow glwin)

     ;/* show the first form */
     (fl_set_button (fd_glcontrol->polybutt glcontrol) 1)
     (fl_show_form (fd_glcontrol->glcontrol glcontrol) FL_PLACE_CENTER
		   FL_FULLBORDER "glcontrol")

     (fl_do_forms)
     0))

(define (draw_cube)
  (glColor3f 1.0, 0.0, 0.0 )
  (glBegin prim )
  (glVertex3f 1.0, 1.0, 1.0 )
  (glVertex3f 1.0, -1.0, 1.0 )
  (glVertex3f 1.0, -1.0, -1.0 )
  (glVertex3f 1.0, 1.0, -1.0 )
  (glEnd)
  
  (glBegin prim )
  (glVertex3f -1.0, 1.0, 1.0 )
  (glVertex3f -1.0, 1.0, -1.0 )
  (glVertex3f -1.0, -1.0, -1.0 )
  (glVertex3f -1.0, -1.0, 1.0 )
  (glEnd)
  
  ;/* Y faces */
  (glColor3f 0.0, 1.0, 0.0 )
  (glBegin prim )
  (glVertex3f  1.0, 1.0,  1.0 )
  (glVertex3f  1.0, 1.0, -1.0 )
  (glVertex3f -1.0, 1.0, -1.0 )
  (glVertex3f -1.0, 1.0,  1.0 )
  (glEnd)
  
  (glBegin prim )
  (glVertex3f  1.0, -1.0,  1.0 )
  (glVertex3f -1.0, -1.0,  1.0 )
  (glVertex3f -1.0, -1.0, -1.0 )
  (glVertex3f  1.0, -1.0, -1.0 )
  (glEnd)
  
  ;/* Z faces */
  (glColor3f 0.0, 0.0, 1.0 )
  (glBegin prim )
  (glVertex3f  1.0,  1.0,  1.0 )
  (glVertex3f -1.0,  1.0,  1.0 )
  (glVertex3f -1.0, -1.0,  1.0 )
  (glVertex3f  1.0, -1.0,  1.0 )
  (glEnd)
  
  (glBegin prim )
  (glVertex3f  1.0, 1.0, -1.0 )
  (glVertex3f  1.0,-1.0, -1.0 )
  (glVertex3f -1.0,-1.0, -1.0 )
  (glVertex3f -1.0, 1.0, -1.0 )
  (glEnd)
  )

(define xrot 0)
(define yrot 0)
(define zrot 0)
(define (animate ev data)
  (glClear (logior GL_COLOR_BUFFER_BIT GL_DEPTH_BUFFER_BIT))
  (glPushMatrix)
  (glRotatef xrot  1.0  0.0  0.0 )
  (glRotatef yrot  0.0  1.0  0.0 )
  (glRotatef zrot  0.0  0.0  1.0 )
  (glTranslatef -1.0 1.2 -0.5) 
  (draw_cube)
  (glPopMatrix)
  (glFinish)

  (if(ev->type != Expose)
    (begin
      (set! xrot (+ xrot 10.0))
      (set! yrot (+ yrot  7.0))
      (set! zrot (- zrot  3.0))))
  
  (glXSwapBuffers fl_display  glwin)
  0)

;/* Initialization and expose handler */
(define (handle_expose xev data)
  (let* ((wsiz (fl_get_winsize glwin))
	 (w (car wsiz))
	 (h (cadr wsiz)))
    (glViewport 0 0  w  h) 

    (glClearColor 0.0 0.0 0.0 0.0)

    (glMatrixMode GL_PROJECTION )
    (glLoadIdentity)
    (glFrustum -1.0  1.0   -1.0  1.0   1.0  10.0 ) 
    (glTranslatef 0.0  0.0  -5.0 )  

    (glMatrixMode GL_MODELVIEW ) 
    (glLoadIdentity)
    (glCullFace GL_BACK )
    (glEnable GL_CULL_FACE ) 
    (glShadeModel GL_FLAT )
    (animate xev  0)
    0))

;/* Form definition generated by fdesign */

;/* callback functions */
(define (change_primitive ob data)
  (set! prim data))

(define (animate_callback ob data)
  (fl_set_idle_callback 
   (if (= (fl_get_button ob) FL_TRUE)
       animate 0)  0))

(define (create_form_glcontrol)
  (let ((form '())
	(obj '()))
    (set-fdui-form! glcontrol (fl_bgn_form FL_NO_BOX  241  121))
    (set! obj (fl_add_box FL_UP_BOX 0 0 241 121 ""))
    (set! obj (fl_add_frame FL_ENGRAVED_FRAME 20 26 90 70 ""))
    (set! obj (fl_add_checkbutton FL_RADIO_BUTTON 30 61 70 30 "Polygon"))
    (set-fdui-polybutt! glcontrol obj)
    (fl_set_object_callback obj change_primitive GL_POLYGON)
    (set! obj (fl_add_checkbutton FL_RADIO_BUTTON 30 31 70 30 "Line"))
    (fl_set_object_callback obj change_primitive GL_LINE_LOOP)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 150 21 80 30 "Done"))
    (fl_set_button_shortcut obj "Dd" 1)
    (set! obj (fl_add_roundbutton FL_PUSH_BUTTON 150 71 80 30 "Animate"))
    (fl_set_object_callback obj animate_callback 0)
    (fl_end_form)
    glcontrol))

;/*---------------------------------------*/
(glwin)
