;/* 
; * Demo showing the use of preemptive and post-object handler,
; * and one possible way of implementing a "tool tip"
; *
; *  This file is part of xforms package
; *  T.C. Zhao and M. Overmars (1997)
; *
; */
(require 'forms)

(define fd_form0 '())
(define fd_butt '())
(define fd_enter '())
(define fd_leave '())
(define fd_push '())
(define fd_release '())
(define fd_peek '())
(define fd_override '())
(define fd_event '())
(define fd_done '())

;/* which event to take over is better kept in a state varible even though
;   query the status via fl_get_button is cheap
; */
(define (preemptive_handler ob event mx my key xev)
  (let* ((override (fl_get_button fd_override))
	 (buf "")
	 (what (if (= override FL_TRUE) "preempted" "detected")))
    (cond ((= event FL_ENTER)
	   (if (= (fl_get_button fd_enter) FL_TRUE) 
	       (begin
		 ;/* ok, want to handle enter */
		 (fl_set_object_label fd_event
				      (string-append "FL_ENTER" " " what))
		 (if (= override FL_TRUE) FL_PREEMPT 0))))
	  ((= event FL_LEAVE)
	   (if (= (fl_get_button fd_leave) FL_TRUE)
	       (begin
		 (fl_set_object_label fd_event
				      (string-append "FL_LEAVE" " " what))
		 (if (= override FL_TRUE) FL_PREEMPT 0))))
	  ((or (= event FL_PUSH)
	       (= event FL_MOUSE)) ;/* one of the quirks of the button class */
	   (if (= (fl_get_button fd_push) FL_TRUE)
	       (begin
		 (fl_set_object_label fd_event
				      (string-append "FL_PUSH" " " what))
		 (if (= override FL_TRUE) FL_PREEMPT 0))))
	  ((= event FL_RELEASE)
	   (if (= (fl_get_button fd_release) FL_TRUE)
	       (begin
		 (fl_set_object_label fd_event
				      (string-append "FL_RELEASE" " " what))
		 (if (= override FL_TRUE) FL_PREEMPT 0))))
	  (else
	   0))))

(define INTERVAL  800) ;/* wait this long before show tip */
(define timeoutID  -1) ;/* we can also use ob->u_ldata to hold it */

(define (do_tips id p)
  (let* ((ob p)
	 (ob_vdata (fl_get_object_vdata ob))
	 (obpos (fl_get_object_position ob))
	 (ob_x (car obpos))
	 (ob_y (cadr obpos))
	 (obsiz (fl_get_object_size ob))
	 (ob_h (cadr obsiz))
	 (ob_form (fl_get_object_form ob))
	 (ob_formpos (fl_get_form_position ob_form))
	 (ob_fx (car ob_formpos))
	 (ob_fy (cadr ob_formpos)))
    (fl_show_oneliner ob_vdata (+ ob_fx ob_x)
		      (+ ob_fy ob_y ob_h 1))
    (set! timeoutID (fl_add_timeout INTERVAL do_tips ob))))

;/*
; * use the post handler as a tipper
; */
(define (post_handler ob event mx my key xev)
  (if (equal? (fl_get_object_vdata ob) 0)
      0
      (if (= event FL_ENTER) 
	  (set! timeoutID (fl_add_timeout INTERVAL do_tips ob))
	  (if (or (= event FL_LEAVE)
		  (= event FL_PUSH))
	      (begin
		(fl_hide_oneliner)
		(if (not (= timeoutID FL_FALSE))
		    (begin
		      (fl_remove_timeout timeoutID)
		      (set! timeoutID 0)))))))
  0)

(define (set_tip ob s)
  (fl_set_object_vdata ob s)
  (fl_set_object_posthandler ob post_handler))

(define (preemptive . args)
  (fl_initialize args)

  (set! fd_form0 (create_form_form0))

  ;/* fill-in form initialization code */

  (fl_set_button fd_peek  1)
  (fl_set_button fd_enter  1)
  (fl_set_button fd_leave  1)
  (fl_set_button fd_push  1)
  (fl_set_button fd_release  1)

  (fl_set_object_prehandler fd_butt preemptive_handler)

  (set_tip fd_done  "Want to quit ?\nPress me")
  (set_tip fd_peek  "Turn preempting off")
  (set_tip fd_override  "Turn preempting on")

  ;/* show the first form */
  (fl_show_form fd_form0 FL_PLACE_CENTER FL_TRANSIENT "Preemptive")
  (letrec ((loop (lambda ()
		   (if (not (equal? fd_done (fl_do_forms)))
		       (loop)))))
    (loop))
  (fl_finish)
  (exit 0))

;/* Form definition file generated with fdesign. */

(define (create_form_form0)
  (let ((obj '())
	(form '()))
    (set! form (fl_bgn_form FL_NO_BOX  320  250))
    (set! obj (fl_add_box FL_UP_BOX 0 0 320 250 ""))
    (set! obj (fl_add_frame FL_ENGRAVED_FRAME 200 70 95 100 ""))
    (set! obj (fl_add_button FL_NORMAL_BUTTON 20 70 170 100 "A Button"))
    (set! fd_butt obj)
    (set! obj (fl_add_checkbutton FL_PUSH_BUTTON 210 70 45 30 "Enter"))
    (set! fd_enter obj)
    (set! obj (fl_add_checkbutton FL_PUSH_BUTTON 210 95 40 30 "Leave"))
    (set! fd_leave obj)
    (set! obj (fl_add_checkbutton FL_PUSH_BUTTON 210 120 50 30 "Push"))
    (set! fd_push obj)
    (set! obj (fl_add_checkbutton FL_PUSH_BUTTON 210 140 60 30 "Release"))
    (set! fd_release obj)
    (set! obj (fl_add_text FL_NORMAL_TEXT 55 15 220 30 "Pre-emptive Handler"))
    (fl_set_object_lsize obj FL_MEDIUM_SIZE)
    (fl_set_object_lalign obj FL_ALIGN_CENTER)
    (fl_set_object_lstyle obj FL_BOLD_STYLE)
    (set! obj (fl_add_checkbutton FL_RADIO_BUTTON 190 40 35 30 "Peek"))
    (set! fd_peek obj)
    (fl_set_object_color obj FL_COL1 FL_BLUE)
    (set! obj (fl_add_checkbutton FL_RADIO_BUTTON 240 40 35 30 "Override"))
    (set! fd_override obj)
    (fl_set_object_color obj FL_COL1 FL_BLUE)
    (set! obj (fl_add_box FL_FLAT_BOX 40 180 245 25 ""))
    (set! fd_event obj)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 170 210 100 30 "Done"))
    (set! fd_done obj)
    (fl_end_form)
    form))

(preemptive)
