;;; colbox example
;;; from forms documentation p. 215 ff.

(define FL_COLBOX (+ FL_USER_CLASS_START 1))
; 1001 <= Class number <= 9999

(define FL_NORMAL_COLBOX 0) ; the only type

(define (fl_create_colbox type x y w h label)
  ; create a generic object class with an appropriate ID
  (let ((ob '()))
    (set! ob (fl_make_object FL_COLBOX type x y w h label handle_colbox))
    ; initialize some components
    (fl_set_object_ldata ob 0) 
    ; JL : due to missing counter component of button SPEC
    (fl_set_object_color ob FL_RED FL_BLUE)
    (fl_set_object_boxtype ob FL_UP_BOX) ; to show something
    ob))

(define (fl_add_colbox type x y w h label)
  (let ((ob (fl_create_colbox type x y w h label)))
    (fl_add_object (fl_current_form) ob)
    ob))

(define (handle_colbox ob event mx my key xev)
  (let* ((opos (fl_get_object_position ob))
         (ob_x (car opos))
         (ob_y (cadr opos))
         (osiz (fl_get_object_size ob)) ; JL : get...geometry draws first 
         (ob_w (car osiz))
         (ob_h (cadr osiz))
         (ocol (fl_get_object_color ob))
         (ob_col1 (car ocol))
         (ob_col2 (cadr ocol))
	 (ob_bw (fl_get_object_bw ob))
         (ob_boxtype (fl_get_object_boxtype ob))
         )
    (if (= event FL_DRAW)
	; draw box
	(if (= (fl_get_object_pushed ob) FL_TRUE)
	    (begin
	      (fl_drw_box ob_boxtype ob_x ob_y ob_w ob_h ob_col2 ob_bw)
	      (display (string-append 
			"Colbox pushed "
			(number->string (fl_get_colbox ob))
			" times\n")))
	    (fl_drw_box ob_boxtype ob_x ob_y ob_w ob_h ob_col1 ob_bw)))
    ; fall through
    (cond ((= event FL_DRAWLABEL)
	   ; draw label
	   (fl_draw_object_label ob))
	  ((= event FL_PUSH)
	   (fl_set_object_ldata ob (+ (fl_get_object_ldata ob) 1))
	   (fl_redraw_object ob))
	  ((= event FL_RELEASE)
	   (fl_redraw_object ob))
	  (else
	   #t)); (fl_redraw_object ob)))
    0))

(define (fl_get_colbox ob)
  (if (or (equal? ob FL_FALSE)
	  (not (= (fl_get_object_class ob) FL_COLBOX)))
      (begin
	(display "get_colbox : Bad argument or wrong type\n")
	0)
      (fl_get_object_ldata ob)))

;;; demo 
(define cbform_bexit '())

(define (colbox . args)
  (let ((cbform '()))
    (fl_initialize args)
    (set! cbform (create_form_colbox))
    (fl_show_form cbform FL_PLACE_CENTER FL_TRANSIENT "colbox")
    (letrec ((loop (lambda ()
                     (if (not (equal? (fl_do_forms) cbform_bexit))
                         (loop)))))
      (loop))
    (fl_finish)
    0))

(define (create_form_colbox)
  (let ((obj '())
        (form '())
        (oldbw (fl_get_border_width)))
    (fl_set_border_width -2)
    (set! form (fl_bgn_form FL_NO_BOX  310  190))
    (set! obj (fl_add_box FL_UP_BOX 0 0 310 190 ""))
    (set! obj (fl_add_labelframe FL_ENGRAVED_FRAME 40 45 100 120 "ColboxA"))
    (fl_set_object_boxtype obj FL_FLAT_BOX)
    (fl_set_object_lstyle obj FL_BOLD_STYLE)
    (set! obj (fl_add_colbox FL_NORMAL_COLBOX 50 55 80 30 "colbox1"))
    (fl_set_object_color obj FL_COL1 FL_BLUE)
    (set! obj (fl_add_colbox FL_NORMAL_COLBOX 50 90 80 30 "colbox2"))
    (fl_set_object_color obj FL_COL1 FL_GREEN)
    (set! obj (fl_add_colbox FL_NORMAL_COLBOX 50 125 80 30 "colbox3"))
    (fl_set_object_color obj FL_COL1 FL_RED)
    (set! obj (fl_add_labelframe FL_ENGRAVED_FRAME 180 45 100 120 "ColboxB"))
    (fl_set_object_boxtype obj FL_FLAT_BOX)
    (fl_set_object_lstyle obj FL_BOLD_STYLE)
    (set! obj (fl_add_colbox FL_NORMAL_COLBOX 190 55 80 30 "Box1"))
    (fl_set_object_color obj FL_BLUE FL_COL1)
    (set! obj (fl_add_colbox FL_NORMAL_COLBOX 190 90 80 30 "Box2"))
    (fl_set_object_color obj FL_RED FL_COL1)
    (set! obj (fl_add_colbox FL_NORMAL_COLBOX 190 125 80 30 "Box3"))
    (fl_set_object_color obj FL_GREEN FL_COL1)
    (set! obj (fl_add_button FL_NORMAL_BUTTON 125 10 65 25 "Exit"))
    (set! cbform_bexit obj)
    (fl_end_form)
    (fl_set_border_width oldbw)
    form))

(colbox)
