; Starfield demo
;   inspired by OpenGL (TM) demo
; by Johannes Leveling, Mar 20 1999

(require 'forms)
(require 'random)

(define PI 3.141592657)

(define RAND_MAX 16384)
(define (rand)
  (random RAND_MAX))


(define NORMAL 0)
(define WEIRD  1)

(define STREAK 0)
(define CIRCLE 1)

(define MAXSTARS 200)
(define MAXPOS 10000)
(define MAXWARP 10)
(define MAXANGLES 6000)

(define srec-type      0)
(define srec-x0        1)
(define srec-x1        2)
(define srec-y0        3)
(define srec-y1        4)
(define srec-z0        5)
(define srec-z1        6)
(define srec-offsetX   7)
(define srec-offsetY   8)
(define srec-offsetR   9)
(define srec-rotation 10)

(define flag NORMAL)
(define starCount (/ MAXSTARS 2))
(define speed 1.0)
(define nitro 0)

(define starv (make-vector MAXSTARS #f))
(define fsin $sin)
(define fcos $cos)


(define (NewStar n d)
  (vector-set! starv n 
	       (if (and (= (random 4) 0) 
			(= flag WEIRD))
		   (vector 
		    ; type 
		    (if (= (random 4) 0) 
			CIRCLE
			STREAK)
		    ; x0/1 y0/1 z0/1
		    (- (remainder (rand) MAXPOS) (/ MAXPOS 2))
		    0
		    (- (remainder (rand) MAXPOS) (/ MAXPOS 2))
		    0
		    (+ (remainder (rand) MAXPOS) d)
		    0
		    ; offsetX/Y/R
		    (- (remainder (rand) 100) 50)
		    (- (remainder (rand) 100) 50)
		    (- (remainder (rand) 25)  12.5)
		    0.0)
		   (vector 
		    ; type 
		    (if (= (random 4) 0) 
			CIRCLE
			STREAK)
		    ; x0/1 y0/1 z0/1
		    (- (remainder (rand) MAXPOS) (/ MAXPOS 2))
		    0
		    (- (remainder (rand) MAXPOS) (/ MAXPOS 2))
		    0
		    (+ (remainder (rand) MAXPOS) d)
		    0
		    ; offsetX/Y/R
		    0 0 0
		    0.0))))

(define (RotatePoint x y rotation)
  (list (- (* x (fCos rotation))
	   (* y (fSin rotation)))
	(+ (* y (fCos rotation))
	   (* x (fSin rotation)))))

(define offset (* speed 60.0))
(define (MoveStars n)
  (if (< n starCount)
      (begin
	(let* ((starn (vector-ref starv n))
	       (x0 (vector-ref starn srec-x0))
	       (y0 (vector-ref starn srec-y0))
	       (z0 (vector-ref starn srec-z0))
	       (nrot (+ (vector-ref 
			 starn srec-rotation) 
			(vector-ref 
			 starn srec-offsetR)))
	       )
	  (vector-set! starn srec-x1 x0)
	  (vector-set! starn srec-y1 y0)
	  (vector-set! starn srec-z1 z0)
	  (vector-set! starn srec-x0 
		       (+ x0 (vector-ref 
			      starn srec-offsetX)))
	  (vector-set! starn srec-y0 
		       (+ y0 (vector-ref 
			      starn srec-offsetY)))
	  (vector-set! starn srec-z0 (- z0 offset))
	  (vector-set! starn srec-rotation
		       (if (> nrot MAXANGLES)
			   0.0 nrot))
	  )
	(MoveStars (+ n 1)))))

(define (StarPoint n)
  (let* ((nstar (vector-ref starv n))
         (xn (/ (* (vector-ref nstar srec-x0) windW) 
                (vector-ref nstar srec-z0)))
         (yn (/ (* (vector-ref nstar srec-y0) windH) 
                (vector-ref nstar srec-z0)))
         (x1 0)
         (y1 0)
         (width 0)
         (i 0)
         (rot (vector-ref nstar srec-rotation))
         (rp (RotatePoint xn yn rot))
	 (x0 (+ xn (/ windW 2.0)))
	 (y0 (+ yn (/ windH 2.0))))
    
    (if (and (>= x0 0.0)
             (< x0 windW)
             (>= y0 0.0)
             (< y0 windH))
        (begin
          (if (= (vector-ref nstar srec-type) STREAK) 
              (begin
                (set! x1 (/ (* (vector-ref nstar srec-x1) windW) 
                            (vector-ref nstar srec-z1)))
                (set! y1 (/ (* (vector-ref nstar srec-y1) windH) 
                            (vector-ref nstar srec-z1)))
                (set! rp (RotatePoint x1 y1 rot)) 
                (set! x1 (+ x1 (/ windW 2.0)))
                (set! y1 (+ y1 (/ windH 2.0)))
                (fl_set_linewidth
		 (inexact->exact (+ (/ MAXPOS 100.0 
				       (vector-ref nstar srec-z0)) 1.0)))
                (if (and (< (abs (- x0 x1)) 1.0)
                         (< (abs (- y0 y1)) 1.0))
                    (begin 
		      (fl_point (inexact->exact x0)
				(inexact->exact y0)
				FL_GRAY80)
		      )
                    (begin 
		      (fl_line  (inexact->exact x0)
				(inexact->exact y0)
				(inexact->exact x1)
				(inexact->exact y1)
				FL_WHITE)
		      )))
              (begin
                (set! width (+ (/ MAXPOS 10.0 
                                  (vector-ref nstar srec-z0) 1.0)))
		(fl_polyl 
		 (map (lambda (i)
			(list 
			 (inexact->exact 
			  (+ x0 
			     (* width 
				(fcos (/ (* i MAXANGLES) 8.0)))))
			 (inexact->exact
			  (+ y0
			     (* width
				(fsin (/ (* i MAXANGLES) 8.0)))))))
		      '(0 1 2 3 4 5 6 7))
		 8 FL_RED)
		))
          FL_TRUE)
        FL_FALSE)))


(define (ShowStars n) 
  (if (< n starCount)
      (begin
	(let ((nstar (vector-ref starv n)))
	  (if (or (> (vector-ref nstar srec-z0) speed)
		  (and (> (vector-ref nstar srec-z0) 0.0)
		       (< speed MAXWARP)))
	      (if (= (StarPoint n) FL_FALSE) 
		  (NewStar n MAXPOS)
		  )
	      (NewStar n MAXPOS)))
	(ShowStars (+ n 1)))))


(define (Init)
  (letrec ((loop (lambda (n)
		   (if (< n MAXSTARS)
		       (begin
			 (NewStar n 100)
			 (loop (+ n 1)))))))
    (loop 0))
  )

(define (Idle xev arg)
;  (fl_freeze_form form)
  (fl_rectf 0 0 windW windH FL_BLACK)
  (MoveStars 0)
  (ShowStars 0)
;  (fl_unfreeze_form form)
  (if (> nitro 0)
      (begin
	(set! speed (+ (/ nitro 10) 1.0))
	(if (> speed MAXWARP) 
	    (set! speed MAXWARP))
	(set! nitro (+ nitro 1))
	(if (> nitro (* MAXWARP 10))
	    (set! nitro (- nitro)))
	)
      (if (< nitro 0) 
	  (begin
	    (set! nitro (+ nitro 1))
	    (set! speed (+ (/ (- nitro)  10) 1.0))
	    (if (> speed MAXWARP) 
		(set! speed MAXWARP)))))
  ;(fl_flush)
  )

(define form '())
(define obj '())
(define windW #f)
(define windH #f)
(define (stars . args)
  (display "Click left mouse button in \"Starfield\" window to exit.\n")
  ;
  (set! windW 300)
  (set! windH 300)
  (fl_initialize args)
  (set! form (fl_bgn_form FL_NO_BOX windW windH))
  (set! obj (fl_add_box FL_FLAT_BOX 0 0 windW windH ""))
  (fl_set_object_dblbuffer obj FL_TRUE)
  (fl_set_object_color obj FL_BLACK FL_BLACK)
  (set! obj (fl_add_button FL_HIDDEN_BUTTON 0 0 windW windH ""))
  (fl_set_object_callback obj (lambda (o data) (exit 0)) 0)
  (fl_end_form)
  (Init)
  (fl_set_idle_callback idle 0)
  (fl_show_form form FL_PLACE_GEOMETRY FL_FULLBORDER "Starfield")
  (fl_do_forms)
  )


(stars)