;;; turtlegraphics primitives for the SCM interpreter by Aubrey Jaffer
;;; with SForms extension
;;; Author : Johannes Leveling

(require 'forms)

(define testing #f)
;; turtle bitmap
(define turtle_width 40)
(define turtle_height 40)

(define (bits2string bits)
  (if (null? bits)
      ""
      (string-append 
       (make-string 1 (integer->char (car bits)))
       (bits2string (cdr bits)))))

;;; turtle bitmap
(define turtle_bits
  (bits2string
   '(
     #x00 #x00 #x00 #x00 #x00 #x00 #x00 #x00 #x00 #x00 #x00 #x80
	  #xff #x00 #x00 #x00 #xe0 #x84 #x07 #x00 #x00 #x10 #x0c #x78
	  #x00 #x00 #x0c #x08 #xc0 #x00 #x00 #x02 #x18 #x40 #x03 #x00
	  #x01 #x28 #x40 #x02 #x80 #x0f #xcc #x7f #x02 #xc0 #xf8 #x07
	  #x10 #x06 #x40 #x00 #x04 #x30 #x0c #x60 #x00 #x04 #x7c #x08
	  #xf8 #x00 #x04 #xc2 #x0f #x8c #x01 #x0f #x01 #x0b #x26 #x01
	  #xf1 #x01 #x11 #x03 #x81 #x01 #x81 #x20 #x07 #x83 #x80 #x80
	  #x40 #x9a #x83 #x00 #x8f #xf8 #x42 #xc6 #xff #xe3 #x0f #x7e
	  #x63 #x18 #x22 #x00 #xc0 #x31 #xf0 #x13 #x00 #x00 #x11 #x00
	  #x11 #x00 #x00 #x0f #x00 #x1b #x00 #x00 #x06 #x00 #x0e #x0e
	  #x00 #x00 #x00 #x80 #x09 #x00 #x00 #x00 #xc0 #x88 #x00 #x00
	  #x00 #x00 #xff #x00 #x00 #x02 #x00 #x08 #x00 #x00 #x05 #x38
	  #x08 #x00 #x80 #x08 #x44 #x08 #x00 #x40 #x04 #x44 #x08 #x00
	  #x20 #x02 #x48 #x04 #x00 #x10 #x01 #x70 #x06 #x00 #x88 #x00
	  #xc0 #x01 #x00 #x44 #x00 #x40 #x00 #x00 #x22 #x0e #x40 #x00
	  #x00 #x11 #x0a #x40 #x00 #x00 #x09 #x0c #x40 #x00 #x80 #x06
	  #x3e #x20 #x00 #x80 #xf1 #xc1 #x1f #x00)))

;;; 
(define (gr:assert cond arg arg_no proc_name)
  (if cond
      #t
      (begin
	(set! errobj arg)
	(error (string-append proc_name ": Wrong type in arg" 
			      (number->string arg_no)
			      "(see errobj)\n")))))

(define (gr:line x1 y1 x2 y2 c)
  (fl_line (inexact->exact x1) (inexact->exact y1)
	   (inexact->exact x2) (inexact->exact y2)
	   (inexact->exact c)))

(define (gr:point x1 y1 c)
  (fl_point (inexact->exact x1) (inexact->exact y1)
	    (inexact->exact c)))

(define (gr:events)
  (fl_flush)
  )

(define M_PI 3.14159265358979323846) ; pi

(define (SIN_deg x)
  (sin (* (/ x 180.0) M_PI)))
(define (COS_deg x)
  (cos (* (/ x 180.0) M_PI)))

;;;
(define PROGNAME        "scm")
(define WINDOWNAME      "TurtleSCM graphics window")
(define ICONNAME        "TurtleSCM")

(define GR:MAX_XSIZE    1024)
(define GR:MAX_YSIZE    1024)
(define GR:DEF_XSIZE    640)
(define GR:DEF_YSIZE    480)
(define GR:MIN_XSIZE    64)
(define GR:MIN_YSIZE    64)

;;; fake CGA colormap with X 
(define GR:COLORS       16)           ; CGA/EGA counterpart  
(define GR:COLOR00      FL_BLACK)
(define GR:COLOR01      FL_BLUE)
(define GR:COLOR02      FL_GREEN)
(define GR:COLOR03      FL_CYAN)
(define GR:COLOR04      FL_RED)
(define GR:COLOR05      FL_MAGENTA)
(define GR:COLOR06      FL_YELLOW)
(define GR:COLOR07      FL_GRAY63)
(define GR:COLOR08      FL_GRAY35)
(define GR:COLOR09      FL_DODGERBLUE)
(define GR:COLOR10      FL_CHARTREUSE)
(define GR:COLOR11      FL_DARKCYAN)
(define GR:COLOR12      FL_INDIANRED)
(define GR:COLOR13      FL_ORCHID)
(define GR:COLOR14      FL_DARKORANGE)
(define GR:COLOR15      FL_WHITE)

(define gr:graphicsavail #f)
(define gr:grmode_on     #f)

(define gr:dir           0.0)
(define gr:max_x         0)
(define gr:max_y         0)
(define gr:max_color     0)
(define gr:x             0.0)
(define gr:y             0.0)
(define gr:color         0)

(define (valid_XYC x y color)
  (gr:events)
  (if (and (<= x gr:max_x) 
	   (<= y gr:max_y) 
	   (<= color gr:max_color)
	   (>= x 0) 
	   (>= y 0) 
	   (>= color 0))
      #t
      #f))

(define gr:display  '())
(define gr:win      '())
(define gr:pixmap   "")
(define gr:colortbl (make-vector GR:COLORS #f))
;;;
(define gr:windowname  WINDOWNAME)
(define gr:iconname    ICONNAME)
(define gr:progname    PROGNAME)

;;;
;;; user callable SCM routines
;;;
(define (gr:helpgr)
  (display "\
Ret   Name               nargs    args        returns\n\
---------------------------------------------------------\n\
B  graphics-avail?         0       -          #t if graphics available\n\
B  graphics-mode!          0       -          #f if no graphics\n\
B  text-mode!              0       -          #t on success\n\
B  clear-graphics!         0       -          #f if not in graphics mode\n\
i  max-x                   0       -          maximum value of x\n\
i  max-y                   0       -          maximum value of y\n\
i  max-color               0       -          maximum value of color\n\
B  valid-xyc?              3       x y color  #t if valid\n\
B  set-dot!                3       x y color  #t on success\n\
i  get-dot                 2       x y        color of the dot in (x,y)\n\
                                              or #f if (x,y) not legal\n\
\n\
NOTE: Origin (0,0) is in the upper left corner.\n\n\
" (current-output-port)))

(define (gr:helpturtlegr)
  (display "\
Ret   Name               nargs    args        returns\n\
---------------------------------------------------------\n\
B  goto-home!              0       -          #f if not in graphics mode\n\
B  goto-center!            0       -          #f if not in graphics mode\n\
B  goto-nw!                0       -          #f if not in graphics mode\n\
B  goto-ne!                0       -          #f if not in graphics mode\n\
B  goto-sw!                0       -          #f if not in graphics mode\n\
B  goto-se!                0       -          #f if not in graphics mode\n\
B  draw                    1       length     #t if target within drawing area\n\
B  draw-to                 2       x y        #t if (x,y) within drawing area\n\
B  draw-to!                2       x y        #t if (x,y) within drawing area\n\
B  move                    1       length     #t if target within drawing area\n\
B  move-to!                2       x y        #t if (x,y) within drawing area\n\
i  where-x                 0       -          current x-coordinate\n\
i  where-y                 0       -          current y-coordinate\n\
i  turn-right              1       angle      drawing direction in degrees\n\
i  turn-left               1       angle      drawing direction in degrees\n\
i  turn-to!                1       angle      drawing direction in degrees\n\
i  what-direction          0       -          drawing direction in degrees\n\
B  set-color!              1       color      #t if color valid\n\
i  what-color              0       -          current drawing color\n\n\
" (current-output-port)))

(define (gr:available?)
  (if gr:graphicsavail #t #f))

(define (gr:maxx)
  (if (not gr:grmode_on)
      #f
      (begin
        (gr:events)
	(inexact->exact (- (car (fl_get_winsize gr:win)) 1)))))

(define (gr:maxy)
  (if (not gr:grmode_on)
      #f
      (begin
        (gr:events)
	(inexact->exact (- (cadr (fl_get_winsize gr:win)) 1)))))

(define (gr:maxc)
  (if (not gr:grmode_on)
      #f
      (inexact->exact gr:max_color)))

(define (gr:validXYC? x y c)
  (gr:assert (number? x) x 1 "valid-xyc?")
  (gr:assert (number? y) y 2 "valid-xyc?")
  (gr:assert (number? c) c 3 "valid-xyc?")
  (if (not gr:grmode_on)
      #f
      (if (valid_XYC x y c)
	  #t
	  #f)))

(define (gr:grmode!)
  (if (not gr:graphicsavail)
      #f
      (begin
	(if (not gr:grmode_on) 
	    (fl_winshow gr:win)
	    )
	(set! gr:grmode_on #t)
	#t)))

(define (gr:txtmode!)
  (if (not gr:graphicsavail)
      #f
      (begin
        (if gr:grmode_on
	    (fl_winhide gr:win)
	    )
	(fl_flush)
        (set! gr:grmode_on #f)
	#t)))

(define (gr:cleargraph!)
  (if (not gr:grmode_on)
      #f
      (begin
        (fl_clear_window gr:display gr:win)
        (gr:events)
	#t)))

(define (gr:setdot! x y c)
  (gr:assert (number? x) x 1 "set-dot!")
  (gr:assert (number? y) y 2 "set-dot!")
  (gr:assert (number? c) c 3 "set-dot!")
  (if (not gr:grmode_on)
      #f
      (begin
	(if testing
	    (display 
	     (string-append "set-dot! called ("
			    (number->string x) ","
			    (number->string y) ","
			    (number->string c) ")\n") (current-error-port))
	    )
	(if (not (valid_XYC x y c))
	    #f
	    (begin
	      (gr:point x y c)
	      (gr:events)
	      #t)))))


(define (gr:getdot x y)
  (gr:assert (number? x) x 1 "get-dot")
  (gr:assert (number? y) y 2 "get-dot")
  (if (not gr:grmode_on)
      #f
      (begin
	(if testing
	    (display (string-append
		      "get-dot called ("
		      (number->string x) ","
		      (number->string y) ")\n" (current-error-port)))
	    )
	(if (not (valid_XYC x y 0))
	    #f
	    (begin 
	      (let ((pv (fl_get_pixel_color gr:win x y)))
		(letrec ((loop 
			  (lambda (i)
			    (if (> i GR:COLORS)
				(begin
				  (display (string-append 
					    gr:progname ": "
					    "get-dot" 
					    ": Got an illegal pixel value "
					    (number->string pv)))
				  #f)
				(if (= (fl_get_pixel 
					(vector-ref gr:colortbl i))
				       pv)
				    i
				    (loop (+ i 1)))))))
		  (loop 0))
		))))))

(define (gr:draw s)
  (gr:assert (number? s) s 1 "draw")
  (if (not gr:grmode_on)
      #f
      (begin
	(if testing
	    (display (string-append 
		      "draw called ("
		      (number->string s) 
		      ")\n" (current-error-port)))
	    )
	(let ((ok #t)
	      (x (+ gr:x (* (COS_deg gr:dir) s)))
	      (y (+ gr:y (* (SIN_deg gr:dir) s))))
	  (cond ((> x gr:max_x)
		 (set! x gr:max_x)
		 (set! ok #f))
		((< x 0.0)
		 (set! x 0.0)
		 (set! ok #f))
		((> y gr:max_y)
		 (set! y gr:max_y)
		 (set! ok #f))
		((< y 0.0)
		 (set! y 0.0)
		 (set! ok #f))
		(else 
		 #t))
	  (gr:line gr:x gr:y x y (vector-ref gr:colortbl gr:color))
	  (gr:events)
	  (set! gr:x x)
	  (set! gr:y y)
	  ok))))


(define (gr:move s)
  (gr:assert (number? s) s 1 "move")
  (if (not gr:grmode_on)
      #f     
      (begin
	(if testing
	    (display (string-append 
		      "move called ("
		      (number->string s)
		      ")\n") (current-error-port))
	    )
	(let ((ok #t)
	      (x (+ gr:x (* (COS_deg gr:dir) s)))
	      (y (+ gr:y (* (SIN_deg gr:dir) s))))
	  (cond ((> x gr:max_x)
		 (set! x gr:max_x)
		 (set! ok #f))
		((< x 0.0)
		 (set! x 0.0)
		 (set! ok #f))
		((> y gr:max_y)
		 (set! y gr:max_y)
		 (set! ok #f))
		((< y 0.0)
		 (set! y 0.0)
		 (set! ok #f))
		(else 
		 #t))	
	  (set! gr:x x)
	  (set! gr:y y)
	  ok))))

(define (gr:drawto x y)
  (gr:assert (number? x) x 1 "draw-to")
  (gr:assert (number? y) y 2 "draw-to")
  (if (not gr:grmode_on)
      #f 
      (begin
	(if testing
	    (display (string-append
		      "draw-to called ("
		      (number->string x) ","
		      (number->string y) ")\n") (current-error-port))
	    )
        (if (not (valid_XYC x y 0))
	    #f
	    (begin
	      (gr:line gr:x gr:y x y gr:color)
	      (gr:events)
	      #t)))))

(define (gr:drawto! x y)
  (gr:assert (number? x) x 1 "draw-to!")
  (gr:assert (number? y) y 2 "draw-to!")
  (if (not gr:grmode_on)
      #f
      (begin
	(if testing
	    (display (string-append
		      "draw-to! called ("
		      (number->string x) ","
		      (number->string y) ")\n") (current-error-port))
	    )
        (if (not (valid_XYC x y 0))
	    #f
	    (begin
	      (gr:line gr:x gr:y x y gr:color)
	      (gr:events)
	      (set! gr:x x)
	      (set! gr:y y)
	      #t)))))



(define (gr:moveto! x y)
  (gr:assert (number? x) x 1 "move-to!")
  (gr:assert (number? y) y 2 "move-to!")
  (if (not gr:grmode_on)
      #f
      (begin
	(if testing
	    (display (string-append
		      "move-to! called ("
		      (number->string x) ","
		      (number->string y) ")\n") (current-error-port))
	    )
        (if (not (valid_XYC x y 0))
	    #f
	    (begin
	      (set! gr:x x)
	      (set! gr:y y)
	      #t)))))

(define (gr:setcolor! c)
  (gr:assert (number? c) c 1 "set-color!")
  (if (not gr:grmode_on)
      #f        
      (begin
	(if testing
	    (display (string-append 
		      "set-color! called ("
		      (number->string c) 
		      ")\n") (current-error-port))
	    )
        (if (not (valid_XYC 0 0 c))
	    #f
	    (begin
	      (set! gr:color c)
	      (fl_set_foreground (fl_get_gc) (vector-ref gr:colortbl c))
	      (gr:events)
	      #t)))))

(define (gr:turnright d)
  (gr:assert (number? d) d 1 "turn-right")
  (if (not gr:grmode_on)
      #f
      (let ((di (modulo (inexact->exact d) 360)))
	(set! gr:dir (- gr:dir di))
	(set! gr:dir (modulo (inexact->exact gr:dir) 360))
	gr:dir)))

(define (gr:turnleft d)
  (gr:assert (number? d) d 1 "turn-left")
  (if (not gr:grmode_on)
      #f       
      (let ((di (modulo (inexact->exact d) 360)))
	(set! gr:dir (+ gr:dir di))
	(set! gr:dir (modulo (inexact->exact gr:dir) 360))
	gr:dir)))

(define (gr:turnto! d)
  (gr:assert (number? d) d 1 "turn-to!")
  (if (not gr:grmode_on)
      #f
      (let ((di (modulo (inexact->exact d) 360)))
	(set! gr:dir di)
	gr:dir)))

(define (gr:gotohome!)
  (if (not gr:grmode_on)
      #f
      (begin
	(set! gr:x 0.0)
        (set! gr:y 0.0)
	#t)))

(define (gr:gotocenter!)
  (if (not gr:grmode_on)
      #f
      (begin
	(gr:events)
	(set! gr:x (inexact->exact (/ (+ gr:max_x 1.0) 2.0)))
        (set! gr:y (inexact->exact (/ (+ gr:max_y 1.0) 2.0)))
	#t)))

(define (gr:gotonw!)
  (if (not gr:grmode_on)
      #f
      (begin
	(gr:events)
	(set! gr:x 0.0)
        (set! gr:y 0.0)
	#t)))

(define (gr:gotosw!)
  (if (not gr:grmode_on)
      #f
      (begin
	(gr:events)
	(set! gr:x 0.0)
        (set! gr:y gr:max_y)
	#t)))

(define (gr:gotone!)
  (if (not gr:grmode_on)
      #f
      (begin
	(gr:events)
	(set! gr:x gr:max_x)
        (set! gr:y 0.0)
	#t)))

(define (gr:gotose!)
  (if (not gr:grmode_on)
      #f
      (begin
	(gr:events)
	(set! gr:x gr:max_x)
        (set! gr:y gr:max_y)
	#t)))

(define (gr:whatcolor)
  (if (not gr:grmode_on)
      #f
      (inexact->exact gr:color)))

(define (gr:whatdirection)
  (if (not gr:grmode_on)
      #f
      (inexact->exact gr:dir)))

(define (gr:wherex)
  (if (not gr:grmode_on)
      #f
      (inexact->exact gr:x)))

(define (gr:wherey)
  (if (not gr:grmode_on)
      #f
      (inexact->exact gr:y)))

;;; initialization
(define (init_turtlegr)
  (if (null? gr:win)
      (begin
	(set! gr:x 0.0)
	(set! gr:y 0.0)
	(set! gr:dir 0.0)
	(set! gr:max_x 0)
	(set! gr:max_y 0)
	(set! gr:max_color 0) 
	;
	(fl_set_border_width 3)
	(set! gr:display (fl_initialize ""))
	(set! gr:win (fl_wincreate gr:windowname))
	(fl_winreshape gr:win 0 0 GR:DEF_XSIZE GR:DEF_YSIZE)
	(fl_winminsize gr:win GR:MIN_XSIZE GR:MIN_YSIZE)
	(fl_winmaxsize gr:win GR:MAX_XSIZE GR:MAX_YSIZE)
	(fl_wintitle gr:win gr:windowname)
	(fl_winbackground gr:win FL_BLACK)

	;(fl_set_form_atclose gr:form gr:at_close 0)
	;(fl_set_object_dblbuffer gr:obj FL_TRUE)
	;(fl_set_object_gravity gr:obj FL_NorthWestGravity)
	;(fl_set_object_color gr:obj FL_BLACK FL_BLACK)
	(set! gr:pixmap (fl_create_from_bitmapdata  ; transparent bg
			 gr:win 
			 turtle_bits turtle_width turtle_height))
	(fl_winicon gr:win gr:pixmap gr:pixmap)
	(fl_winicontitle gr:win gr:iconname)


	(set! gr:max_color (- GR:COLORS 1))
	;;; initialize color table
	(if (= (fl_get_visual_depth) 1) ; bw screen / 1 bitplane
	    (begin
	      (set! gr:colortbl (make-vector GR:COLORS FL_WHITE))
	      (vector-set! gr:colortbl 0 FL_BLACK))
	    (begin
	      (vector-set! gr:colortbl  0 GR:COLOR00)
	      (vector-set! gr:colortbl  1 GR:COLOR01)
	      (vector-set! gr:colortbl  2 GR:COLOR02)
	      (vector-set! gr:colortbl  3 GR:COLOR03)
	      (vector-set! gr:colortbl  4 GR:COLOR04)
	      (vector-set! gr:colortbl  5 GR:COLOR05)
	      (vector-set! gr:colortbl  6 GR:COLOR06)
	      (vector-set! gr:colortbl  7 GR:COLOR07)
	      (vector-set! gr:colortbl  8 GR:COLOR08)
	      (vector-set! gr:colortbl  9 GR:COLOR09)
	      (vector-set! gr:colortbl 10 GR:COLOR10)
	      (vector-set! gr:colortbl 11 GR:COLOR11)
	      (vector-set! gr:colortbl 12 GR:COLOR12)
	      (vector-set! gr:colortbl 13 GR:COLOR13)
	      (vector-set! gr:colortbl 14 GR:COLOR14)
	      (vector-set! gr:colortbl 15 GR:COLOR15)
	      )
	    )
	(set! gr:max_x (- GR:DEF_XSIZE 1))
	(set! gr:max_y (- GR:DEF_YSIZE 1))
	(set! gr:color 0)
	(set! gr:graphicsavail #t)
	)
      (set! gr:graphicsavail #f)
      )
  
  (set! gr:grmode_on #f)
  (display (string-append 
	    "\nSCM Turtlegraphics Copyright "
	    "(C) 1992 sjm@cc.tut.fi, jtl@cc.tut.fi\n"
	    "Type `(help-gr)' or `(help-turtlegr)' for a quick reference of\n"
	    "the new primitives.\n"))
  (display (string-append 
	    "SForms SCM Turtlegraphics emulation\n"
	    "(C) 1998 Free Software Foundation\n"
	    "Author : Johannes Leveling\n"
	    "email  : Johannes.Leveling@Informatik.Uni-Oldenburg.De\n"))
  ;
  (if (not gr:graphicsavail) 
      (display (string-append gr:progname 
			      ": No X server found. "
			      "Turtlegraphics not available.\n") 
	       (current-error-port))
      (gr:events)
      ))


; 0 args
(define help-gr         gr:helpgr)
(define help-turtlegr   gr:helpturtlegr)
(define graphics-mode!  gr:grmode!)
(define text-mode!      gr:txtmode!)
(define clear-graphics! gr:cleargraph!)
(define graphics-avail? gr:available?)
(define max-x           gr:maxx)
(define max-y           gr:maxy)
(define max-color       gr:maxc)
(define what-color      gr:whatcolor)
(define what-direction  gr:whatdirection)
(define where-x         gr:wherex)
(define where-y         gr:wherey)
(define goto-home!      gr:gotohome!)
(define goto-center!    gr:gotocenter!)
(define goto-nw!        gr:gotonw!)
(define goto-sw!        gr:gotosw!)
(define goto-ne!        gr:gotone!)
(define goto-se!        gr:gotose!)
; 1 arg
(define draw            gr:draw)
(define move            gr:move)
(define set-color!      gr:setcolor!)
(define turn-right      gr:turnright)
(define turn-left       gr:turnleft)
(define turn-to!        gr:turnto!)
; 2 args
(define get-dot         gr:getdot)
(define draw-to!        gr:drawto!)
(define draw-to         gr:drawto)
(define move-to!        gr:moveto!)
; 3 args
(define set-dot!        gr:setdot!)
(define valid-xyc?      gr:validXYC?)

;;;
(init_turtlegr)
(provide 'turtle-graphics)



