/*
**
** SForms : Interface to the XForms library for SCM
**
** This file is part of the SForms extension for SCM.
** (for XForms version 0.88 & SCM version 5b2 / 5c3)
**
** Author : Johannes Leveling
** eMail  : Johannes.Leveling@Informatik.Uni-Oldenburg.De
** Copyright (C) 1997, 1998 Free Software Foundation, Inc.
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this software; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

/*
** s_bitmap.c
** oct 28 1998
** nov 07 1998
*/
#include "forms.h"
#include "s_macro.h"

/* initialize symbols */
void s_init_bitmap_sym()
{
  /* types */
  sysintern("fl_normal_bitmap",  MAKINUM(FL_NORMAL_BITMAP));
  /* types */
  sysintern("fl_normal_pixmap",  MAKINUM(FL_NORMAL_PIXMAP));
  return;
}

/* routines */
SUBR_objectptr__5int_charptr(fl_create_bitmap);
SUBR_objectptr__5int_charptr(fl_add_bitmap);
SUBR_void__objectptr_2int_charptr(fl_set_bitmap_data);
SUBR_void__objectptr_charptr(fl_set_bitmap_file);
static char s_fl_read_bitmapfile[]="fl_read_bitmapfile";
SCM nfl_read_bitmapfile(SCM _w, SCM _s)
{
  unsigned int u1, u2;
  int i1, i2;
  Pixmap p;
  /* */
  ASSERT(WINP(_w), _w, ARG1, s_fl_read_bitmapfile);
  ASSERT(NIMP(_s) && STRINGP(_s), _s, ARG2, s_fl_read_bitmapfile);
  SFDEBUGMSG(s_fl_read_bitmapfile);
  p=fl_read_bitmapfile(WIN(_w), CHARS(_s), &u1, &u2, &i1, &i2);
  return cons(mkpix(p), 
	      cons(MAKINUM(u1), 
		   cons(MAKINUM(u2), 
			cons(MAKINUM(i1), 
			     cons(MAKINUM(i2), EOL)))));
} /* fl_read_bitmapfile */
static char s_fl_create_from_bitmapdata[]="fl_create_from_bitmapdata";
SCM nfl_create_from_bitmapdata(SCM _w, SCM _s, SCM _args)
{
  SCM w, h; /* width / height */
  ASSERT(ilength(_args)==2, UNDEFINED, WNA, s_fl_create_from_bitmapdata);
  w=CAR(_args); _args=CDR(_args);
  h=CAR(_args);
  ASSERT(WINP(_w), _w, ARG1, s_fl_create_from_bitmapdata);
  ASSERT(NIMP(_s) && STRINGP(_s), _s, ARG2, s_fl_create_from_bitmapdata);
  ASSERT(INUMP(w), w, ARG3, s_fl_create_from_bitmapdata);
  ASSERT(INUMP(h), h, ARG4, s_fl_create_from_bitmapdata);
  SFDEBUGMSG(s_fl_create_from_bitmapdata);
  return mkpix(fl_create_from_bitmapdata(WIN(_w), CHARS(_s), 
					 INUM(w), INUM(h)));
} /* fl_create_from_bitmapdata */
/* PIXMAP stuff */
SUBR_objectptr__5int_charptr(fl_create_pixmap);
SUBR_objectptr__5int_charptr(fl_add_pixmap);
static char s_fl_set_pixmap_data[]="fl_set_pixmap_data";
SCM nfl_set_pixmap_data(SCM _o, SCM _sl)
{
  int i, c;
  ASSERT(OBJP(_o), _o, ARG1, s_fl_set_pixmap_data);
  ASSERT(CONSP(_sl), _sl, ARG2, s_fl_set_pixmap_data);
  SFDEBUGMSG(s_fl_set_pixmap_data);
  i=ilength(_sl);
  {
    char *pdata[i];
    for (c=0; c<i; c++, _sl=CDR(_sl))
      {
	ASSERT(NIMP(CAR(_sl)) && STRINGP(CAR(_sl)), CAR(_sl), 
	       ARG2, s_fl_set_pixmap_data);
        pdata[c]=CHARS(CAR(_sl));
      }
    fl_set_pixmap_data(OBJ(_o), &pdata[0]);
  }
  return UNSPECIFIED;
} /* fl_set_pixmapdata */
SUBR_void__objectptr_charptr(fl_set_pixmap_file);
SUBR_void__objectptr_3int(fl_set_pixmap_align);
static char s_fl_set_pixmap_pixmap[]="fl_set_pixmap_pixmap";
SCM nfl_set_pixmap_pixmap(SCM _o, SCM _p1, SCM _p2)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_set_pixmap_pixmap);
  ASSERT(PIXP(_p1), _p1, ARG2, s_fl_set_pixmap_pixmap);
  ASSERT(PIXP(_p2), _p2, ARG3, s_fl_set_pixmap_pixmap);
  SFDEBUGMSG(s_fl_set_pixmap_pixmap);
  fl_set_pixmap_pixmap(OBJ(_o), PIX(_p1), PIX(_p2));
  return UNSPECIFIED;
} /* fl_set_pixmap_pixmap */
SUBR_void__3int(fl_set_pixmap_colorcloseness);
SUBR_void__objectptr(fl_free_pixmap_pixmap);
static char s_fl_get_pixmap_pixmap[]="fl_get_pixmap_pixmap";
SCM nfl_get_pixmap_pixmap(SCM _o)
{
  Pixmap p, p1, p2;
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_pixmap_pixmap);
  SFDEBUGMSG(s_fl_get_pixmap_pixmap);
  p=fl_get_pixmap_pixmap(OBJ(_o), &p1, &p2);
  return cons(mkpix(p), 
	      cons(mkpix(p1), 
		   cons(mkpix(p2), EOL)));
} /* fl_get_pixmap_pixmap */
static char s_fl_read_pixmapfile[]="fl_read_pixmapfile";
SCM nfl_read_pixmapfile(SCM _w, SCM _s, SCM _c)
{
  Pixmap r, p;
  unsigned int u1, u2;
  int i1, i2;
  ASSERT(WINP(_w), _w, ARG1, s_fl_read_pixmapfile);
  ASSERT(NIMP(_s) && STRINGP(_s), _s, ARG2, s_fl_read_pixmapfile);
  ASSERT(INUMP(_c), _c, ARG3, s_fl_read_pixmapfile);
  SFDEBUGMSG(s_fl_read_pixmapfile);
  r=fl_read_pixmapfile(WIN(_w), CHARS(_s), 
		       &u1, &u2, &p, &i1, &i2, INUM(_c));
  return cons(mkpix(r), 
	      cons(MAKINUM(u1), 
		   cons(MAKINUM(u2), 
			cons(mkpix(p), 
			     cons(MAKINUM(i1), 
				  cons(MAKINUM(i2), EOL))))));
} /* fl_read_pixmapfile */
static char s_fl_create_from_pixmapdata[]="fl_create_from_pixmapdata";
SCM nfl_create_from_pixmapdata(SCM _w, SCM _sl, SCM _c)
{
  int i, c;
  unsigned int u1, u2;
  int i1, i2;
  Pixmap r, p;
  /* */
  ASSERT(WINP(_w), _w, ARG1, s_fl_create_from_pixmapdata);
  ASSERT(CONSP(_sl), _sl, ARG2, s_fl_create_from_pixmapdata);  
  ASSERT(INUMP(_c), _c, ARG3, s_fl_create_from_pixmapdata);
  i=ilength(_sl);
  {
    char *pdata[i];
    for (c=0; c<i; c++, _sl=CDR(_sl))
      {
	ASSERT(NIMP(CAR(_sl)) && STRINGP(CAR(_sl)), CAR(_sl), ARG2, 
	       s_fl_create_from_pixmapdata);
	pdata[c]=CHARS(CAR(_sl));
      }
    r=fl_create_from_pixmapdata(WIN(_w), &pdata[0], 
				&u1, &u2, &p, &i1, &i2, INUM(_c));
  }
  return cons(mkpix(r), 
	      cons(MAKINUM(u1), 
		   cons(MAKINUM(u2), 
			cons(mkpix(p), 
			   cons(MAKINUM(i1), 
				cons(MAKINUM(i2), EOL)))))); 
} /* fl_create_from_pixmapdata */
static char s_fl_free_pixmap[]="fl_free_pixmap";
SCM nfl_free_pixmap(SCM _p)
{
  ASSERT(PIXP(_p), _p, ARG1, s_fl_free_pixmap);
  fl_free_pixmap(PIX(_p));
  return UNSPECIFIED;
} /* fl_free_pixmap */

/* initialize functions */
void s_init_bitmap_fct()
{
  MSUBR_L(fl_create_bitmap);
  MSUBR_L(fl_add_bitmap);
  MSUBR_L(fl_set_bitmap_data);
  MSUBR_2(fl_set_bitmap_file);
  MSUBR_2(fl_read_bitmapfile);
  MSUBR_L(fl_create_from_bitmapdata);
  /* */
  MSUBR_L(fl_create_pixmap);
  MSUBR_L(fl_add_pixmap);
  MSUBR_2(fl_set_pixmap_data);
  MSUBR_2(fl_set_pixmap_file);
  MSUBR_L(fl_set_pixmap_align);
  MSUBR_3(fl_set_pixmap_pixmap);
  MSUBR_3(fl_set_pixmap_colorcloseness);
  MSUBR_1(fl_free_pixmap_pixmap);
  MSUBR_1(fl_get_pixmap_pixmap);
  MSUBR_3(fl_read_pixmapfile);
  MSUBR_3(fl_create_from_pixmapdata);
  MSUBR_1(fl_free_pixmap);
  return;
}

/*
** EOF
*/
