/*
**
** SForms : Interface to the XForms library for SCM
**
** This file is part of the SForms extension for SCM.
** (for XForms version 0.88 & SCM version 5b2 / 5c3)
**
** Author : Johannes Leveling
** eMail  : Johannes.Leveling@Informatik.Uni-Oldenburg.De
** Copyright (C) 1997, 1998 Free Software Foundation, Inc.
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this software; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/


/*
** s_callb.c
*/
#include "forms.h"
#include "s_macro.h"


/* callback maintainance */
/* for filter, handler, callbacks */ 

/* block / unblock signals */
/*
#include "signal.h"
#ifdef SIG_UNBLOCK   
extern void ignore_signals();
extern void restore_signals();

void cb_block()
{
  SFDEBUGMSG("ignore (");
  ignore_signals();
  return;
}

void cb_unblock()
{
  restore_signals();
  SFDEBUGMSG(") restore");
  return;
}
#else                                 
#define cb_block() ;
#define cb_unblock() ;
#endif 
*/
/* SIG_UNBLOCK */


/* add callback */
void cb_add(SCM *_loc, SCM _key, SCM _proc)
{
  SCM lst, entry;
  /*ASSERT(CONSP(*_loc), *_loc, ARG1, "cb_add : invalid list");*/
  lst=*_loc;
  entry=cons2(_key, _proc, EOL);
  for(; NIMP(lst); lst=CDR(lst)) 
    {
      ASSERT(CONSP(CAR(lst)), lst, 0, "cb_add : invalid procedure");
      if (equal(CAR(CAR(lst)), _key)!=BOOL_F)
	{
	  SFDEBUGMSG("cb_add : procedure already present"); 
	  return;
	}
    }
  *_loc=cons(entry, *_loc);
  return;
}

/* find callback (return procedure) */
SCM cb_get(SCM _loc, SCM _key)
{
  SCM entry;
  ASSERT(CONSP(_loc), _loc, ARG1, "cb_get : invalid list");
  for (;NIMP(_loc); _loc=CDR(_loc))
    {
      ASSERT(CONSP(CAR(_loc)), CAR(_loc), 0, "cb_get : invalid procedure");
      entry=CAR(_loc);
      if (equal(CAR(entry), _key)!=BOOL_F) return CAR(CDR(entry));
    }
  SFDEBUGMSG("cb_get : no callback found !");
  return BOOL_F;
}

/* remove callback */
void cb_del(SCM *_loc, SCM _key)
{
  SCM entry, lst=EOL;
  ASSERT(CONSP(*_loc), *_loc, ARG1, "cb_del : invalid list");
  for (;NIMP(*_loc); *_loc=CDR(*_loc))
    {
      ASSERT(CONSP(CAR(*_loc)), CAR(*_loc), 0, "cb_del : invalid procedure");
      entry=CAR(*_loc);
      if (equal(CAR(entry), _key)==BOOL_F)
	lst=cons(entry, lst);
    }
  *_loc=lst;
  return;
}

/* BASIC */
/* io callback */
void cb_io_callback(int i, void *_data)
{
  SCM proc;
  SFDEBUGMSG("* io callback");
  proc=cb_get(*loc_io_callback, MAKINUM(i));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return;
  apply(proc,
	cons(MAKINUM(i), 
	     cons((SCM)_data, EOL)), 
	EOL);
  return;
}

/* signal callback / handler */
void cb_signal_callback(int i, void *_data)
{
  SCM proc;
  SFDEBUGMSG("* signal callback");
  proc=cb_get(*loc_signal_callback, MAKINUM(i));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return;
  apply(proc,
	cons(MAKINUM(i), 
	     cons((SCM)_data, EOL)), 
	EOL);
  return;
}

/* timeout handler */
void cb_timeout_handler(int i, void *_data)
{
  SCM proc;
  SFDEBUGMSG("* timeout handler");
  proc=cb_get(*loc_timeout_handler, MAKINUM(i));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return;
  apply(proc,
	cons(MAKINUM(i), 
	     cons((SCM)_data, EOL)), 
	EOL);
  return;
}

/* form atclose callback */
int cb_form_atclose_callback(FL_FORM *f, void *v)
{
  SCM proc;
  SFDEBUGMSG("* form atclose callback");
  proc=cb_get(*loc_form_atclose_callback, mkform(f));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons((SCM)v, EOL), 
			 EOL));
}

/* atclose callback */
int cb_atclose_callback(FL_FORM *f, void *v)
{
  SCM proc;
  SFDEBUGMSG("* atclose callback");
  proc=cb_get(*loc_form_atclose_callback, mkform(f));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons((SCM)v, EOL), 
			 EOL));
}

/* atdeactivate callback */
void cb_form_atdeactivate_callback(FL_FORM *f, void *v)
{
  SCM proc;
  SFDEBUGMSG("* form atdeactivate callback");
  proc=cb_get(*loc_form_atdeactivate_callback, mkform(f));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return;
  apply(proc, 
	cons((SCM)v, EOL), 
	EOL);
  return;
}

/* atactivate callback */
void cb_form_atactivate_callback(FL_FORM *f, void *v)
{
  SCM proc;
  SFDEBUGMSG("* form atactivate callback");
  proc=cb_get(*loc_form_atactivate_callback, mkform(f));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return;
  apply(proc,
	cons((SCM)v, EOL), 
	EOL); 
  return;
}

/* form callback */
void cb_form_callback(FL_OBJECT *o, void *v)
{
  SCM proc;
  SFDEBUGMSG("* form callback");
  proc=cb_get(*loc_form_callback, mkform(o->form));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return;
  apply(proc, 
	cons(mkobj(o), 
	     cons(MAKINUM((int)v), EOL)), 
	EOL);
  return;
}

/* raw callback */
int cb_raw_callback(FL_FORM *f, void *v)
{
  SCM proc;
  SFDEBUGMSG("* raw callback");
  proc=cb_get(*loc_raw_callback, mkform(f));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons((SCM)v, EOL), 
			 EOL));
}

/* object prehandler */
int cb_object_prehandler(FL_OBJECT *o, int i1, FL_Coord c1, FL_Coord c2,
			  int i2, void *v)
{
  SCM proc;
  SFDEBUGMSG("* object prehandler");
  proc=cb_get(*loc_object_prehandler, mkobj(o));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc,
			 cons(mkobj(o), 
			      cons(MAKINUM(i1), 
				   cons(MAKINUM(c1), 
					cons(MAKINUM(c2),
					     cons(MAKINUM(i2),
						  cons((SCM)v, 
						       EOL)))))), 
			 EOL));
}

/* object posthandler */
int cb_object_posthandler(FL_OBJECT *o, int i1, FL_Coord c1, FL_Coord c2,
			   int i2, void *v)
{
  SCM proc;
  SFDEBUGMSG("* object posthandler");
  proc=cb_get(*loc_object_posthandler, mkobj(o));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc,
			 cons(mkobj(o), 
			      cons(MAKINUM(i1), 
				   cons(MAKINUM(c1), 
					cons(MAKINUM(c2),
					     cons(MAKINUM(i2),
						  cons((SCM)v, 
						       EOL)))))), 
			 EOL));
}

/* object callback */
void cb_object_callback(FL_OBJECT *o, long data)
{
  SCM proc;
  SFDEBUGMSG("* object callback");
  proc=cb_get(*loc_object_callback, mkobj(o));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return;
  apply(proc, 
	cons(mkobj(o), 
	     cons(MAKINUM(data), EOL)), 
	EOL);
  return;
  
}

/* font enumerator */
void cb_font_enumerator(const char *s)
{
  SCM proc;
  SFDEBUGMSG("* font enumerator");
  proc=cb_get(*loc_font_enumerator, MAKINUM(0));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return;
  apply(proc, 
	cons(MAKSTR((char *)s), EOL), 
	EOL);
  return;
}

/* symbol handler */
void cb_symbol_handler(FL_Coord i1, FL_Coord i2, 
		       FL_Coord i3, FL_Coord i4, int i5, FL_COLOR i6)
{
  SCM proc;
  SFDEBUGMSG("* symbol handler");
  proc=cb_get(*loc_symbol_handler, MAKINUM(0));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return;
  apply(proc,
	cons(MAKINUM(i1),
	     cons(MAKINUM(i2),
		  cons(MAKINUM(i3),
		       cons(MAKINUM(i4),
			    cons(MAKINUM(i5),
				 cons(MAKINUM(i6), EOL)))))), 
	EOL);
  return;
}

/* error handler */
void cb_error_handler(const char *s1, const char *s2, ...)
{ /* ignore additional args */
  SCM proc;
  SFDEBUGMSG("* error handler");
  proc=cb_get(*loc_error_handler, MAKINUM(0));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return;
  apply(proc, 
	cons(MAKSTR((char *)s1), 
	     cons(MAKSTR((char *)s2), EOL)), 
	EOL); 
  return;
}


/* new object handler (fl_make_object) */
int cb_new_object_handler(FL_OBJECT *o, int i1, 
			  FL_Coord i2, FL_Coord i3, int i4, void *v)
{
  SCM proc;
  SFDEBUGMSG("* new object handler");
  proc=cb_get(*loc_new_object_handler, MAKINUM(0));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return -1;
  return (int)INUM(apply(proc,
			 cons(mkobj(o), 
			      cons(MAKINUM(i1), 
				   cons(MAKINUM(i2), 
					cons(MAKINUM(i3), 
					     cons(MAKINUM(i4), 
						  /* XEvent ignored for now */
						  cons(MAKINUM(0), EOL)))))), 
		    EOL));
}


/* BITMAP */
/* - */

/* BOX */
/* - */

/* BROWSER */
/* browser dblclick callback */
void cb_browser_dblclick_callback(FL_OBJECT *o, long _data)
{
  SCM proc;
  SFDEBUGMSG("* browser dblclick callback");
  proc=cb_get(*loc_browser_dblclick_callback, mkobj(o));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return;
  apply(proc, 
	cons2(mkobj(o), MAKINUM(_data), EOL), EOL);
  return;
}

/* BUTTON */
/* draw button */
void cb_draw_button(FL_OBJECT *o)
{
  SCM proc;
  SFDEBUGMSG("* draw button");
  proc=cb_get(*loc_draw_button, MAKINUM(o->objclass)); /* !? */
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return;
  apply(proc, 
	cons(mkobj(o), EOL), EOL);
  return;
}

/* cleanup button */
void cb_cleanup_button(FL_BUTTON_STRUCT *bs)
{
  SCM proc;
  SFDEBUGMSG("* cleanup button");
  proc=cb_get(*loc_cleanup_button, MAKINUM(0));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return;
  apply(proc, EOL, EOL); /* FL_BUTTON_STRUCT ignored */
  return;
}


/* CANVAS */
/* canvas handler */
int cb_canvas_handler(FL_OBJECT *o, Window w, int i1, int i2, 
		      XEvent *xev, void *v)
{
  int ret;
  SCM proc;
  SFDEBUGMSG("* canvas handler");
  proc=cb_get(*loc_canvas_handler, cons(mkobj(o), MAKINUM(xev->type)));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  ret=(int)INUM(apply(proc, 
		      cons(mkobj(o),
			   cons(mkwin(w), 
				cons(MAKINUM(i1), 
				     cons(MAKINUM(i2),
					  cons(mkxev(xev), 
					       cons((SCM)v, EOL)))))), 
		      EOL));
  return ret;
}

/* canvas modifier */
int cb_canvas_modify1(FL_OBJECT *o)
{
  SCM proc;
  SFDEBUGMSG("* canvas modify1");
  proc=cb_get(*loc_canvas_modify1, mkobj(o));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons(mkobj(o), EOL), 
			 EOL));
}
int cb_canvas_modify2(FL_OBJECT *o)
{
  SCM proc;
  SFDEBUGMSG("* canvas modify2");
  proc=cb_get(*loc_canvas_modify2, mkobj(o));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons(mkobj(o), EOL), 
			 EOL));
}
int cb_canvas_modify3(FL_OBJECT *o)
{
  SCM proc;
  SFDEBUGMSG("* canvas modify3");
  proc=cb_get(*loc_canvas_modify3, mkobj(o));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons(mkobj(o), EOL), 
			 EOL));
}

/* CHART */
/* - */

/* CHOICE */
/* choice item callback */
int cb_choice_item_callback(int i)
{
  SCM proc;
  SFDEBUGMSG("* choice item callback");
  proc=cb_get(*loc_choice_item_callback, MAKINUM(i));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons(MAKINUM(i), EOL), 
			 EOL)); 
}

/* CLIPBOARD */
/* clipboard stuff / lose selection */
int cb_stuff_clipboard(FL_OBJECT *o, long type)
{
  SCM proc;
  SFDEBUGMSG("* stuff clipboard");
  proc=cb_get(*loc_stuff_clipboard_callback, mkobj(o));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons(mkobj(o), cons(MAKINUM(type), EOL)), 
			 EOL));
			 
		    
}

/* clipboard request selection */
int cb_request_clipboard(FL_OBJECT *o, long type, const void *data, long size)
{
  SCM proc;
  SFDEBUGMSG("* request clipboard");
  proc=cb_get(*loc_request_clipboard_callback, mkobj(o));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons(mkobj(o), 
			      cons(MAKINUM(type), 
				   cons((SCM)data, 
					cons(MAKINUM(size), EOL)))), 
			 EOL));
}

/* CLOCK */
/* - */

/* COUNTER */
/* counter filter */
const char *cb_counter_filter(FL_OBJECT *o, double d, int i)
{
  SCM proc;
  SFDEBUGMSG("* counter filter");
  proc=cb_get(*loc_counter_filter, mkobj(o));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return NULL;
  return (const char *)
    CHARS(apply(proc, 
		cons(mkobj(o), 
		     cons(makdbl((double)d, 0.0), 
			  cons(MAKINUM(i), EOL))), 
		EOL));
}

/* CURSOR */
/* - */

/* DIAL */
/* - */

/* FILESYS */
/* dirlist filter */
int cb_dirlist_filter(const char *s, int i)
{
  SCM proc;
  SFDEBUGMSG("* dirlist filter");
  proc=cb_get(*loc_dirlist_filter, MAKINUM(0));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons(MAKSTR((char *)s), 
			      cons(MAKINUM(i), EOL)), 
			 EOL));
}

/* FLPS */
/* - */

/* FOLDER */
/* - */

/* FRAME */
/* - */

/* FREE */
/* free handler */
int cb_free_handler(FL_OBJECT *o, int i1, 
		    FL_Coord c1, FL_Coord c2, int i2, void *xev)
{
  SCM proc;
  SFDEBUGMSG("* free handler");
  proc=cb_get(*loc_free_handler, MAKINUM(0));
  return (int)INUM(apply(proc, 
			 cons(mkobj(o), 
			      cons(MAKINUM(i1), 
				   cons(MAKINUM(c1), 
					cons(MAKINUM(c2), 
					     cons(MAKINUM(i2),
						  /* XEvent ignored for now */ 
						  cons(MAKINUM(0),
						       EOL)))))),
			 EOL));
}

/* GL */
/* ? */

/* GOODIES */
/* file selector callback */
int cb_fselector_callback(const char *s, void *v)
{
  SCM proc;
  SFDEBUGMSG("* fselector_callback");
  proc=cb_get(*loc_fselector_callback, MAKINUM(0));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons(MAKSTR((char *)s), 
			      cons((SCM)v, EOL)), 
			 EOL));
}

/* file selector appbutton callback */
void cb_fselector_appbutton(void *v)
{
  SCM proc;
  SFDEBUGMSG("* fselector appbutton callback");
  proc=cb_get(*loc_fselector_appbutton, MAKINUM(0));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return;
  apply(proc, 
	cons((SCM)v, EOL), 
	EOL);
  return;
}

/* INPUT */
/* input filter / validator */
int cb_input_filter(FL_OBJECT *o, const char *s1, const char *s2, int i)
{
  SCM proc;
  SFDEBUGMSG("* input filter");
  proc=cb_get(*loc_input_filter, mkobj(o));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons(mkobj(o), 
			      cons(MAKSTR((char *)s1), 
				   cons(MAKSTR((char *)s2), 
					cons(MAKINUM(i), EOL)))), 
			 EOL));
}

/* MENU */
/* menu item callback */
int cb_menu_item_callback(int i)
{
  SCM proc;
  SFDEBUGMSG("* menu item callback");
  proc=cb_get(*loc_menu_item_callback, MAKINUM(i));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons(MAKINUM(i), EOL), 
			 EOL)); 
}

/* MENUBAR */
/* menubar item callback */
int cb_menubar_item_callback(int i)
{
  SCM proc;
  SFDEBUGMSG("* menubar item callback");
  proc=cb_get(*loc_menubar_item_callback, MAKINUM(i));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons(MAKINUM(i), EOL), 
			 EOL)); 
}

/* POPUP */
/* pup item callback */
int cb_pup_item_callback(int i)
{
  SCM proc;
  SFDEBUGMSG("* pup item callback");
  proc=cb_get(*loc_pup_item_callback, MAKINUM(i));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons(MAKINUM(i), EOL), 
			 EOL)); 
}

/* pup enter callback */
void cb_pup_enter_callback(int i, void *v)
{
  SCM proc;
  SFDEBUGMSG("* pup enter callback");
  proc=cb_get(*loc_pup_enter_callback, 
	      MAKINUM(fl_current_pup()));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return;
  apply(proc, 
	cons(MAKINUM(i), 
	     cons(MAKINUM((int)v), EOL)), 
	EOL);
  return;
}

/* pup leave callback */
void cb_pup_leave_callback(int i, void *v)
{
  SCM proc;
  SFDEBUGMSG("* pup leave callback");
  proc=cb_get(*loc_pup_leave_callback, 
	      MAKINUM(fl_current_pup()));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return;
  apply(proc, 
	cons(MAKINUM(i), 
	     cons(MAKINUM((int)v), EOL)), 
	EOL);
  return;
}

/* pup menu callback */
int cb_pup_menu_callback(int i)
{
  SCM proc;
  SFDEBUGMSG("* pup menu callback");
  proc=cb_get(*loc_pup_menu_callback, MAKINUM(i));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons(MAKINUM(i), EOL), 
			 EOL)); 
}

/* POSITIONER */
/* - */

/* SCROLLBAR */
/* - */

/* SLIDER */
/* slider filter */
const char *cb_slider_filter(FL_OBJECT *o, double d, int i)
{
  SCM proc;
  SFDEBUGMSG("* slider filter");
  proc=cb_get(*loc_slider_filter, mkobj(o));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return NULL;
  return (const char *)
    CHARS(apply(proc, 
		cons(mkobj(o), 
		     cons(makdbl((double)d, 0.0), 
			  cons(MAKINUM(i), EOL))), 
		EOL));
}

/* TEXT */

/* TIMER */
/* timer filter */
char *cb_timer_filter(FL_OBJECT *o, double d)
{
  SCM proc;
  SFDEBUGMSG("* timer filter");
  proc=cb_get(*loc_timer_filter, mkobj(o));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return NULL;
  return (char *)
    CHARS(apply(proc, 
		cons(mkobj(o), 
		     cons(makdbl((double)d, 0.0), EOL)), 
		EOL));
}

/* XYPLOT */
/* xyplot symbol handler */
void cb_xyplot_symbol_handler(FL_OBJECT *o, int i1, FL_POINT *p, 
			      int i2, int i3, int i4)
{ /* CHECK */
  SCM proc;
  SFDEBUGMSG("* xyplot symbol handler");
  proc=cb_get(*loc_xyplot_symbol_handler, mkobj(o));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return;
  {
    SCM plst=EOL;
    FL_POINT pt, *cp;
    for (cp=&pt; cp!=NULL; cp++)
      {
	plst=cons(cons(MAKINUM((pt.x)), MAKINUM(pt.y)),
		  plst); 
      }
    apply(proc, 
	  cons(mkobj(o), 
	       cons(MAKINUM(i1), 
		    cons(plst, 
			 cons(MAKINUM(i2), 
			      cons(MAKINUM(i3), 
				   cons(MAKINUM(i4), EOL)))))), 
	  EOL);
  }
  return;
}

/* XBASIC */
/* event callback */
int cb_event_callback(XEvent *xev, void *v)
{
  SCM proc;
  SFDEBUGMSG("* event callback");
  proc=cb_get(*loc_event_callback, MAKINUM(0));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons(mkxev(xev), 
			      cons((SCM)v, EOL)), 
			 EOL));
}

/* idle callback */
int cb_idle_callback(XEvent *xev, void *v)
{
  SCM proc;
  SFDEBUGMSG("* idle callback");
  proc=cb_get(*loc_idle_callback, MAKINUM(0));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons(mkxev(xev), 
			      cons((SCM)v, EOL)), 
			 EOL));
}

/* 2nd event callback */
int cb_sevent_callback(XEvent *xev, void *v)
{
  SCM proc;
  SFDEBUGMSG("* sevent callback");
  proc=cb_get(*loc_sevent_callback, MAKINUM(0));
  if (proc==BOOL_F ||
      procedurep(proc)!=BOOL_T) return 0;
  return (int)INUM(apply(proc, 
			 cons(mkxev(xev), 
			      cons((SCM)v, EOL)), 
			 EOL));
}

/*
** EOF 
*/


