/*
**
** SForms : Interface to the XForms library for SCM
**
** This file is part of the SForms extension for SCM.
** (for XForms version 0.88 & SCM version 5b2 / 5c3)
**
** Author : Johannes Leveling
** eMail  : Johannes.Leveling@Informatik.Uni-Oldenburg.De
** Copyright (C) 1997, 1998 Free Software Foundation, Inc.
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this software; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

/*
** s_canvas.c
** oct 28 1998
** - GL/Mesa not supported (see s_gl.c)
*/
#include "forms.h"
#include "s_macro.h"

/* initialize symbols */
void s_init_canvas_sym()
{
  /* types */
  sysintern("fl_normal_canvas",   MAKINUM(FL_NORMAL_CANVAS));
  sysintern("fl_scrolled_canvas", MAKINUM(FL_SCROLLED_CANVAS));
  return;
}

/* routines */
static char s_fl_create_generic_canvas[]="fl_create_generic_canvas";
SCM nfl_create_generic_canvas(SCM _i1, SCM _i2, SCM _args)
{
  SCM _i3, _i4, _i5, _i6, _s;
  FL_OBJECT *obj;
  ASSERT(ilength(_args)==5, _args, ARG3, s_fl_create_generic_canvas);
  _i3=CAR(_args); _args=CDR(_args);
  _i4=CAR(_args); _args=CDR(_args);
  _i5=CAR(_args); _args=CDR(_args);
  _i6=CAR(_args); _args=CDR(_args);
  _s=CAR(_args);
  ASSERT(INUMP(_i1), _i1, ARG1, s_fl_create_generic_canvas);
  ASSERT(INUMP(_i2), _i3, ARG2, s_fl_create_generic_canvas);
  ASSERT(INUMP(_i3), _i3, ARG3, s_fl_create_generic_canvas);
  ASSERT(INUMP(_i4), _i4, ARG4, s_fl_create_generic_canvas);
  ASSERT(INUMP(_i5), _i5, ARG5, s_fl_create_generic_canvas);
  ASSERT(INUMP(_i6), _i6, ARGn, s_fl_create_generic_canvas);
  ASSERT(NIMP(_s) && STRINGP(_s), _s, ARGn, s_fl_create_generic_canvas);
  SFDEBUGMSG(s_fl_create_generic_canvas);
  obj=fl_create_generic_canvas(INUM(_i1), INUM(_i2), INUM(_i3), 
	    INUM(_i4), INUM(_i5), INUM(_i6), CHARS(_s));
  return mkobj(obj);
}
SUBR_objectptr__5int_charptr(fl_add_canvas);
SUBR_objectptr__5int_charptr(fl_create_canvas);
/* fl_create_mesacanvas */
/* fl_add_mesacanvas */
SUBR_void__objectptr_int(fl_set_canvas_colormap);
SUBR_dummy2(fl_set_canvas_visual);
SUBR_void__objectptr_int(fl_set_canvas_depth);
SUBR_dummy3(fl_set_canvas_attributes);
/* */
static char s_fl_add_canvas_handler[]="fl_add_canvas_handler";
SCM nfl_add_canvas_handler(SCM _o, SCM _i, SCM _args)
{
  SCM _proc, _v;
  ASSERT(ilength(_args)==2, _args, ARG3, s_fl_add_canvas_handler);
  _proc=CAR(_args); _args=CDR(_args);
  _v=CAR(_args);
  ASSERT(OBJP(_o), _o, ARG1, s_fl_add_canvas_handler);
  ASSERT(INUMP(_i), _i, ARG2, s_fl_add_canvas_handler);
  ASSERT(BOOL_T==procedurep(_proc), _proc, ARG3, s_fl_add_canvas_handler);
  /*ASSERT(INUMP(_v), _v, ARG4, s_fl_add_canvas_handler);*/
  SFDEBUGMSG(s_fl_add_canvas_handler);
  cb_add(loc_canvas_handler, cons(_o, _i), _proc);
  fl_add_canvas_handler(OBJ(_o), INUM(_i), cb_canvas_handler, (void *)_v);
  return UNSPECIFIED;
} /* fl_add_canvas_handler */
SUBR_window__objectptr(fl_get_canvas_id);
SUBR_int__objectptr(fl_get_canvas_colormap);
SUBR_int__objectptr(fl_get_canvas_depth);
static char s_fl_remove_canvas_handler[]="fl_remove_canvas_handler";
SCM nfl_remove_canvas_handler(SCM _o, SCM _i, SCM _proc)
{ 
  ASSERT(OBJP(_o), _o, ARG1, s_fl_remove_canvas_handler);
  ASSERT(INUMP(_i), _i, ARG2, s_fl_remove_canvas_handler);
  ASSERT(BOOL_T==procedurep(_proc), _proc, ARG3, s_fl_remove_canvas_handler);
  SFDEBUGMSG(s_fl_remove_canvas_handler);
  cb_del(loc_canvas_handler, cons(_o, _i)); 
  fl_remove_canvas_handler(OBJ(_o), INUM(_i), cb_canvas_handler);
  return UNSPECIFIED;
} /* fl_remove_canvas_handler */
SUBR_void__objectptr(fl_hide_canvas);    
SUBR_void__objectptr_int(fl_canvas_yield_to_shortcut);
SUBR_void__objectptr_int(fl_share_canvas_colormap);
static char s_fl_modify_canvas_prop[]="fl_modifiy_canvas_prop";
SCM nfl_modify_canvas_prop(SCM _o, SCM _proc1, SCM _args)
{ 
  SCM _proc2, _proc3;
  ASSERT(ilength(_args)==2, _args, ARG3, s_fl_modify_canvas_prop);
  _proc2=CAR(_args); _args=CDR(_args);
  _proc3=CAR(_args);
  SFDEBUGMSG(s_fl_modify_canvas_prop);
  cb_add(loc_canvas_modify1, _o, _proc1); 
  cb_add(loc_canvas_modify2, _o, _proc2); 
  cb_add(loc_canvas_modify3, _o, _proc3); 
  fl_modify_canvas_prop(OBJ(_o), 
			cb_canvas_modify1, 
			cb_canvas_modify2, 
			cb_canvas_modify3);
  return UNSPECIFIED;
} /* fl_modify_canvas_prop */

/* OpenGL canvases */
/*SUBR_objectptr__5int_charptr(fl_create_glcanvas);*/
/*SUBR_objectptr__5int_charptr(fl_add_glcanvas);*/
/* fl_set_glcanvas_defaults */
/* fl_get_glcanvas_defaults */
/* fl_set_glcanvas_attributes */
/* fl_get_glcanvas_attributes */
/*SUBR_void__objectptr_int(fl_set_glcanvas_direct);*/
/*SUBR_void__objectptr(fl_activate_glcanvas);*/
/* fl_get_glcanvas_xvisualinfo */
/* fl_get_glcanvas_context */
/* fl_glwincreate */
/* fl_glwinopen */


/* initialize functions */
void s_init_canvas_fct()
{
  MSUBR_L(fl_create_generic_canvas);
  MSUBR_L(fl_add_canvas);
  MSUBR_L(fl_create_canvas);
  MSUBR_2(fl_set_canvas_colormap);
  MSUBR_d2(fl_set_canvas_visual);
  MSUBR_2(fl_set_canvas_depth);
  MSUBR_d3(fl_set_canvas_attributes);
  MSUBR_L(fl_add_canvas_handler);
  MSUBR_1(fl_get_canvas_id);
  MSUBR_1(fl_get_canvas_colormap);
  MSUBR_1(fl_get_canvas_depth);
  MSUBR_3(fl_remove_canvas_handler);
  MSUBR_1(fl_hide_canvas);    
  MSUBR_2(fl_canvas_yield_to_shortcut);
  MSUBR_2(fl_share_canvas_colormap);
  MSUBR_L(fl_modify_canvas_prop);
  /* */
  /*MSUBR_L(fl_create_glcanvas);*/
  /*MSUBR_L(fl_add_glcanvas);*/
  /*MSUBR_2(fl_set_glcanvas_direct);*/
  /*MSUBR_1(fl_activate_glcanvas);*/
  return;
}

/*
** EOF
*/
