/*
**
** SForms : Interface to the XForms library for SCM
**
** This file is part of the SForms extension for SCM.
** (for XForms version 0.88 & SCM version 5b2 / 5c3)
**
** Author : Johannes Leveling
** eMail  : Johannes.Leveling@Informatik.Uni-Oldenburg.De
** Copyright (C) 1997, 1998 Free Software Foundation, Inc.
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this software; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

/*
** s_choice.c
** oct 28 1998
** nov 07 1998
*/
#include "forms.h"
#include "s_macro.h"

/* initialize symbols */
void s_init_choice_sym()
{
  /* types */
  sysintern("fl_normal_choice",   MAKINUM(FL_NORMAL_CHOICE));
  sysintern("fl_normal_choice2",  MAKINUM(FL_NORMAL_CHOICE2));
  sysintern("fl_droplist_choice", MAKINUM(FL_DROPLIST_CHOICE));
  /* others */
  sysintern("fl_choice_mcol",     MAKINUM(FL_CHOICE_MCOL));
  sysintern("fl_choice_maxitems", MAKINUM(FL_CHOICE_MAXITEMS)); 
  return;
}

/* routines */
SUBR_objectptr__5int_charptr(fl_create_choice);
SUBR_objectptr__5int_charptr(fl_add_choice);
/* */
SUBR_void__objectptr(fl_clear_choice);
SUBR_void__objectptr_charptr(fl_addto_choice);
SUBR_void__objectptr_int_charptr(fl_replace_choice);
SUBR_void__objectptr_int(fl_delete_choice);
SUBR_void__objectptr_int(fl_set_choice);
SUBR_void__objectptr_charptr(fl_set_choice_text);
SUBR_int__objectptr(fl_get_choice);
SUBR_charptr__objectptr_int(fl_get_choice_item_text);
SUBR_int__objectptr(fl_get_choice_maxitems);
SUBR_charptr__objectptr(fl_get_choice_text);
SUBR_void__objectptr_int(fl_set_choice_fontsize);
SUBR_void__objectptr_int(fl_set_choice_fontstyle);
SUBR_void__objectptr_int(fl_set_choice_align);
SUBR_void__objectptr_2int(fl_set_choice_item_mode);
SUBR_void__objectptr_int_charptr(fl_set_choice_item_shortcut);
static char s_fl_set_choice_entries[]="fl_set_choice_entries";
SCM nfl_set_choice_entries(SCM _o, SCM _pelst)
{

 int i, c;
  ASSERT(OBJP(_o), _o, ARG1, s_fl_set_choice_entries);
  ASSERT(NIMP(_pelst) && CONSP(_pelst), _pelst, ARG2, s_fl_set_choice_entries);
  SFDEBUGMSG(s_fl_set_choice_entries);
  i=ilength(_pelst);
  {
    FL_PUP_ENTRY pe[i+1]; /* last entry with NULL text field */
    SCM _pe;
    for (c=0; c<i; c++, _pelst=CDR(_pelst))
      {
        _pe=CAR(_pelst);
        ASSERT(NIMP(_pe) && CONSP(_pe), _pe, ARG2, s_fl_set_choice_entries);
        ASSERT(ilength(_pe)<=4, _pe, ARG2, s_fl_set_choice_entries);
        pe[c].text=NULL;
        pe[c].callback=NULL;
        pe[c].shortcut=NULL;
	pe[c].mode=0;
        if (ilength(_pe)>=1)
          { SCM _t;
            _t=CAR(_pe); _pe=CDR(_pe);
            ASSERT((INUMP(_t) && INUM(_t)==0) || 
                    (NIMP(_t) && STRINGP(_t)), _t, ARG1, 
                   s_fl_set_choice_entries);
            if (!INUMP(_t))
              pe[c].text    =CHARS(_t);
          }
        if (ilength(_pe)>=1)
          { SCM _c;
            _c=CAR(_pe); _pe=CDR(_pe);
            ASSERT(BOOL_T==procedurep(_c), _c, ARG2, s_fl_set_choice_entries);
            cb_add(loc_choice_item_callback, MAKINUM(c), _c);
            pe[c].callback=cb_choice_item_callback;
          }
        if (ilength(_pe)>=1)
          { SCM _s;
            _s=CAR(_pe); _pe=CDR(_pe);
            ASSERT(NIMP(_s) && STRINGP(_s), _s, ARG3, s_fl_set_choice_entries);
            pe[c].shortcut=CHARS(_s);
        }
        if (ilength(_pe)==1)
          { SCM _m;
            _m=CAR(_pe); 
            ASSERT(INUMP(_m), _m, ARG4, s_fl_set_choice_entries);
            pe[c].mode    =INUM(_m);
          }
      }
    pe[i+1].text=NULL;
    return MAKINUM(fl_set_choice_entries(OBJ(_o), &pe[0]));
  }
} /* fl_set_choice_entries */

/* initialize functions */
void s_init_choice_fct()
{
  MSUBR_L(fl_create_choice);
  MSUBR_L(fl_add_choice);
  MSUBR_1(fl_clear_choice);
  MSUBR_2(fl_addto_choice);
  MSUBR_3(fl_replace_choice);
  MSUBR_2(fl_delete_choice);
  MSUBR_2(fl_set_choice);
  MSUBR_2(fl_set_choice_text);
  MSUBR_1(fl_get_choice);
  MSUBR_2(fl_get_choice_item_text);
  MSUBR_1(fl_get_choice_maxitems);
  MSUBR_1(fl_get_choice_text);
  MSUBR_2(fl_set_choice_fontsize);
  MSUBR_2(fl_set_choice_fontstyle);
  MSUBR_2(fl_set_choice_align);
  MSUBR_3(fl_set_choice_item_mode);
  MSUBR_3(fl_set_choice_item_shortcut);
  MSUBR_2(fl_set_choice_entries);
  return;
}

/*
** EOF
*/
