/*
**
** SForms : Interface to the XForms library for SCM
**
** This file is part of the SForms extension for SCM.
** (for XForms version 0.88 & SCM version 5b2 / 5c3)
**
** Author : Johannes Leveling
** eMail  : Johannes.Leveling@Informatik.Uni-Oldenburg.De
** Copyright (C) 1997, 1998 Free Software Foundation, Inc.
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this software; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

/*
** s_cursor.c
** oct 25 1998
** nov 07 1998
*/
#include "forms.h"
#include "s_macro.h"

/* initialize symbols */
void s_init_cursor_sym()
{
  /* types */
  sysintern("fl_default_cursor",   MAKINUM(FL_DEFAULT_CURSOR)); 
  sysintern("fl_invisible_cursor", MAKINUM(FL_INVISIBLE_CURSOR));   
  /* other types : X11/cursorfont.h */
  return;
}

/* routines */
SUBR_void__window_int(fl_set_cursor);
SUBR_void__3int(fl_set_cursor_color);
static char s_fl_create_bitmap_cursor[]="fl_create_bitmap_cursor";
SCM nfl_create_bitmap_cursor(SCM _s1, SCM _s2, SCM _args)
{
  SCM _i1, _i2, _i3, _i4;
  ASSERT(ilength(_args)==4, UNDEFINED, WNA, s_fl_create_bitmap_cursor);
  _i1=CAR(_args); _args=CDR(_args);
  _i2=CAR(_args); _args=CDR(_args);
  _i3=CAR(_args); _args=CDR(_args);
  _i4=CAR(_args);
  ASSERT(NIMP(_s1) && STRINGP(_s1), _s1, ARG1, s_fl_create_bitmap_cursor);
  ASSERT(NIMP(_s2) && STRINGP(_s2), _s2, ARG2, s_fl_create_bitmap_cursor);
  ASSERT(INUMP(_i1), _i1, ARG3, s_fl_create_bitmap_cursor);
  ASSERT(INUMP(_i2), _i2, ARG4, s_fl_create_bitmap_cursor);
  ASSERT(INUMP(_i3), _i3, ARG5, s_fl_create_bitmap_cursor);
  ASSERT(INUMP(_i4), _i4, ARGn, s_fl_create_bitmap_cursor);
  SFDEBUGMSG(s_fl_create_bitmap_cursor);
  return MAKINUM(fl_create_bitmap_cursor(CHARS(_s1), CHARS(_s2), 
					 INUM(_i1), INUM(_i2), 
					 INUM(_i3), INUM(_i4)));
} /* fl_create_bitmap_cursor */
static char s_fl_create_animated_cursor[]="fl_create_animated_cursor";
SCM nfl_create_animated_cursor(SCM _il, SCM _i)
{
  int i, c;
  ASSERT(NIMP(_il) && CONSP(_il), _il, ARG1, s_fl_create_animated_cursor);
  ASSERT(INUMP(_i), _i, ARG2, s_fl_create_animated_cursor);
  SFDEBUGMSG(s_fl_create_animated_cursor);
  i=ilength(_il);
  {
    int il[i];
    for (c=0; c<i; c++, _il=CDR(_il))
      { 
	ASSERT(INUMP(CAR(_il)), CAR(_il), ARGn, s_fl_create_animated_cursor);
	il[c]=INUM(CAR(_il)); 
      }
    return MAKINUM(fl_create_animated_cursor(&il[0], INUM(_i)));
  }
} /* fl_create_animated_cursor */
SUBR_int__int(fl_get_cursor_byname); /* Cursor is unsigned long */

/* initialize functions */
void s_init_cursor_fct()
{
  MSUBR_2(fl_set_cursor);
  MSUBR_3(fl_set_cursor_color);
  MSUBR_L(fl_create_bitmap_cursor);
  MSUBR_2(fl_create_animated_cursor);
  MSUBR_1(fl_get_cursor_byname);
  return;
}

/*
** EOF
*/
