/*
**
** SForms : Interface to the XForms library for SCM
**
** This file is part of the SForms extension for SCM.
** (for XForms version 0.88 & SCM version 5b2 / 5c3)
**
** Author : Johannes Leveling
** eMail  : Johannes.Leveling@Informatik.Uni-Oldenburg.De
** Copyright (C) 1997, 1998 Free Software Foundation, Inc.
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this software; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

/*
** s_extra.c
** Following functions are NOT explicitly supported 
** by XForms (in spite of fl_ prefix).
** oct 26 1998 : OK
*/
#include "forms.h"
#include "s_macro.h"

/* initialize symbols */
void s_init_extra_sym()
{
  /* */
  return;
}

/* routines */

/* addtional routines for forms structure */
static char s_fl_get_form_fdui[]="fl_get_form_fdui";
SCM nfl_get_form_fdui(SCM _f)
{
  ASSERT(FORMP(_f), _f, ARG1, s_fl_get_form_fdui);
  SFDEBUGMSG(s_fl_get_form_fdui);
  if (FORM(_f)->fdui!=NULL)
    return (SCM)FORM(_f)->fdui;
  return MAKINUM(0);
}
static char s_fl_set_form_fdui[]="fl_set_form_fdui";
SCM nfl_set_form_fdui(SCM _f, SCM _v)
{
  ASSERT(FORMP(_f), _f, ARG1, s_fl_set_form_fdui);
  SFDEBUGMSG(s_fl_set_form_fdui);
  FORM(_f)->fdui=(void *)_v;
  return UNSPECIFIED;
}
static char s_fl_get_form_vdata[]="fl_get_form_vdata";
SCM nfl_get_form_vdata(SCM _f) 
{ /* u_vdata */
  ASSERT(FORMP(_f), _f, ARG1, s_fl_get_form_vdata);
  SFDEBUGMSG(s_fl_get_form_vdata);
  if (FORM(_f)->u_vdata!=NULL)
    return (SCM)FORM(_f)->u_vdata;
  return MAKINUM(0);
} 
static char s_fl_set_form_vdata[]="fl_set_form_vdata";
SCM nfl_set_form_vdata(SCM _f, SCM _v)
{ /* u_vdata */
  ASSERT(FORMP(_f), _f, ARG1, s_fl_set_form_vdata);
  SFDEBUGMSG(s_fl_set_form_vdata);
  FORM(_f)->u_vdata=(void *)_v;
  return UNSPECIFIED;
}
static char s_fl_get_form_cdata[]="fl_get_form_cdata";
SCM nfl_get_form_cdata(SCM _f)
{ /* u_cdata */
  ASSERT(FORMP(_f), _f, ARG1, s_fl_get_form_cdata);
  SFDEBUGMSG(s_fl_get_form_cdata);
  return MAKSTR(FORM(_f)->u_cdata);
}
static char s_fl_set_form_cdata[]="fl_set_form_cdata";
SCM nfl_set_form_cdata(SCM _f, SCM _s)
{ /* u_cdata */
  ASSERT(FORMP(_f), _f, ARG1, s_fl_set_form_cdata);
  ASSERT(NIMP(_s) && STRINGP(_s), _s, ARG2, s_fl_set_form_cdata);
  SFDEBUGMSG(s_fl_set_form_cdata);
  FORM(_f)->u_cdata=CHARS(_s);
  return UNSPECIFIED;
}
static char s_fl_get_form_ldata[]="fl_get_form_ldata";
SCM nfl_get_form_ldata(SCM _f)
{
  ASSERT(FORMP(_f), _f, ARG1, s_fl_get_form_ldata);
  SFDEBUGMSG(s_fl_get_form_ldata);
  return long2num(FORM(_f)->u_ldata);
}
static char s_fl_set_form_ldata[]="fl_set_form_ldata";
SCM nfl_set_form_ldata(SCM _f, SCM _l)
{
  ASSERT(FORMP(_f), _f, ARG1, s_fl_set_form_ldata);
  ASSERT(INUMP(_l), _l, ARG2, s_fl_set_form_ldata);
  SFDEBUGMSG(s_fl_set_form_ldata);
  FORM(_f)->u_ldata=INUM(_l);
  return UNSPECIFIED;
}
static char s_fl_get_form_size[]="fl_get_form_size";
SCM nfl_get_form_size(SCM _f)
{
  ASSERT(FORMP(_f), _f, ARG1, s_fl_get_form_size);
  SFDEBUGMSG(s_fl_get_form_size);
  return cons(MAKINUM(FORM(_f)->w), 
	      cons(MAKINUM(FORM(_f)->h), EOL));
}
static char s_fl_get_form_geometry[]="fl_get_form_geometry";
SCM nfl_get_form_geometry(SCM _f)
{
  ASSERT(FORMP(_f), _f, ARG1, s_fl_get_form_geometry);
  SFDEBUGMSG(s_fl_get_form_geometry);
  return cons(MAKINUM(FORM(_f)->x), 
	      cons(MAKINUM(FORM(_f)->y), 
		   cons(MAKINUM(FORM(_f)->w), 
			cons(MAKINUM(FORM(_f)->h), EOL))));
}
static char s_fl_get_form_position[]="fl_get_form_position";
SCM nfl_get_form_position(SCM _f)
{
  ASSERT(FORMP(_f), _f, ARG1, s_fl_get_form_position);
  SFDEBUGMSG(s_fl_get_form_position);
  return cons(MAKINUM(FORM(_f)->x), 
	      cons(MAKINUM(FORM(_f)->y), EOL));
}
static char s_fl_get_form_visible[]="fl_get_form_visible";
SCM nfl_get_form_visible(SCM _f)
{
  ASSERT(FORMP(_f), _f, ARG1, s_fl_get_form_visible);
  SFDEBUGMSG(s_fl_get_form_visible);
  return MAKINUM(FORM(_f)->visible);
}

/* additional routines for object structure */
static char s_fl_get_object_form[]="fl_get_object_form";
SCM nfl_get_object_form(SCM _o) 
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_form);
  SFDEBUGMSG(s_fl_get_object_form);
  return mkform(OBJ(_o)->form);
}
/* no setter function (fl_set_object_form) ! */
static char s_fl_get_object_vdata[]="fl_get_object_vdata";
SCM nfl_get_object_vdata(SCM _o)
{ /* u_vdata */
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_vdata);
  SFDEBUGMSG(s_fl_get_object_vdata);
  if (OBJ(_o)->u_vdata!=NULL)
    return (SCM)OBJ(_o)->u_vdata;
  return MAKINUM(0);
}
static char s_fl_set_object_vdata[]="fl_set_object_vdata";
SCM nfl_set_object_vdata(SCM _o, SCM _v)
{ /* u_vdata */
  ASSERT(OBJP(_o), _o, ARG1, s_fl_set_object_vdata);
  SFDEBUGMSG(s_fl_set_object_vdata);
  OBJ(_o)->u_vdata=(void *)_v;
  return UNSPECIFIED;
}
static char s_fl_get_object_cdata[]="fl_get_object_cdata";
SCM nfl_get_object_cdata(SCM _o)
{ /* u_cdata */
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_cdata);
  SFDEBUGMSG(s_fl_get_object_cdata);
  return MAKSTR(OBJ(_o)->u_cdata);
}
static char s_fl_set_object_cdata[]="fl_set_object_cdata";
SCM nfl_set_object_cdata(SCM _o, SCM _s)
{ /* u_cdata */
  ASSERT(OBJP(_o), _o, ARG1, s_fl_set_object_cdata);
  ASSERT(NIMP(_s) && STRINGP(_s), _s, ARG2, s_fl_set_object_cdata);
  SFDEBUGMSG(s_fl_set_object_cdata);
  OBJ(_o)->u_cdata=CHARS(_s);
  return UNSPECIFIED;
}
static char s_fl_get_object_ldata[]="fl_get_object_ldata";
SCM nfl_get_object_ldata(SCM _o)
{ /* u_ldata */
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_ldata);
  SFDEBUGMSG(s_fl_get_object_ldata);
  return long2num(OBJ(_o)->u_ldata);
}
static char s_fl_set_object_ldata[]="fl_set_object_ldata";
SCM nfl_set_object_ldata(SCM _o, SCM _l)
{ /* u_ldata */
  ASSERT(OBJP(_o), _o, ARG1, s_fl_set_object_ldata);
  ASSERT(INUMP(_l), _l, ARG2, s_fl_set_object_ldata);
  SFDEBUGMSG(s_fl_set_object_ldata);
  OBJ(_o)->u_ldata=INUM(_l);
  return UNSPECIFIED;
}
static char s_fl_set_object_wantkey[]="fl_set_object_wantkey";
SCM nfl_set_object_wantkey(SCM _o, SCM _i)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_set_object_wantkey);
  ASSERT(INUMP(_i), _i, ARG2, s_fl_set_object_wantkey);
  SFDEBUGMSG(s_fl_set_object_wantkey);
  OBJ(_o)->wantkey=INUM(_i);
  return UNSPECIFIED;
}
static char s_fl_set_object_input[]="fl_set_object_input";
SCM nfl_set_object_input(SCM _o, SCM _i)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_set_object_input);
  ASSERT(INUMP(_i), _i, ARG2, s_fl_set_object_input);
  SFDEBUGMSG(s_fl_set_object_input);
  OBJ(_o)->input=INUM(_i);
  return UNSPECIFIED;
}
static char s_fl_set_object_radio[]="fl_set_object_radio";
SCM nfl_set_object_radio(SCM _o, SCM _i)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_set_object_radio);
  ASSERT(INUMP(_i), _i, ARG2, s_fl_set_object_radio);
  SFDEBUGMSG(s_fl_set_object_radio);
  OBJ(_o)->radio=INUM(_i);
  return UNSPECIFIED;
}
static char s_fl_get_object_lsize[]="fl_get_object_lsize";
SCM nfl_get_object_lsize(SCM _o)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_lsize);
  SFDEBUGMSG(s_fl_get_object_lsize);
  return MAKINUM(OBJ(_o)->lsize);
}
static char s_fl_get_object_lstyle[]="fl_get_object_lstyle";
SCM nfl_get_object_lstyle(SCM _o)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_lstyle);
  SFDEBUGMSG(s_fl_get_object_lstyle);
  return MAKINUM(OBJ(_o)->lstyle);
}
static char s_fl_get_object_type[]="fl_get_object_type";
SCM nfl_get_object_type(SCM _o)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_type);
  SFDEBUGMSG(s_fl_get_object_type);
  return MAKINUM(OBJ(_o)->type);
}
static char s_fl_get_object_class[]="fl_get_object_class";
SCM nfl_get_object_class(SCM _o)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_class);
  SFDEBUGMSG(s_fl_get_object_class);
  return MAKINUM(OBJ(_o)->objclass);
}
static char s_fl_get_object_label[]="fl_get_object_label";
SCM nfl_get_object_label(SCM _o)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_label);
  SFDEBUGMSG(s_fl_get_object_label);
  return MAKSTR(OBJ(_o)->label);
}
static char s_fl_get_object_visible[]="fl_get_object_visible";
SCM nfl_get_object_visible(SCM _o)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_visible);
  SFDEBUGMSG(s_fl_get_object_visible);
  return MAKINUM(OBJ(_o)->visible);
}
static char s_fl_get_object_active[]="fl_get_object_active";
SCM nfl_get_object_active(SCM _o)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_active);
  SFDEBUGMSG(s_fl_get_object_active);
  return MAKINUM(OBJ(_o)->active);
}
static char s_fl_get_object_pushed[]="fl_get_object_pushed";
SCM nfl_get_object_pushed(SCM _o)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_pushed);
  SFDEBUGMSG(s_fl_get_object_pushed);
  return MAKINUM(OBJ(_o)->pushed);
}
static char s_fl_get_object_color[]="fl_get_object_color";
SCM nfl_get_object_color(SCM _o)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_color);
  SFDEBUGMSG(s_fl_get_object_color);
  return cons(MAKINUM(OBJ(_o)->col1), 
	      cons(MAKINUM(OBJ(_o)->col2), EOL));
}
static char s_fl_get_object_lcol[]="fl_get_object_lcol";
SCM nfl_get_object_lcol(SCM _o)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_lcol);
  SFDEBUGMSG(s_fl_get_object_lcol);
  return MAKINUM(OBJ(_o)->lcol);
}
static char s_fl_get_object_bw[]="fl_get_object_bw";
SCM nfl_get_object_bw(SCM _o)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_bw);
  SFDEBUGMSG(s_fl_get_object_bw);
  return MAKINUM(OBJ(_o)->bw);
}
static char s_fl_get_object_align[]="fl_get_object_align";
SCM nfl_get_object_align(SCM _o)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_align);
  SFDEBUGMSG(s_fl_get_object_align);
  return MAKINUM(OBJ(_o)->align);
}

static char s_fl_get_object_boxtype[]="fl_get_object_boxtype";
SCM nfl_get_object_boxtype(SCM _o)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_boxtype);
  SFDEBUGMSG(s_fl_get_object_boxtype);
  return MAKINUM(OBJ(_o)->boxtype);
}
static char s_fl_get_object_size[]="fl_get_object_size";
SCM nfl_get_object_size(SCM _o)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_size);
  SFDEBUGMSG(s_fl_get_object_size);
  return cons(MAKINUM(OBJ(_o)->w), 
	      cons(MAKINUM(OBJ(_o)->h), EOL));
}
static char s_fl_get_object_window[]="fl_get_object_window";
SCM nfl_get_object_window(SCM _o) 
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_object_window);
  SFDEBUGMSG(s_fl_get_object_window);
  return mkwin(OBJ(_o)->form->window);
}

/* from XForms documentation */
static char s_fl_flush[]="fl_flush";
SCM nfl_flush()
{
  SFDEBUGMSG(s_fl_flush);
  XFlush(fl_get_display());
  return UNSPECIFIED;
}
static char s_fl_update[]="fl_update";
SCM nfl_update()
{
  SFDEBUGMSG(s_fl_update);
  XSync(fl_get_display(), 0);
  return UNSPECIFIED;
}
static char s_fl_clear_window[]="fl_clear_window";
SCM nfl_clear_window(SCM _d, SCM _w)
{
  ASSERT(DISP(_d), _d, ARG1, s_fl_clear_window);
  ASSERT(WINP(_w), _w, ARG2, s_fl_clear_window);
  SFDEBUGMSG(s_fl_clear_window);
  XClearWindow(DIS(_d), WIN(_w));
  return UNSPECIFIED;
}
static char s_fl_get_button_spec[]="fl_get_button_spec";
SCM nfl_get_button_spec(SCM _o)
{
  FL_BUTTON_SPEC *bs;
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_button_spec);
  SFDEBUGMSG(s_fl_get_button_spec);  
  bs=(FL_BUTTON_SPEC*)OBJ(_o)->spec;
  if (bs==NULL)
    return EOL;
  return 
    cons
    (mkpix(bs->pixmap), 
     cons
     (mkpix(bs->mask), 
      cons(MAKINUM(bs->bits_w), 
	   cons(MAKINUM(bs->bits_h), 
		cons(MAKINUM(bs->val),
		     cons(MAKINUM(bs->mousebut),
			  cons(MAKINUM(bs->timdel),
			       cons(MAKINUM(bs->event),
				    cons(long2num(bs->cspecl), 
					 cons(MAKINUM((int)bs->cspecv), 
					      cons(MAKSTR(bs->filename),
						   EOL)))))))))));
}

static char s_fl_get_pixel_color[]="fl_get_pixel_color";
SCM nfl_get_pixel_color(SCM _w, SCM _i1, SCM _i2)
{
  ASSERT(WINP(_w), _w, ARG1, s_fl_get_pixel_color);
  ASSERT(INUMP(_i1), _i1, ARG2, s_fl_get_pixel_color);
  ASSERT(INUMP(_i2), _i2, ARG3, s_fl_get_pixel_color);
  SFDEBUGMSG(s_fl_get_pixel_color);
  { 
    XImage *xim;
    unsigned long dot;
    xim=XGetImage(fl_display, WIN(_w), INUM(_i1), INUM(_i2), 
		  1, 1, AllPlanes, XYPixmap); 
    dot=XGetPixel(xim, 0, 0);
    return ulong2num(dot);
  }
}
  
/* initialize functions */
void s_init_extra_fct()
{
  MSUBR_1(fl_get_form_fdui);
  MSUBR_2(fl_set_form_fdui);
  MSUBR_1(fl_get_form_vdata);
  MSUBR_2(fl_set_form_vdata);
  MSUBR_1(fl_get_form_cdata);
  MSUBR_2(fl_set_form_cdata);
  MSUBR_1(fl_get_form_ldata);
  MSUBR_2(fl_set_form_ldata);
  MSUBR_1(fl_get_form_size); /* same as (fl_winsize (fl_winget)) */
  MSUBR_1(fl_get_form_geometry);
  MSUBR_1(fl_get_form_position);
  MSUBR_1(fl_get_form_visible);
  /* */
  MSUBR_1(fl_get_object_form);
  MSUBR_1(fl_get_object_vdata);
  MSUBR_2(fl_set_object_vdata);
  MSUBR_1(fl_get_object_cdata);
  MSUBR_2(fl_set_object_cdata);
  MSUBR_1(fl_get_object_ldata);
  MSUBR_2(fl_set_object_ldata);
  MSUBR_2(fl_set_object_wantkey);
  MSUBR_2(fl_set_object_input);
  MSUBR_2(fl_set_object_radio);
  MSUBR_1(fl_get_object_lsize);
  MSUBR_1(fl_get_object_lstyle);
  MSUBR_1(fl_get_object_type);
  MSUBR_1(fl_get_object_class);
  MSUBR_1(fl_get_object_label);
  MSUBR_1(fl_get_object_visible);
  MSUBR_1(fl_get_object_active);
  MSUBR_1(fl_get_object_pushed);
  MSUBR_1(fl_get_object_color); /* colors */
  MSUBR_1(fl_get_object_lcol);
  MSUBR_1(fl_get_object_bw);
  MSUBR_1(fl_get_object_align);
  MSUBR_1(fl_get_object_boxtype);
  MSUBR_1(fl_get_object_size);
  MSUBR_1(fl_get_object_window);
  /* */
  MSUBR_0(fl_flush);
  MSUBR_0(fl_update);
  MSUBR_2(fl_clear_window);
  MSUBR_1(fl_get_button_spec);
  MSUBR_3(fl_get_pixel_color);
  return;
}

/*
** EOF
*/
