/*
**
** SForms : Interface to the XForms library for SCM
**
** This file is part of the SForms extension for SCM.
** (for XForms version 0.88 & SCM version 5b2 / 5c3)
**
** Author : Johannes Leveling
** eMail  : Johannes.Leveling@Informatik.Uni-Oldenburg.De
** Copyright (C) 1997, 1998 Free Software Foundation, Inc.
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this software; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

/*
** s_filesys.c
** oct 25 1998 
** nov 09 1998 
*/
#include "forms.h"
#include "s_macro.h"

/* convenience functions to read a directory */

/* initialize symbols */
void s_init_filesys_sym()
{
  /* file types */
  sysintern("ft_file",       MAKINUM(FT_FILE)); 
  sysintern("ft_dir",        MAKINUM(FT_DIR));
  sysintern("ft_link",       MAKINUM(FT_LINK)); 
  sysintern("ft_sock",       MAKINUM(FT_SOCK));
  sysintern("ft_fifo",       MAKINUM(FT_FIFO)); 
  sysintern("ft_blk",        MAKINUM(FT_BLK)); 
  sysintern("ft_chr",        MAKINUM(FT_CHR)); 
  sysintern("ft_other",      MAKINUM(FT_OTHER));
  /* sorting */
  sysintern("fl_alphasort",  MAKINUM(FL_ALPHASORT));
  sysintern("fl_ralphasort", MAKINUM(FL_RALPHASORT));
  sysintern("fl_mtimesort",  MAKINUM(FL_MTIMESORT));
  sysintern("fl_rmtimesort", MAKINUM(FL_RMTIMESORT));
  sysintern("fl_sizesort",   MAKINUM(FL_SIZESORT));
  sysintern("fl_rsizesort",  MAKINUM(FL_RSIZESORT));
  return;
}

/* routines */

/* read dir with pattern filtering */
static char s_fl_get_dirlist[]="fl_get_dirlist";
SCM nfl_get_dirlist(SCM _s1, SCM _s2, SCM _i)
{
  int n;
  const FL_Dirlist *dl=fl_get_dirlist(CHARS(_s1), CHARS(_s2), 
				      &n, INUM(_i)), *ds;
  const FL_Dirlist *dlend=dl+n;
  SCM sdl, sde;
  /* */
  ASSERT(NIMP(_s1) && STRINGP(_s1), _s1, ARG1, s_fl_get_dirlist);
  ASSERT(NIMP(_s2) && STRINGP(_s2), _s2, ARG2, s_fl_get_dirlist);
  ASSERT(INUMP(_i), _i, ARG3, s_fl_get_dirlist);
  SFDEBUGMSG(s_fl_get_dirlist);
  sdl=EOL;
  for (ds=dl; dl<dlend; dl++)
    {
      sde=cons(MAKSTR(dl->name), 
	       cons(MAKINUM(dl->type), 
		    cons(long2num(dl->dl_mtime), 
			 cons(ulong2num(dl->dl_size), 
			      EOL)))); /* filler[3] ignored */
      sdl=cons(sde, sdl);
    }
  fl_free_dirlist((FL_Dirlist*)ds);
  return reverse(sdl);
} /* fl_get_dirlist */
static char s_fl_set_dirlist_filter[]="fl_set_dirlist_filter";
SCM nfl_set_dirlist_filter(SCM _proc)
{
  ASSERT(BOOL_T==procedurep(_proc), _proc, ARG1, s_fl_set_dirlist_filter);
  SFDEBUGMSG(s_fl_set_dirlist_filter);
  cb_add(loc_dirlist_filter, MAKINUM(0), _proc);
  fl_set_dirlist_filter(cb_dirlist_filter);
  return UNSPECIFIED;
} /* fl_set_dirlist_filter */
SUBR_void__int(fl_set_dirlist_sort);
SUBR_dummy1(fl_free_dirlist);
/* free all directory caches */
SUBR_void__void(fl_free_all_dirlist);
SUBR_int__charptr(fl_is_valid_dir);
static char s_fl_fmtime[]="fl_fmtime";
SCM nfl_fmtime(SCM _s)
{
  ASSERT(NIMP(_s), _s, ARG1, s_fl_fmtime);
  SFDEBUGMSG(s_fl_fmtime);
  return ulong2num(fl_fmtime(CHARS(_s)));
} /* fl_fmtime */
SUBR_charptr__charptr(fl_fix_dirname);

/* initialize functions */
void s_init_filesys_fct()
{
  MSUBR_3(fl_get_dirlist);
  MSUBR_1(fl_set_dirlist_filter);
  MSUBR_1(fl_set_dirlist_sort);
  MSUBR_1(fl_free_dirlist);
  MSUBR_0(fl_free_all_dirlist);
  MSUBR_1(fl_is_valid_dir);
  MSUBR_1(fl_fmtime);
  MSUBR_1(fl_fix_dirname);
  return;
}

/*
** EOF
*/
