/*
**
** SForms : Interface to the XForms library for SCM
**
** This file is part of the SForms extension for SCM.
** (for XForms version 0.88 & SCM version 5b2 / 5c3)
**
** Author : Johannes Leveling
** eMail  : Johannes.Leveling@Informatik.Uni-Oldenburg.De
** Copyright (C) 1997, 1998 Free Software Foundation, Inc.
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this software; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

/*
** s_forms.c
** nov 26 1998
*/
#include "forms.h"

/* specify initialization file (alias definitions) */
static char s_sforms_file[]="/usr/local/sforms-0.88b/SForms.scm";
/*static char s_sforms_file[]="SForms.scm";*/

/* to compile SForms extension for GUILE, uncomment the following */
/* "#define FOR_GUILE" statement in the file "s_macro.h" */

/* uncomment the following line to enable debug messages */
/*#define SFDEBUG */

#include "s_macro.h"
#include "s_forms.h"
/* */

void SFDEBUGMSG(char *name)
{ /* optimized out if SFDEBUG is not defined */
#ifdef SFDEBUG                   
  fprintf(stderr, "; %s\n", name); 
#endif
  return;
}

void init_sforms_banner()
{
  fputs("SForms (XForms) X11 toolkit extension 0.88b\n\
for SCM / GUILE.\n\
Copyright (C) 1997, 1998 Free Software Foundation, Inc.\n\
SForms comes with NO WARRANTY, to the extent permitted by law.\n\
You may redistribute copies of SForms under the terms of the\n\
GNU General Public License.\n\
For more information about these matters, see the file named COPYING.\n", 
	stdout); 
  return;
}

/* for new types */
long tc16_form;
long tc16_obj;
long tc16_win;
long tc16_pix;
long tc16_dis;
long tc16_xev;
long tc16_gc;
long tc16_fnt;

/* statistics */
int nyi=0;   /* not yet implemented */
int yi =0;   /* implemented (functions) */
int di=0;    /* dummies */


SCM *loc_io_callback;
SCM *loc_signal_callback;
SCM *loc_timeout_handler;
SCM *loc_form_atclose_callback;
SCM *loc_atclose_callback;
SCM *loc_form_atdeactivate_callback;
SCM *loc_form_atactivate_callback;
SCM *loc_form_callback;
SCM *loc_raw_callback;
SCM *loc_object_prehandler;
SCM *loc_object_posthandler;
SCM *loc_object_callback;
SCM *loc_font_enumerator;
SCM *loc_symbol_handler;
SCM *loc_error_handler;
SCM *loc_new_object_handler;
SCM *loc_browser_dblclick_callback;
SCM *loc_draw_button;
SCM *loc_cleanup_button;
SCM *loc_canvas_handler;
SCM *loc_canvas_modify1;
SCM *loc_canvas_modify2;
SCM *loc_canvas_modify3;
SCM *loc_choice_item_callback;
SCM *loc_stuff_clipboard_callback;
SCM *loc_request_clipboard_callback;
SCM *loc_counter_filter;
SCM *loc_dirlist_filter;
SCM *loc_free_handler;
SCM *loc_fselector_callback;
SCM *loc_fselector_appbutton;
SCM *loc_input_filter;
SCM *loc_menu_item_callback;
SCM *loc_menubar_item_callback;
SCM *loc_pup_item_callback;
SCM *loc_pup_enter_callback;
SCM *loc_pup_leave_callback;
SCM *loc_pup_menu_callback;
SCM *loc_slider_filter;
SCM *loc_timer_filter;
SCM *loc_xyplot_symbol_handler;
SCM *loc_event_callback;
SCM *loc_idle_callback;
SCM *loc_sevent_callback;


/* initialize callbacks, filter, handler, ... */
void init_fl_callbacks()
{
#ifdef SFDEBUG
  fputs(";  Initializing XForms callbacks...\n", stderr);
#endif
  /* */
  loc_io_callback     = &CDR(sysintern("*fl_io_callback*", EOL));
  loc_signal_callback = &CDR(sysintern("*fl_signal_callback*", EOL));
  loc_timeout_handler = &CDR(sysintern("*fl_timeout_handler*", EOL));
  loc_form_atclose_callback= 
    &CDR(sysintern("*fl_form_atclose_callback*", EOL));
  loc_atclose_callback= 
    &CDR(sysintern("*fl_atclose_callback*", EOL));
  loc_form_atdeactivate_callback= 
    &CDR(sysintern("*fl_form_atdeactivate_callback*", EOL));
  loc_form_atactivate_callback= 
    &CDR(sysintern("*fl_form_atactivate_callback*", EOL));
  loc_form_callback   = &CDR(sysintern("*fl_form_callback*", EOL));
  loc_raw_callback    = &CDR(sysintern("*fl_raw_callback*", EOL));
  loc_object_prehandler= 
    &CDR(sysintern("*fl_object_prehandler*", EOL));
  loc_object_posthandler= 
    &CDR(sysintern("*fl_object_posthandler*", EOL));
  loc_object_callback = &CDR(sysintern("*fl_object_callback*", EOL));
  loc_font_enumerator = &CDR(sysintern("*fl_font_enumerator*", EOL));
  loc_symbol_handler  = &CDR(sysintern("*fl_symbol_handler*", EOL));
  loc_error_handler   = &CDR(sysintern("*fl_error_handler*", EOL));
  loc_new_object_handler= 
    &CDR(sysintern("*fl_new_object_handler*", EOL));
  /* */
  loc_browser_dblclick_callback= 
    &CDR(sysintern("*fl_browser_dblclick_callback*", EOL));
  loc_draw_button     = &CDR(sysintern("*fl_draw_button*", EOL));
  loc_cleanup_button  = &CDR(sysintern("*fl_cleanup_button*", EOL));
  loc_canvas_handler  = &CDR(sysintern("*fl_canvas_handler*", EOL));
  loc_canvas_modify1  = &CDR(sysintern("*fl_canvas_modify1*", EOL));
  loc_canvas_modify2  = &CDR(sysintern("*fl_canvas_modify2*", EOL));
  loc_canvas_modify3  = &CDR(sysintern("*fl_canvas_modify3*", EOL));
  loc_choice_item_callback= 
    &CDR(sysintern("*fl_choice_item_callback*", EOL));
  loc_stuff_clipboard_callback= 
    &CDR(sysintern("*fl_stuff_clipboard*", EOL));
  loc_request_clipboard_callback= 
    &CDR(sysintern("*fl_request_clipboard*", EOL));
  loc_counter_filter  = &CDR(sysintern("*fl_counter_filter*", EOL));
  loc_dirlist_filter  = &CDR(sysintern("*fl_dirlist_filter*", EOL));
  loc_free_handler    = &CDR(sysintern("*fl_free_handler*", EOL));
  loc_fselector_callback= 
    &CDR(sysintern("*fl_fselector_callback*", EOL));
  loc_fselector_appbutton= 
    &CDR(sysintern("*fl_fselector_appbutton*", EOL));
  loc_input_filter    = &CDR(sysintern("*fl_input_filter*", EOL));  
  loc_menu_item_callback= 
    &CDR(sysintern("*fl_menu_item_callback*", EOL));
  loc_menubar_item_callback= 
    &CDR(sysintern("*fl_menubar_item_callback*", EOL));
  loc_pup_item_callback= 
    &CDR(sysintern("*fl_pup_item_callback*", EOL));
  loc_pup_enter_callback= 
    &CDR(sysintern("*fl_pup_enter_callback*", EOL));
  loc_pup_leave_callback= 
    &CDR(sysintern("*fl_pup_leave_callback*", EOL));
  loc_pup_menu_callback= 
    &CDR(sysintern("*fl_pup_menu_callback*", EOL));
  loc_slider_filter   = &CDR(sysintern("*fl_slider_filter*", EOL));
  loc_timer_filter    = &CDR(sysintern("*fl_timer_filter*", EOL));
  loc_xyplot_symbol_handler=
    &CDR(sysintern("*fl_xyplot_symbol_handler*", EOL));
  loc_event_callback  = &CDR(sysintern("*fl_event_callback*", EOL));
  loc_idle_callback   = &CDR(sysintern("*fl_idle_callback*", EOL));
  loc_sevent_callback = &CDR(sysintern("*fl_sevent_callback*", EOL));
  /* */
#ifdef SFDEBUG
  fputs(";  ...done\n", stderr);
#endif
  return;
}

/* initialize symbols */
void init_fl_symbols()
{
#ifdef SFDEBUG
  fputs(";  Initializing XForms symbols...\n", stderr);
#endif
  /* */
  sysintern("fl_version",         MAKINUM(FL_VERSION));
  sysintern("fl_revision",        MAKINUM(FL_REVISION));
  sysintern("fl_fixlevel",        MAKINUM(FL_FIXLEVEL)); 
  sysintern("fl_include_version", MAKINUM(FL_INCLUDE_VERSION)); 
  /* */
  /* */
  /* */
  s_init_basic_sym();
  s_init_bitmap_sym();
  s_init_box_sym();
  s_init_browser_sym();
  s_init_button_sym();
  s_init_canvas_sym();
  s_init_chart_sym();
  s_init_choice_sym();
  s_init_clipbd_sym();
  s_init_clock_sym();
  s_init_counter_sym();
  s_init_cursor_sym();
  s_init_dial_sym();
  s_init_filesys_sym();
  s_init_flps_sym();
  s_init_frame_sym();
  s_init_free_sym();
  s_init_goodies_sym();
  s_init_input_sym();
  s_init_menu_sym();
#ifdef fl_create_menubar
  s_init_menubar_sym();
#endif
  s_init_popup_sym();
  s_init_positioner_sym();
  s_init_scrollbar_sym();
  s_init_slider_sym();
  s_init_folder_sym();
  s_init_text_sym();
  s_init_timer_sym();
  s_init_xbasic_sym();
  s_init_xyplot_sym();
  s_init_extra_sym();
  s_init_event_sym();
#ifdef SFDEBUG
  fputs(";  ...done\n", stderr);
#endif
  return;
}

/* global "variables" */
static char s_fl_event[]="fl_event";
SCM nfl_event()   {return mkobj(FL_EVENT);}
static char s_fl_current_form[]="fl_current_form";
SCM nfl_current_form() {return mkform(fl_current_form);}
static char s_fl_display[]="fl_display";
SCM nfl_display() {return mkdis(fl_display);}
static char s_fl_screen[]="fl_screen";
SCM nfl_screen()  {return MAKINUM(fl_screen);}
static char s_fl_root[]="fl_root";
SCM nfl_root()    {return mkwin(fl_root);}
static char s_fl_vroot[]="fl_vroot";
SCM nfl_vroot()   {return mkwin(fl_vroot);}
static char s_fl_scrh[]="fl_scrh";
SCM nfl_scrh()    {return MAKINUM(fl_scrh);}
static char s_fl_scrw[]="fl_scrw";
SCM nfl_scrw()    {return MAKINUM(fl_scrw);}
static char s_fl_vmode[]="fl_vmode";
SCM nfl_vmode()   {return MAKINUM(fl_vmode);}
static char s_fl_state[]="fl_state";
SCM nfl_state()   {return UNDEFINED;}          
static char s_fl_ul_magic_char[]="fl_ul_magic_char";
SCM nfl_ul_magic_char() {return makfrom0str(fl_ul_magic_char);}
static char s_fl_textgc[]="fl_textgc";
SCM nfl_textgc()  {return mkxgc(fl_textgc);}
/* */
static char s_fl_formp[]="fl_form?", 
  s_fl_objectp[]="fl_object?", 
  s_fl_windowp[]="fl_window?",
  s_fl_pixmapp[]="fl_pixmap",
  s_fl_displayp[]="fl_display?",
  s_fl_xeventp[]="fl_xevent?", 
  s_fl_gcp[]="fl_gc?", 
  s_fl_fontstructp[]="fl_fontstruct?";

SCM nfl_formp(SCM _o)    {return (FORMP(_o) ? BOOL_T : BOOL_F);}
SCM nfl_objectp(SCM _o)  {return (OBJP(_o) ? BOOL_T : BOOL_F);}
SCM nfl_windowp(SCM _o)  {return (WINP(_o) ? BOOL_T : BOOL_F);}
SCM nfl_pixmapp(SCM _o)  {return (PIXP(_o) ? BOOL_T : BOOL_F);}
SCM nfl_displayp(SCM _o) {return (DISP(_o) ? BOOL_T : BOOL_F);}
SCM nfl_xeventp(SCM _o)  {return (XEVP(_o) ? BOOL_T : BOOL_F);}
SCM nfl_gcp(SCM _o)      {return (SGCP(_o) ? BOOL_T : BOOL_F);}
SCM nfl_fontstructp(SCM _o) {return (FNTP(_o) ? BOOL_T : BOOL_F);}


/* initialize functions */
void init_fl_functions()
{
#ifdef SFDEBUG
  fputs(";  Initializing XForms functions...\n", stderr);
#endif
  /* */
  MSUBR_0(fl_event);
  MSUBR_0(fl_current_form);
  MSUBR_0(fl_display);
  MSUBR_0(fl_screen);
  MSUBR_0(fl_root);
  MSUBR_0(fl_vroot);
  MSUBR_0(fl_scrh);
  MSUBR_0(fl_scrw);
  MSUBR_0(fl_vmode);
  MSUBR_0(fl_state);
  MSUBR_0(fl_ul_magic_char);
  MSUBR_0(fl_textgc);
  /* */
  make_subr(s_fl_formp,    tc7_subr_1, nfl_formp);
  make_subr(s_fl_objectp,  tc7_subr_1, nfl_objectp);
  make_subr(s_fl_windowp,  tc7_subr_1, nfl_windowp);
  make_subr(s_fl_pixmapp,  tc7_subr_1, nfl_pixmapp);
  make_subr(s_fl_displayp, tc7_subr_1, nfl_displayp);
  make_subr(s_fl_xeventp,  tc7_subr_1, nfl_xeventp);
  make_subr(s_fl_gcp,      tc7_subr_1, nfl_gcp);
  make_subr(s_fl_fontstructp, tc7_subr_1, nfl_fontstructp);
  /* */
  s_init_basic_fct();
  s_init_bitmap_fct();
  s_init_box_fct();
  s_init_browser_fct();
  s_init_button_fct();
  s_init_canvas_fct();
  s_init_chart_fct();
  s_init_choice_fct();
  s_init_clipbd_fct();
  s_init_clock_fct();
  s_init_counter_fct();
  s_init_cursor_fct();
  s_init_dial_fct();
  s_init_filesys_fct();
  s_init_flps_fct();
  s_init_frame_fct();
  s_init_free_fct();
  s_init_goodies_fct();
  s_init_input_fct();
  s_init_menu_fct();
#ifdef fl_create_menubar
  s_init_menubar_fct();
#endif
  s_init_popup_fct();
  s_init_positioner_fct();
  s_init_scrollbar_fct();
  s_init_slider_fct();
  s_init_folder_fct();
  s_init_text_fct();
  s_init_timer_fct();
  s_init_xbasic_fct();
  s_init_xyplot_fct();
  s_init_extra_fct();
  s_init_event_fct();
#ifdef SFDEBUG
  fputs(";  ...done\n", stderr);
#endif
  return;
}

/* initialization call */
void init_sforms()
{
#ifndef FOR_GUILE
  if (2 <=scm_verbose) 
#endif
    init_sforms_banner();
#ifdef SFDEBUG
  fputs("; Initializing XForms extension :\n", stderr);
#endif
  /* */
  tc16_form = newsmob(&formsmob);
  tc16_obj  = newsmob(&objsmob);
  tc16_win  = newsmob(&winsmob);
  tc16_pix  = newsmob(&pixsmob);
  tc16_dis  = newsmob(&dissmob);
  tc16_xev  = newsmob(&xevsmob);
  tc16_gc   = newsmob(&gcsmob);
  tc16_fnt  = newsmob(&fntsmob);
  /* */
  init_fl_callbacks();
  init_fl_symbols();
  init_fl_functions();
  /* */ 
  if (scm_ldprog(s_sforms_file)==BOOL_F)
    {
      fprintf(stderr, "; Error : Couldn't find file \"%s\"\n", s_sforms_file);
    }
  else
    {
      add_feature("forms");
#ifdef SFDEBUG
      fprintf(stderr, "; - XForms extension -\n");
      fprintf(stderr, "; Implemented Functions : %d\n", yi);
      fprintf(stderr, "; Dummy functions       : %d\n", di);
      fprintf(stderr, "; Not yet implemented   : %d\n", nyi);
#endif
    }
#ifdef FOR_GUILE
  scm_register_module_xxx("sforms", init_sforms);
#endif
  return;
}

/*
** EOF
*/






