/*
**
** SForms : Interface to the XForms library for SCM
**
** This file is part of the SForms extension for SCM.
** (for XForms version 0.88 & SCM version 5b2 / 5c3)
**
** Author : Johannes Leveling
** eMail  : Johannes.Leveling@Informatik.Uni-Oldenburg.De
** Copyright (C) 1997, 1998 Free Software Foundation, Inc.
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this software; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

/*
** s_forms.h - definition of new types
** nov 07 1998 
*/

/* New types (represented as smobs) :
**  - FL_FORM
**  - FL_OBJECT
**  - Window
**  - Pixmap
**  - Display
**  - GC
**  - XEvent
**  - XFontstruct
*/

/* these functions dont really belong here ... */
#ifdef FOR_GUILE /* GUILE */
SCM equal0(SCM p1, SCM p2) 
{
  return (CDR(p1)==CDR(p2)) ? BOOL_T : BOOL_F;
}

/* looks like GUILE is case-sensitive ! (R4RS compliant ???) */
char *upper_string(char *s)
{
  int i=0;
  while(s[i])
    {
      s[i]=scm_upcase(s[i]);
      i++;
    }
  return &s[0];
}
#endif 

/* to avoid returning #f */
SCM MAKSTR(char *s)
{
  if (!s) return makfromstr("\0", (sizet)strlen("\0")); /* instead of #f */
  return makfromstr(s, (sizet)strlen(s));
}

/*
** making types
*/
SCM mkform(FL_FORM *form)
{
  SCM z;
  NEWCELL(z);
  DEFER_INTS;
  CAR(z) = tc16_form; 
  SETCDR(z, form);
  ALLOW_INTS;
  return z;
}

SCM mkobj(FL_OBJECT *obj)
{
  SCM z;
  NEWCELL(z);
  DEFER_INTS;
  CAR(z) = tc16_obj; 
  SETCDR(z, obj);
  ALLOW_INTS;
  return z;
}

SCM mkwin(Window win)
{
  SCM z;
  NEWCELL(z);
  DEFER_INTS;
  CAR(z) = tc16_win; 
  SETCDR(z, win);
  ALLOW_INTS;
  return z;
}

SCM mkpix(Pixmap pix)
{
  SCM z;
  NEWCELL(z);
  DEFER_INTS;
  CAR(z) = tc16_pix; 
  SETCDR(z, pix);
  ALLOW_INTS;
  return z;
}

SCM mkdis(Display *dis)
{
  SCM z;
  NEWCELL(z);
  DEFER_INTS;
  CAR(z)=tc16_dis;
  SETCDR(z, dis);
  ALLOW_INTS;
  return z;
}

SCM mkxev(XEvent *xev)
{
  SCM z;
  XEvent *cx;
  cx=(XEvent *)must_malloc(sizeof(XEvent), "mkxev");
  memcpy(cx, xev, sizeof(XEvent));
  NEWCELL(z);
  DEFER_INTS;
  CAR(z) = tc16_xev;
  SETCDR(z, cx);
  ALLOW_INTS;
  return z;
}

SCM mkxgc(GC gc)
{
  SCM z;
  NEWCELL(z);
  DEFER_INTS;
  CAR(z) = tc16_gc;
  SETCDR(z, gc);
  ALLOW_INTS;
  return z;
}

SCM mkfnt(XFontStruct *fnt)
{
  SCM z;
  NEWCELL(z);
  DEFER_INTS;
  CAR(z) = tc16_fnt;
  SETCDR(z, fnt);
  ALLOW_INTS;
  return z;
}

/* comparison */


/* output */
#define PRINT_TYPE(st)          \
lputs("#<", port);              \
lputs(st, port);                \
iprin1(MAKINUM(CDR(exp)), port, writing);\
lputs(">", port);  


static int prinform(SCM exp, SCM port, PRINT_STATE writing)
{
  PRINT_TYPE("FL_FORM ");
  return !0;
}

static int prinobj(SCM exp, SCM port, PRINT_STATE writing)
{
  PRINT_TYPE("FL_OBJECT ");
  return !0;
}

static int prinwin(SCM exp, SCM port, PRINT_STATE writing)
{
  PRINT_TYPE("Window ");
  return !0;
}

static int prinpix(SCM exp, SCM port, PRINT_STATE writing)
{
  PRINT_TYPE("Pixmap");
  return !0;
}

static int prindis(SCM exp, SCM port, PRINT_STATE writing)
{
  PRINT_TYPE("Display ")
  return !0;
}

static int prinxev(SCM exp, SCM port, PRINT_STATE writing)
{
  PRINT_TYPE("XEvent ");
  return !0;
}

static int pringc(SCM exp, SCM port, PRINT_STATE writing)
{
  PRINT_TYPE("GC ");
  return !0;
}

static int prinfnt(SCM exp, SCM port, PRINT_STATE writing)
{
  PRINT_TYPE("XFontstruct ");
  return !0;
}

/* */
#ifdef FOR_GUILE
sizet free_xevent(SCM p)  /* CELLPTR */
#else
sizet free_xevent(CELLPTR p)
#endif
{
  extern void free(void *);
  free(CHARS(p));
  return sizeof(XEvent);
}

/* smob type structures */
static smobfuns formsmob = {
  mark0, free0,
  prinform, equal0
};

static smobfuns objsmob = {
  mark0, free0,
  prinobj, equal0
};

static smobfuns winsmob = {
  mark0, free0,
  prinwin, equal0
};

static smobfuns pixsmob = {
  mark0, free0,
  prinpix, equal0
};

static smobfuns dissmob = {
  mark0, free0,
  prindis, equal0
};

static smobfuns xevsmob = {
  mark0, free_xevent,
  prinxev, equal0
};

static smobfuns gcsmob = {
  mark0, free0,
  pringc, equal0
};

static smobfuns fntsmob = {
  mark0, free0,
  prinfnt, equal0
};

/*
** EOF
*/


