/*
**
** SForms : Interface to the XForms library for SCM
**
** This file is part of the SForms extension for SCM.
** (for XForms version 0.88 & SCM version 5b2 / 5c3)
**
** Author : Johannes Leveling
** eMail  : Johannes.Leveling@Informatik.Uni-Oldenburg.De
** Copyright (C) 1997, 1998 Free Software Foundation, Inc.
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this software; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

/*
** s_free.c
** nov 09 1998
*/
#include "forms.h"
#include "s_macro.h"

/* initialize symbols */
void s_init_free_sym()
{
  /* types */
  sysintern("fl_normal_free",     MAKINUM(FL_NORMAL_FREE));
  sysintern("fl_inactive_free",   MAKINUM(FL_INACTIVE_FREE));
  sysintern("fl_input_free",      MAKINUM(FL_INPUT_FREE));
  sysintern("fl_continuous_free", MAKINUM(FL_CONTINUOUS_FREE));
  sysintern("fl_all_free",        MAKINUM(FL_ALL_FREE));
  return;
}

/* routines */
static char s_fl_create_free[]="fl_create_free";
SCM nfl_create_free(SCM _i1, SCM _i2, SCM _args)
{
  SCM _i3, _i4, _i5, _s, _proc;
  SFDEBUGMSG(s_fl_create_free);
  ASSERT(ilength(_args)==5, _args, UNDEFINED, s_fl_create_free);
  _i3=CAR(_args); _args=CDR(_args);
  _i4=CAR(_args); _args=CDR(_args);
  _i5=CAR(_args); _args=CDR(_args);
  _s=CAR(_args); _args=CDR(_args);
  _proc=CAR(_args);
  ASSERT(INUMP(_i1), _i1, ARG1, s_fl_create_free);
  ASSERT(INUMP(_i2), _i2, ARG2, s_fl_create_free);
  ASSERT(INUMP(_i3), _i3, ARG3, s_fl_create_free);
  ASSERT(INUMP(_i4), _i4, ARG4, s_fl_create_free);
  ASSERT(INUMP(_i5), _i5, ARG5, s_fl_create_free);
  ASSERT(NIMP(_s) && STRINGP(_s), _s, ARGn, s_fl_create_free);
  ASSERT(BOOL_T==procedurep(_proc), _proc, ARGn, s_fl_create_free);
  cb_add(loc_free_handler, MAKINUM(0), _proc);
  return mkobj(fl_create_free(INUM(_i1), INUM(_i2), INUM(_i3), 
			      INUM(_i4), INUM(_i5), CHARS(_s), 
			      cb_free_handler));
} /* fl_create_free */
static char s_fl_add_free[]="fl_add_free";
SCM nfl_add_free(SCM _i1, SCM _i2, SCM _args)
{
  SCM _i3, _i4, _i5, _s, _proc;
  SFDEBUGMSG(s_fl_add_free);
  ASSERT(ilength(_args)==5, _args, UNDEFINED, s_fl_add_free);
  _i3=CAR(_args); _args=CDR(_args);
  _i4=CAR(_args); _args=CDR(_args);
  _i5=CAR(_args); _args=CDR(_args);
  _s=CAR(_args); _args=CDR(_args);
  _proc=CAR(_args);
  ASSERT(INUMP(_i1), _i1, ARG1, s_fl_add_free);
  ASSERT(INUMP(_i2), _i2, ARG2, s_fl_add_free);
  ASSERT(INUMP(_i3), _i3, ARG3, s_fl_add_free);
  ASSERT(INUMP(_i4), _i4, ARG4, s_fl_add_free);
  ASSERT(INUMP(_i5), _i5, ARG5, s_fl_add_free);
  ASSERT(NIMP(_s) && STRINGP(_s), _s, ARGn, s_fl_add_free);
  ASSERT(BOOL_T==procedurep(_proc), _proc, ARGn, s_fl_add_free);
  cb_add(loc_free_handler, MAKINUM(0), _proc);
  return mkobj(fl_add_free(INUM(_i1), INUM(_i2), INUM(_i3), 
			   INUM(_i4), INUM(_i5), CHARS(_s), 
			   cb_free_handler));
} /* fl_add_free */

/* initialize functions */
void s_init_free_fct()
{
  /* */
  MSUBR_L(fl_create_free);
  MSUBR_L(fl_add_free);
  return;
}

/*
** EOF
*/






