/*
**
** SForms : Interface to the XForms library for SCM
**
** This file is part of the SForms extension for SCM.
** (for XForms version 0.88 & SCM version 5b2 / 5c3)
**
** Author : Johannes Leveling
** eMail  : Johannes.Leveling@Informatik.Uni-Oldenburg.De
** Copyright (C) 1997, 1998 Free Software Foundation, Inc.
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this software; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

/*
** s_imacro.h - function returning an integer number
** nov 07 1998
*/

#define SUBR_int__void(_func)              \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(){                          \
SFDEBUGMSG(s_ ##_func);                    \
return MAKINUM(_func());}           /* ok */

#define SUBR_int__int(_func)               \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1){                   \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
return MAKINUM(_func(INUM(_i1)));}  /* ok */ 

#define SUBR_int__2int(_func)              \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2){          \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
return MAKINUM(_func(INUM(_i1),            \
                     INUM(_i2)));}  /* ok */

#define SUBR_int__3int(_func)              \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _i3){ \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
ASSERT(INUMP(_i3), _i3, ARG3, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
return MAKINUM(_func(INUM(_i1), INUM(_i2), \
                     INUM(_i3)));}  /* ok */   

#define SUBR_int__uint(_func)              \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1){                   \
ASSERT(INUMP(_i1) && INUM(_i1) >= 0,       \
                   _i1, ARG1, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
return MAKINUM(_func(INUM(_i1)));}  /* ok */

#define SUBR_int__int_charptr(_func)       \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i, SCM _s){            \
ASSERT(INUMP(_i), _i, ARG1, #_func);       \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG2, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
return MAKINUM(_func(INUM(_i),             \
                     CHARS(_s)));}  /* ok */

#define SUBR_int__charptr_int(_func)       \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _s, SCM _i){            \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG1, #_func);     \
ASSERT(INUMP(_i), _i, ARG2, #_func);       \
SFDEBUGMSG(s_ ##_func);                    \
return MAKINUM(_func(CHARS(_s),            \
                     INUM(_i)));}   /* ok */

#define SUBR_int__charptr(_func)           \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _s){                    \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG1, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
return MAKINUM(_func(CHARS(_s)));}  /* ok */

#define SUBR_int__window(_func)            \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _w){                    \
ASSERT(WINP(_w), _w, ARG1, #_func);        \
SFDEBUGMSG(s_ ##_func);                    \
return MAKINUM(_func(WIN(_w)));}    /* ok */

#define SUBR_int__window_int(_func)        \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _w, SCM _i){            \
ASSERT(WINP(_w), _w, ARG1, #_func);        \
ASSERT(INUMP(_i), _i, ARG2, #_func);       \
SFDEBUGMSG(s_ ##_func);                    \
return MAKINUM(_func(WIN(_w),              \
                     INUM(_i)));}   /* ok */

#define SUBR_int__formptr(_func)           \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _f){                    \
ASSERT(FORMP(_f), _f, ARG1, #_func);       \
SFDEBUGMSG(s_ ##_func);                    \
return MAKINUM(_func(FORM(_f)));}   /* ok */

#define SUBR_int__objectptr(_func)         \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o){                    \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
SFDEBUGMSG(s_ ##_func);                    \
return MAKINUM(_func(OBJ(_o)));}    /* ok */

#define SUBR_int__objectptr_int(_func)     \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _i){            \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(INUMP(_i), _i, ARG2, #_func);       \
SFDEBUGMSG(s_ ##_func);                    \
return MAKINUM(_func(OBJ(_o),              \
                     INUM(_i)));}   /* ok */

#define SUBR_int__objectptr_charptr(_func) \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _s){            \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG2, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
return MAKINUM(_func(OBJ(_o),              \
                     CHARS(_s)));}  /* ok */

/* */
#define SUBR_int__4int(_func)              \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _args){\
SCM _i3, _i4;                              \
ASSERT(ilength(_args)==2, _args,           \
       ARG3, #_func);                      \
_i3=CAR(_args); _args=CDR(_args);          \
_i4=CAR(_args);                            \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
ASSERT(INUMP(_i3), _i3, ARG3, #_func);     \
ASSERT(INUMP(_i4), _i4, ARG4, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
return MAKINUM(_func(INUM(_i1),            \
                     INUM(_i2), INUM(_i3), \
                     INUM(_i4)));}  /* ok */

#define SUBR_int__formptr_2int_charptr(_func)\
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _f, SCM _i1, SCM _args){\
SCM _i2, _s;                               \
ASSERT(ilength(_args)==2, _args,           \
       ARG2, #_func);                      \
_i2=CAR(_args); _args=CDR(_args);          \
_s=CAR(_args);                             \
ASSERT(FORMP(_f), _f, ARG1, #_func);       \
ASSERT(INUMP(_i1), _i1, ARG2, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG3, #_func);     \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG4, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
return MAKINUM(_func(FORM(_f),             \
                     INUM(_i1), INUM(_i2), \
                     CHARS(_s)));}  /* ok */


#define SUBR_int__2int_charptr_int(_func)  \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _args){\
SCM _s, _i3;                               \
ASSERT(ilength(_args)==2, _args,           \
       ARG2, #_func);                      \
_s=CAR(_args); _args=CDR(_args);           \
_i3=CAR(_args);                            \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG3, #_func);     \
ASSERT(INUMP(_i3), _i3, ARG4, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
return MAKINUM(_func(INUM(_i1),            \
                     INUM(_i2), CHARS(_s), \
                     INUM(_i3)));}  /* ok */


/*
** EOF
*/
