/*
**
** SForms : Interface to the XForms library for SCM
**
** This file is part of the SForms extension for SCM.
** (for XForms version 0.88 & SCM version 5b2 / 5c3)
**
** Author : Johannes Leveling
** eMail  : Johannes.Leveling@Informatik.Uni-Oldenburg.De
** Copyright (C) 1997, 1998 Free Software Foundation, Inc.
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this software; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

/*
** s_input.c
** oct 25 1998 
** nov 09 1998
*/
#include "forms.h"
#include "s_macro.h"

/* initialize symbols */
void s_init_input_sym()
{
  /* types */
  sysintern("fl_normal_input",    MAKINUM(FL_NORMAL_INPUT));
  sysintern("fl_float_input",     MAKINUM(FL_FLOAT_INPUT));
  sysintern("fl_int_input",       MAKINUM(FL_INT_INPUT));
  sysintern("fl_date_input",      MAKINUM(FL_DATE_INPUT));
  sysintern("fl_multiline_input", MAKINUM(FL_MULTILINE_INPUT));
  sysintern("fl_hidden_input",    MAKINUM(FL_HIDDEN_INPUT));
  sysintern("fl_secret_input",    MAKINUM(FL_SECRET_INPUT));
  /* for date input */
  sysintern("fl_input_mmdd",      MAKINUM(FL_INPUT_MMDD)); 
  sysintern("fl_input_ddmm",      MAKINUM(FL_INPUT_DDMM));
  /* renamed to fl_cringbell : (fl_ringbell) exists */
  sysintern("fl_cringbell",       MAKINUM(FL_RINGBELL));             
  return;
}

/* routines */
SUBR_objectptr__5int_charptr(fl_create_input);
SUBR_objectptr__5int_charptr(fl_add_input);
SUBR_void__objectptr_charptr(fl_set_input);
SUBR_void__objectptr_int(fl_set_input_return);
SUBR_void__objectptr_2int(fl_set_input_color);
SUBR_void__objectptr_int(fl_set_input_scroll);
SUBR_void__objectptr_2int(fl_set_input_cursorpos);
SUBR_void__objectptr_int(fl_set_input_selected);
SUBR_void__objectptr_2int(fl_set_input_selected_range);
static char s_fl_get_input_selected_range[]="fl_get_input_selected_range";
SCM nfl_get_input_selected_range(SCM _o)
{
  int i1, i2;
  const char *s;
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_input_selected_range);
  SFDEBUGMSG(s_fl_get_input_selected_range);
  s=fl_get_input_selected_range(OBJ(_o), &i1, &i2);
  return cons(makfrom0str((char *)s), 
	      cons(MAKINUM(i1), 
		   cons(MAKINUM(i2), EOL)));
} /* fl_get_input_selected_range */
SUBR_void__objectptr_int(fl_set_input_maxchars);
SUBR_void__objectptr_2int(fl_set_input_format);
SUBR_void__objectptr_int(fl_set_input_hscrollbar);
SUBR_void__objectptr_int(fl_set_input_vscrollbar);
SUBR_void__objectptr_int(fl_set_input_topline);
SUBR_void__objectptr_2int(fl_set_input_scrollbarsize);
SUBR_void__objectptr_int(fl_set_input_xoffset);
SUBR_int__objectptr(fl_get_input_xoffset);
/* */
SUBR_int__objectptr(fl_get_input_topline);
SUBR_int__objectptr(fl_get_input_screenlines);
static char s_fl_get_input_cursorpos[]="fl_get_input_cursorpos";
SCM nfl_get_input_cursorpos(SCM _o)
{
  int i1, i2, i;
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_input_cursorpos);
  i=fl_get_input_cursorpos(OBJ(_o), &i1, &i2);
  return cons(MAKINUM(i), 
	      cons(MAKINUM(i1), 
		   cons(MAKINUM(i2), EOL)));
} /* fl_get_input_cursorpos */
SUBR_int__objectptr(fl_get_input_numberoflines);
SUBR_2int__objectptr(fl_get_input_format);
SUBR_charptr__objectptr(fl_get_input);
static char s_fl_set_input_filter[]="fl_set_input_filter";
SCM nfl_set_input_filter(SCM _o, SCM _proc)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_set_input_filter);
  ASSERT(BOOL_T==procedurep(_proc), _proc, ARG2, s_fl_set_input_filter);
  SFDEBUGMSG(s_fl_set_input_filter);
  cb_add(loc_input_filter, _o, _proc);
  fl_set_input_filter(OBJ(_o), cb_input_filter);
  return UNSPECIFIED;
} /* fl_set_input_filter */
/* edit keys */
static char s_fl_set_input_editkeymap[]="fl_set_input_editkeymap";
#define f_kmap k=CAR(_kmap);
#define n_kmap _kmap=CDR(_kmap);
#define s_kmap(_entry) \
if (k!=BOOL_F) { \
ASSERT(INUMP(k), k, ARG1, s_fl_set_input_editkeymap); \
nkmap._entry=INUM(k); }

SCM nfl_set_input_editkeymap(SCM _kmap)
{
  SCM k;
  FL_EditKeymap nkmap;
  /* */
  ASSERT(CONSP(_kmap) && ilength(_kmap)==23, _kmap, ARG1, 
	 s_fl_set_input_editkeymap);
  SFDEBUGMSG(s_fl_set_input_editkeymap);
  /* basic editing */
  f_kmap; s_kmap(del_prev_char);    n_kmap; 
  f_kmap; s_kmap(del_next_char);    n_kmap; 
  f_kmap; s_kmap(del_prev_word);    n_kmap;
  f_kmap; s_kmap(del_next_word);    n_kmap;
  /* movement */
  f_kmap; s_kmap(moveto_prev_line); n_kmap;
  f_kmap; s_kmap(moveto_next_line); n_kmap;
  f_kmap; s_kmap(moveto_prev_char); n_kmap;
  f_kmap; s_kmap(moveto_next_char); n_kmap;
  f_kmap; s_kmap(moveto_prev_word); n_kmap;
  f_kmap; s_kmap(moveto_next_word); n_kmap;
  f_kmap; s_kmap(moveto_prev_page); n_kmap;
  f_kmap; s_kmap(moveto_next_page); n_kmap;
  f_kmap; s_kmap(moveto_bol);       n_kmap;
  f_kmap; s_kmap(moveto_eol);       n_kmap;
  f_kmap; s_kmap(moveto_bof);       n_kmap;
  f_kmap; s_kmap(moveto_eof);       n_kmap;
  /* misc. stuff */
  f_kmap; s_kmap(transpose);        n_kmap;
  f_kmap; s_kmap(paste);            n_kmap;
  f_kmap; s_kmap(backspace);        n_kmap;
  f_kmap; s_kmap(del_to_bol);       n_kmap;
  f_kmap; s_kmap(del_to_eol);       n_kmap;
  f_kmap; s_kmap(clear_field);      n_kmap;
  f_kmap; s_kmap(del_to_eos); 
  /* */
  /* reserved entries not supported */
  fl_set_input_editkeymap(&nkmap);
  return UNSPECIFIED;
}

/* initialize functions */
void s_init_input_fct()
{
  MSUBR_L(fl_create_input);
  MSUBR_L(fl_add_input);
  MSUBR_2(fl_set_input);
  MSUBR_2(fl_set_input_return);
  MSUBR_3(fl_set_input_color);
  MSUBR_2(fl_set_input_scroll);
  MSUBR_3(fl_set_input_cursorpos);
  MSUBR_2(fl_set_input_selected);
  MSUBR_3(fl_set_input_selected_range);
  MSUBR_1(fl_get_input_selected_range);
  MSUBR_2(fl_set_input_maxchars);
  MSUBR_3(fl_set_input_format);
  MSUBR_2(fl_set_input_hscrollbar);
  MSUBR_2(fl_set_input_vscrollbar);
  MSUBR_2(fl_set_input_topline);
  MSUBR_3(fl_set_input_scrollbarsize);
  MSUBR_2(fl_set_input_xoffset);
  MSUBR_1(fl_get_input_xoffset);
  MSUBR_1(fl_get_input_topline);
  MSUBR_1(fl_get_input_screenlines);
  MSUBR_1(fl_get_input_cursorpos);
  MSUBR_1(fl_get_input_numberoflines);
  MSUBR_1(fl_get_input_format);
  MSUBR_1(fl_get_input);
  MSUBR_1(fl_set_input_filter);
  MSUBR_1(fl_set_input_editkeymap); 
  return;
}

/*
** EOF
*/

