/*
**
** SForms : Interface to the XForms library for SCM
**
** This file is part of the SForms extension for SCM.
** (for XForms version 0.88 & SCM version 5b2 / 5c3)
**
** Author : Johannes Leveling
** eMail  : Johannes.Leveling@Informatik.Uni-Oldenburg.De
** Copyright (C) 1997, 1998 Free Software Foundation, Inc.
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this software; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

/*
** s_lmacro.h - functions returning a list
** nov 07 1998
*/

#define SUBR_2double__objectptr(_func)      \
static char s_ ## _func[]=#_func;           \
SCM n ## _func(SCM _o){                     \
double d1, d2;                              \
ASSERT(OBJP(_o), _o, ARG1, #_func);         \
SFDEBUGMSG(s_ ##_func);                     \
_func(OBJ(_o), &d1, &d2);                   \
return cons2(makdbl(d1, 0.0),               \
             makdbl(d2, 0.0), EOL);} /* ok */

#define SUBR_2float__objectptr(_func)       \
static char s_ ## _func[]=#_func;           \
SCM n ## _func(SCM _o){                     \
float f1, f2;                               \
ASSERT(OBJP(_o), _o, ARG1, #_func);         \
SFDEBUGMSG(s_ ##_func);                     \
_func(OBJ(_o), &f1, &f2);                   \
return cons2(makdbl((double)f1, 0.0),       \
             makdbl((double)f2, 0.0),       \
             EOL);}                  /* ok */

#define SUBR_2float_int__objectptr(_func)   \
static char s_ ## _func[]=#_func;           \
SCM n ## _func(SCM _o){                     \
float f1, f2;                               \
int i;                                      \
ASSERT(OBJP(_o), _o, ARG1, #_func);         \
SFDEBUGMSG(s_ ##_func);                     \
_func(OBJ(_o), &f1, &f2, &i);               \
return cons(makdbl((double)f1, 0.0),        \
	    cons(makdbl((double)f2, 0.0),   \
		 cons(MAKINUM(i),           \
                      EOL)));}       /* ok */
   
#define SUBR_2int__void(_func)              \
static char s_ ## _func[]=#_func;           \
SCM n ## _func(SCM _o){                     \
long i1, i2; /* int */                      \
SFDEBUGMSG(s_ ##_func);                     \
_func(&i1, &i2);                            \
return cons(MAKINUM(i1),                    \
            cons(MAKINUM(i2),               \
                 EOL));}             /* ok */

#define SUBR_2int__objectptr(_func)         \
static char s_ ## _func[]=#_func;           \
SCM n ## _func(SCM _o){                     \
int i1, i2; /* coords */                    \
ASSERT(OBJP(_o), _o, ARG1, #_func);         \
SFDEBUGMSG(s_ ##_func);                     \
_func(OBJ(_o), &i1, &i2);                   \
return cons(MAKINUM(i1),                    \
            cons(MAKINUM(i2),               \
                 EOL));}             /* ok */

#define SUBR_4int__objectptr(_func)         \
static char s_ ## _func[]=#_func;           \
SCM n ## _func(SCM _o){                     \
int i1, i2, i3, i4; /* coords */            \
ASSERT(OBJP(_o), _o, ARG1, #_func);         \
SFDEBUGMSG(s_ ##_func);                     \
_func(OBJ(_o), &i1, &i2, &i3, &i4);         \
return cons(MAKINUM(i1),                    \
            cons(MAKINUM(i2),               \
                cons(MAKINUM(i3),           \
                    cons(MAKINUM(i4),       \
                         EOL))));}   /* ok */

#define SUBR_2int__window(_func)            \
static char s_ ## _func[]=#_func;           \
SCM n ## _func(SCM _w){                     \
int i1, i2; /* coords */                    \
ASSERT(WINP(_w), _w, ARG1, #_func);         \
SFDEBUGMSG(s_ ##_func);                     \
_func(WIN(_w), &i1, &i2);                   \
return cons(MAKINUM(i1),                    \
            cons(MAKINUM(i2),               \
                 EOL));}             /* ok */

#define SUBR_4int__window(_func)            \
static char s_ ## _func[]=#_func;           \
SCM n ## _func(SCM _w){                     \
int i1, i2, i3, i4; /* coords */            \
ASSERT(WINP(_w), _w, ARG1, #_func);         \
SFDEBUGMSG(s_ ##_func);                     \
_func(WIN(_w), &i1, &i2, &i3, &i4);         \
return cons(MAKINUM(i1),                    \
            cons(MAKINUM(i2),               \
                cons(MAKINUM(i3),           \
                    cons (MAKINUM(i4),      \
                          EOL))));}  /* ok */


/*
** EOF
*/
