/*
**
** SForms : Interface to the XForms library for SCM
**
** This file is part of the SForms extension for SCM.
** (for XForms version 0.88 & SCM version 5b2 / 5c3)
**
** Author : Johannes Leveling
** eMail  : Johannes.Leveling@Informatik.Uni-Oldenburg.De
** Copyright (C) 1997, 1998 Free Software Foundation, Inc.
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this software; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

/*
** s_macro.h - macros and declarations
** mar 28 1999 
*/
#ifndef __SFMACRO_H__
#define __SFMACRO_H__

#ifndef FL_FORMS_H
#include "forms.h"
#endif

/* uncomment the following line if SForms extension is for GUILE */
/*#define FOR_GUILE*/

#ifndef FOR_GUILE     /* SCM */
#include "scm.h"
#define PRINT_STATE int
#endif

#ifdef FOR_GUILE      /* GUILE */
#include "libguile/libguile.h"
extern SCM gh_eval_file (char *);
extern int gh_procedure_p(SCM);

extern char *upper_string(char *);

/* backward definitions for GUILE */
/* GUILE functions are more likely to change ... */
#define CELLPTR     SCM_CELLPTR
#define TYP16       SCM_TYP16
#define SETGC8MARK  SCM_SETGC8MARK
#define NEWCELL     SCM_NEWCELL
#define ASSERT      SCM_ASSERT

#define tc7_subr_0  scm_tc7_subr_0
#define tc7_subr_1  scm_tc7_subr_1
#define tc7_subr_2  scm_tc7_subr_2
#define tc7_subr_3  scm_tc7_subr_3
#define tc7_lsubr_2 scm_tc7_lsubr_2
#define sysintern(str, sym)  scm_sysintern(upper_string(str), sym)
#define ARG1        SCM_ARG1
#define ARG2        SCM_ARG2
#define ARG3        SCM_ARG3
#define ARG4        SCM_ARG4
#define ARG5        SCM_ARG5
#define ARG6        SCM_ARG6
#define ARGn        SCM_ARGn
#define WNA         SCM_WNA

#define DEFER_INTS  SCM_DEFER_INTS
#define ALLOW_INTS  SCM_ALLOW_INTS

#define BOOL_T      SCM_BOOL_T
#define BOOL_F      SCM_BOOL_F
#define UNDEFINED   SCM_UNDEFINED
#define UNSPECIFIED SCM_UNSPECIFIED
#define EOL         SCM_EOL
#define CAR         SCM_CAR
#define CDR         SCM_CDR
#define MAKINUM     SCM_MAKINUM
#define MAKICHR     SCM_MAKICHR
#define REAL        SCM_REAL
#define REALPART    SCM_REALPART
#define CHARS       SCM_CHARS
#define INUM        SCM_INUM

#define INUMP       SCM_INUMP
#define NUMBERP     SCM_NUMBERP
#define NIMP        SCM_NIMP
#define STRINGP     SCM_STRINGP /* or SCM_ROSTRINGP */
#define CONSP       SCM_CONSP
#define REALP       SCM_REALP
#define SETCDR      SCM_SETCDR

#define cons        scm_cons
#define cons2       scm_cons2
#define apply       scm_apply
#define equal       scm_equal_p
#define sizet       scm_sizet
#define makfromstr(cs, n) scm_makfromstr(cs, n, 0)
#define makfrom0str scm_makfrom0str
#define makdbl      scm_makdbl
#define ulong2num   scm_ulong2num
#define long2num    scm_long2num
#define reverse     scm_reverse
#define ilength     scm_ilength
#define procedurep  scm_procedure_p
#define make_subr   scm_make_subr
#define lputs       scm_puts
#define iprin1(a, b, c) scm_display(a, b)
#define must_malloc scm_must_malloc
#define newsmob     scm_newsmob
#define add_feature scm_add_feature
#define scm_ldprog  gh_eval_file

#define smobfuns    scm_smobfuns
#define mark0       scm_mark0
#define free0       scm_free0
#define PRINT_STATE scm_print_state*
#endif /* GUILE */

/* */
extern void SFDEBUGMSG(char *);
extern int scm_verbose;

/* callback/filter/handler variables */
extern SCM *loc_io_callback;
extern SCM *loc_signal_callback;
extern SCM *loc_timeout_handler;
extern SCM *loc_form_atclose_callback;
extern SCM *loc_atclose_callback;
extern SCM *loc_form_atdeactivate_callback;
extern SCM *loc_form_atactivate_callback;
extern SCM *loc_form_callback;
extern SCM *loc_raw_callback;
extern SCM *loc_object_prehandler;
extern SCM *loc_object_posthandler;
extern SCM *loc_object_callback;
extern SCM *loc_font_enumerator;
extern SCM *loc_symbol_handler;
extern SCM *loc_error_handler;
extern SCM *loc_new_object_handler;
extern SCM *loc_browser_dblclick_callback;
extern SCM *loc_draw_button;
extern SCM *loc_cleanup_button;
extern SCM *loc_canvas_handler;
extern SCM *loc_canvas_modify1;
extern SCM *loc_canvas_modify2;
extern SCM *loc_canvas_modify3;
extern SCM *loc_choice_item_callback;
extern SCM *loc_stuff_clipboard_callback;
extern SCM *loc_request_clipboard_callback;
extern SCM *loc_counter_filter;
extern SCM *loc_dirlist_filter;
extern SCM *loc_free_handler;
extern SCM *loc_fselector_callback;
extern SCM *loc_fselector_appbutton;
extern SCM *loc_input_filter;
extern SCM *loc_menu_item_callback;
extern SCM *loc_menubar_item_callback;
extern SCM *loc_pup_item_callback;
extern SCM *loc_pup_enter_callback;
extern SCM *loc_pup_leave_callback;
extern SCM *loc_pup_menu_callback;
extern SCM *loc_slider_filter;
extern SCM *loc_timer_filter;
extern SCM *loc_xyplot_symbol_handler;
extern SCM *loc_event_callback;
extern SCM *loc_idle_callback;
extern SCM *loc_sevent_callback;
/* callback/filter/handler routines */
extern void cb_add(SCM *, SCM, SCM);
extern SCM cb_get(SCM , SCM);
extern void cb_protect(SCM);
extern void cb_del(SCM *, SCM);
/* */
extern void cb_io_callback(int, void *);
extern void cb_signal_callback(int, void *); /* handler */
extern void cb_timeout_handler(int, void *);
extern int cb_form_atclose_callback(FL_FORM *, void *);
extern int cb_atclose_callback(FL_FORM *, void *);
extern void cb_form_atdeactivate_callback(FL_FORM *, void *);
extern void cb_form_atactivate_callback(FL_FORM *, void *);
extern void cb_form_callback(FL_OBJECT *, void *);
extern int cb_raw_callback(FL_FORM *, void *);
extern int cb_object_prehandler(FL_OBJECT *, int, FL_Coord, FL_Coord, 
				int, void *);
extern int cb_object_posthandler(FL_OBJECT *, int, FL_Coord, FL_Coord, 
				 int, void *);
extern void cb_object_callback(FL_OBJECT *, long);
extern void cb_font_enumerator(const char *);
extern void cb_symbol_handler(FL_Coord, FL_Coord, 
			      FL_Coord, FL_Coord, int, FL_COLOR);
extern void cb_error_handler(const char *, const char *, ...);
extern int cb_new_object_handler(FL_OBJECT *, int, 
				 FL_Coord, FL_Coord, int, void *);
/* */
extern void cb_browser_dblclick_callback(FL_OBJECT *, long);
extern void cb_draw_button(FL_OBJECT *);
extern void cb_cleanup_button(FL_BUTTON_STRUCT *);
extern int cb_canvas_handler(FL_OBJECT *, Window, int, int, 
			     XEvent *, void *);
extern int cb_canvas_modify1(FL_OBJECT *);
extern int cb_canvas_modify2(FL_OBJECT *);
extern int cb_canvas_modify3(FL_OBJECT *);
extern int cb_choice_item_callback(int);
int cb_stuff_clipboard(FL_OBJECT *, long);
int cb_request_clipboard(FL_OBJECT *, long , const void *, long );
extern const char *cb_counter_filter(FL_OBJECT *, double, int);
extern int cb_dirlist_filter(const char *, int);
extern int cb_free_handler(FL_OBJECT *, int, FL_Coord, FL_Coord, int, void *);
extern int cb_fselector_callback(const char *, void *);
extern void cb_fselector_appbutton(void *);
extern int cb_input_filter(FL_OBJECT *, const char *, const char *, int);
extern int cb_menu_item_callback(int);
extern int cb_menubar_item_callback(int);
extern int cb_pup_item_callback(int);
extern void cb_pup_enter_callback(int, void *);
extern void cb_pup_leave_callback(int, void *);
extern int cb_pup_menu_callback(int);
extern const char *cb_slider_filter(FL_OBJECT *, double, int);
extern char *cb_timer_filter(FL_OBJECT *, double);
extern void cb_xyplot_symbol_handler(FL_OBJECT *, int, 
				     FL_POINT *, int, int, int);
extern int cb_event_callback(XEvent *, void *);
extern int cb_idle_callback(XEvent *, void *);
extern int cb_sevent_callback(XEvent *, void *);

/* types */
/*extern sizet free_xevent(CELLPTR); GUILE*/
/* */
extern long tc16_form;
extern long tc16_obj;
extern long tc16_win;
extern long tc16_pix;
extern long tc16_dis;
extern long tc16_xev;
extern long tc16_gc;
extern long tc16_fnt;
/* */
extern SCM MAKSTR(char *);
extern SCM mkform(FL_FORM *);
extern SCM mkobj(FL_OBJECT *);
extern SCM mkwin(Window);
extern SCM mkpix(Pixmap);
extern SCM mkdis(Display *);
extern SCM mkxev(XEvent *);
extern SCM mkxgc(GC);
extern SCM mkfnt(XFontStruct *);
/* symbol initialization */
extern void s_init_basic_sym();
extern void s_init_xbasic_sym();
extern void s_init_bitmap_sym();
extern void s_init_box_sym();
extern void s_init_browser_sym();
extern void s_init_button_sym();
extern void s_init_canvas_sym();
extern void s_init_chart_sym();
extern void s_init_choice_sym();
extern void s_init_clipbd_sym();
extern void s_init_clock_sym();
extern void s_init_counter_sym();
extern void s_init_cursor_sym();
extern void s_init_dial_sym();
extern void s_init_filesys_sym();
extern void s_init_flps_sym();
extern void s_init_frame_sym();
extern void s_init_free_sym();
extern void s_init_goodies_sym();
extern void s_init_input_sym();
extern void s_init_menu_sym();
extern void s_init_menubar_sym();
extern void s_init_popup_sym();
extern void s_init_positioner_sym();
extern void s_init_scrollbar_sym();
extern void s_init_slider_sym();
extern void s_init_folder_sym();
extern void s_init_text_sym();
extern void s_init_timer_sym();
extern void s_init_xyplot_sym();
extern void s_init_extra_sym();
extern void s_init_event_sym();
/* function initialization */
extern void s_init_basic_fct();
extern void s_init_xbasic_fct();
extern void s_init_bitmap_fct();
extern void s_init_browser_fct();
extern void s_init_box_fct();
extern void s_init_button_fct();
extern void s_init_canvas_fct();
extern void s_init_chart_fct();
extern void s_init_choice_fct();
extern void s_init_clipbd_fct();
extern void s_init_clock_fct();
extern void s_init_counter_fct();
extern void s_init_cursor_fct();
extern void s_init_dial_fct();
extern void s_init_filesys_fct();
extern void s_init_flps_fct();
extern void s_init_frame_fct();
extern void s_init_free_fct();
extern void s_init_goodies_fct();
extern void s_init_input_fct();
extern void s_init_menu_fct();
extern void s_init_menubar_fct();
extern void s_init_popup_fct();
extern void s_init_positioner_fct();
extern void s_init_scrollbar_fct();
extern void s_init_slider_fct();
extern void s_init_folder_fct();
extern void s_init_text_fct();
extern void s_init_timer_fct();
extern void s_init_xyplot_fct();
extern void s_init_extra_fct();
extern void s_init_event_fct();
/* */

/* statistics */
extern int nyi;
extern int yi;
extern int myi;
extern int di;


#define WARN_DUMMY "Warning : dummy function called"

#define DOUBLE(obj) (INUMP(obj)? INUM(obj):REALPART(obj))
#define DOUBLEP(obj) (INUMP(obj) || REALP(obj))

#define FORM(obj)  ((FL_FORM*)CDR(obj))
#define FORMP(obj) (NIMP(obj) && (tc16_form==TYP16(obj)))

#define OBJ(obj)   ((FL_OBJECT*)CDR(obj))
#define OBJP(obj)  (NIMP(obj) && (tc16_obj==TYP16(obj)))

#define WIN(obj)   ((Window)CDR(obj))
#define WINP(obj)  (NIMP(obj) && (tc16_win==TYP16(obj)))

#define PIX(obj)   ((Pixmap)CDR(obj))
#define PIXP(obj)  (tc16_pix==TYP16(obj))

#define DIS(obj)   ((Display *)CDR(obj))
#define DISP(obj)  (NIMP(obj) && (tc16_dis==TYP16(obj)))

#define XEV(obj)   ((XEvent *)CDR(obj))
#define XEVP(obj)  (NIMP(obj) && (tc16_xev==TYP16(obj)))

#define SGC(obj)   ((GC)CDR(obj))
#define SGCP(obj)  (tc16_gc==TYP16(obj))

#define FNT(obj)   ((XFontstruct *)CDR(obj))
#define FNTP(obj)  (NIMP(obj) && (tc16_fnt==TYP16(obj)))

/* functions */
#define MSUBR_0(_func) \
yi++;make_subr(s_ ##_func, tc7_subr_0, n ## _func);
#define MSUBR_1(_func) \
yi++;make_subr(s_ ##_func, tc7_subr_1, n ## _func);
#define MSUBR_2(_func) \
yi++;make_subr(s_ ##_func, tc7_subr_2, n ## _func);
#define MSUBR_3(_func) \
yi++;make_subr(s_ ##_func, tc7_subr_3, n ## _func);
#define MSUBR_L(_func) \
yi++;make_subr(s_ ##_func, tc7_lsubr_2, n ## _func);
#define MSUBR_N(_func) \
nyi++;

/* dummy functions */
#define MSUBR_d0(_func) \
di++;make_subr(s_ ##_func, tc7_subr_0, n ## _func);
#define MSUBR_d1(_func) \
di++;make_subr(s_ ##_func, tc7_subr_1, n ## _func);
#define MSUBR_d2(_func) \
di++;make_subr(s_ ##_func, tc7_subr_2, n ## _func);
#define MSUBR_d3(_func) \
di++;make_subr(s_ ##_func, tc7_subr_3, n ## _func);
#define MSUBR_dL(_func) \
di++;make_subr(s_ ##_func, tc7_lsubr_2, n ## _func);

/* include function macros */
#include "s_nmacro.h"
#include "s_vmacro.h"
#include "s_imacro.h"
#include "s_smacro.h"
#include "s_omacro.h"
#include "s_dmacro.h"
#include "s_fmacro.h"
#include "s_lmacro.h"
#include "s_wmacro.h"
#include "s_0macro.h"
/* */
#endif __SFMACRO_H__

/*
** EOF
*/




