/*
**
** SForms : Interface to the XForms library for SCM
**
** This file is part of the SForms extension for SCM.
** (for XForms version 0.88 & SCM version 5b2 / 5c3)
**
** Author : Johannes Leveling
** eMail  : Johannes.Leveling@Informatik.Uni-Oldenburg.De
** Copyright (C) 1997, 1998 Free Software Foundation, Inc.
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this software; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

/*
** s_popup.c
** oct 27 1998
** nov 09 1998
*/
#include "forms.h"
#include "s_macro.h"

/* initialize symbols */
void s_init_popup_sym()
{
  /* */
  sysintern("fl_maxpupi",   MAKINUM(FL_MAXPUPI));  
  sysintern("fl_pup_padh",  MAKINUM(FL_PUP_PADH));
  return;
}

/* routines */
static char s_fl_setpup_entries[]="fl_setpup_entries";
SCM nfl_setpup_entries(SCM _i, SCM _pelst)
{
  int i, c;
  ASSERT(INUMP(_i), _i, ARG1, s_fl_setpup_entries);
  ASSERT(NIMP(_pelst) && CONSP(_pelst), _pelst, ARG2, s_fl_setpup_entries);
  SFDEBUGMSG(s_fl_setpup_entries);
  i=ilength(_pelst);
  {
    FL_PUP_ENTRY pe[i+1]; /* last entry with NULL text field */
    SCM _pe;
    for (c=0; c<i; c++, _pelst=CDR(_pelst))
      {
	_pe=CAR(_pelst);
	ASSERT(NIMP(_pe) && CONSP(_pe), _pe, ARG2, s_fl_setpup_entries);
	ASSERT(ilength(_pe)<=4, _pe, ARG2, s_fl_setpup_entries);
	pe[c].text=NULL;
	pe[c].callback=NULL;
	pe[c].shortcut=NULL;
	pe[c].mode=0;
	if (ilength(_pe)>=1)
	  { SCM _t;
	    _t=CAR(_pe); _pe=CDR(_pe);
	    ASSERT((INUMP(_t) && INUM(_t)==0) || 
		    (NIMP(_t) && STRINGP(_t)), _t, ARG1, 
		   s_fl_setpup_entries);
	    if (!INUMP(_t))
	      pe[c].text    =CHARS(_t);
	  }
	if (ilength(_pe)>=1)
	  { SCM _c;
	    _c=CAR(_pe); _pe=CDR(_pe);
	    ASSERT(BOOL_T==procedurep(_c), _c, ARG2, s_fl_setpup_entries);
	    cb_add(loc_pup_item_callback, MAKINUM(c), _c);
	    pe[c].callback=cb_pup_item_callback;
	  }
	if (ilength(_pe)>=1)
	  { SCM _s;
	    _s=CAR(_pe); _pe=CDR(_pe);
	    ASSERT(NIMP(_s) && STRINGP(_s), _s, ARG3, s_fl_setpup_entries);
	    pe[c].shortcut=CHARS(_s);
	  }
	if (ilength(_pe)==1)
	  { SCM _m;
	    _m=CAR(_pe); 
	    ASSERT(INUMP(_m), _m, ARG4, s_fl_setpup_entries);
	    pe[c].mode    =INUM(_m);
	  }
      }
    pe[i+1].text=NULL;
    return MAKINUM(fl_setpup_entries(INUM(_i), &pe[0]));
  }
} /* fl_setpup_entries */
SUBR_int__window(fl_newpup);
static char s_fl_defpup[]="fl_defpup";
SCM nfl_defpup(SCM _w, SCM _s, SCM _args)
{ /* two additional (string, integer args) supported */
  int i, i1=0, i2=0;
  SCM a1=0, a2=0;
  ASSERT(WINP(_w), _w, ARG1, s_fl_defpup);
  ASSERT(NIMP(_s) && STRINGP(_s), _s, ARG2, s_fl_defpup);
  ASSERT(EOL==_args ||
	 (NIMP(_args) && CONSP(_args)), _args, ARG3, s_fl_defpup);
  SFDEBUGMSG(s_fl_defpup);
  i=ilength(_args);
  if (ilength(_args)>0)
    {
      a1=CAR(_args); _args=CDR(_args);
      ASSERT((INUMP(a1) && (i1=1)) ||
	     (NIMP(a1) && STRINGP(a1)), a1, ARG3, s_fl_defpup);
    }
  if (ilength(_args)>0)
    {
      a2=CAR(_args); 
      ASSERT((INUMP(a2) && (i2=1)) ||
	 (NIMP(a2) && STRINGP(a2)), a2, ARG3, s_fl_defpup);
    }
  if (i==0)
    return MAKINUM(fl_defpup(WIN(_w), CHARS(_s)));
  if (i==1)
    {
      if (i1==1)
	return MAKINUM(fl_defpup(WIN(_w), CHARS(_s), INUM(a1)));
      else 
	return MAKINUM(fl_defpup(WIN(_w), CHARS(_s), CHARS(a1)));
    }
  if (i==2)
    {
      if (i1==1 && i2==1)
	return MAKINUM(fl_defpup(WIN(_w), CHARS(_s), INUM(a1), INUM(a2)));
      if (i1==1 && i2==0)
	return MAKINUM(fl_defpup(WIN(_w), CHARS(_s), INUM(a1), CHARS(a2)));
      if (i1==0 && i2==1)
	return MAKINUM(fl_defpup(WIN(_w), CHARS(_s), CHARS(a1), INUM(a2)));
      else
	return MAKINUM(fl_defpup(WIN(_w), CHARS(_s), CHARS(a1), CHARS(a2)));
    }
  return MAKINUM(-1);
} /* fl_defpup */
static char s_fl_addtopup[]="fl_addtopup";
SCM nfl_addtopup(SCM _i, SCM _s, SCM _args)
{ /* two additional (string, integer args) supported */
  int i, i1=0, i2=0;
  SCM a1=0, a2=0;
  ASSERT(INUMP(_i), _i, ARG1, s_fl_addtopup);
  ASSERT(NIMP(_s) && STRINGP(_s), _s, ARG2, s_fl_addtopup);
  ASSERT(EOL==_args ||
	 (NIMP(_args) && CONSP(_args)), _args, ARG3, s_fl_addtopup);
  SFDEBUGMSG(s_fl_addtopup);
  i=ilength(_args);
  if (ilength(_args)>0)
    {
      a1=CAR(_args); _args=CDR(_args);
      ASSERT((INUMP(a1) && (i1=1)) ||
	     (NIMP(a1) && STRINGP(a1)), a1, ARG3, s_fl_addtopup);
    }
  if (ilength(_args)>0)
    {
      a2=CAR(_args); 
      ASSERT((INUMP(a2) && (i2=1)) ||
	 (NIMP(a2) && STRINGP(a2)), a2, ARG3, s_fl_addtopup);
    }
  if (i==0)
    return MAKINUM(fl_addtopup(INUM(_i), CHARS(_s)));
  if (i==1)
    {
      if (i1==1)
	return MAKINUM(fl_addtopup(INUM(_i), CHARS(_s), INUM(a1)));
      else 
	return MAKINUM(fl_addtopup(INUM(_i), CHARS(_s), CHARS(a1)));
    }
  if (i==2)
    {
      if (i1==1 && i2==1)
	return MAKINUM(fl_addtopup(INUM(_i), CHARS(_s), INUM(a1), INUM(a2)));
      if (i1==1 && i2==0)
	return MAKINUM(fl_addtopup(INUM(_i), CHARS(_s), INUM(a1), CHARS(a2)));
      if (i1==0 && i2==1)
	return MAKINUM(fl_addtopup(INUM(_i), CHARS(_s), CHARS(a1), INUM(a2)));
      else
	return MAKINUM(fl_addtopup(INUM(_i), CHARS(_s), CHARS(a1), CHARS(a2)));
    }
  return MAKINUM(-1);
} /* fl_addtopup */
SUBR_int__3int(fl_setpup_mode);
SUBR_void__int(fl_freepup);
SUBR_int__int(fl_dopup);
SUBR_int__int(fl_setpup_default_cursor); /* Cursor is integer */
/* */
SUBR_void__2int(fl_setpup_color);
SUBR_void__int(fl_setpup_checkcolor);
SUBR_int__int(fl_setpup_fontsize);
SUBR_int__int(fl_setpup_fontstyle);
/* */
SUBR_int__int(fl_setpup_default_bw);
SUBR_void__2int_charptr(fl_setpup_shortcut);
SUBR_void__2int(fl_setpup_position);
SUBR_void__2int(fl_setpup_selection);
SUBR_void__2int(fl_setpup_shadow);
SUBR_void__2int(fl_setpup_softedge);
SUBR_void__2int(fl_setpup_bw);
SUBR_void__int_charptr(fl_setpup_title);
static char s_fl_setpup_entercb[]="fl_setpup_entercb";
SCM nfl_setpup_entercb(SCM _i, SCM _proc, SCM _v)
{
  ASSERT(INUMP(_i), _i, ARG1, s_fl_setpup_entercb);
  ASSERT(BOOL_T==procedurep(_proc), _proc, ARG2, s_fl_setpup_entercb);
  /*ASSERT(INUMP(_v), _v, ARG3, s_fl_setpup_entercb);*/
  SFDEBUGMSG(s_fl_setpup_entercb);
  cb_add(loc_pup_enter_callback, _i, _proc);
  fl_setpup_entercb(INUM(_i), cb_pup_enter_callback, (void *)_v);
  return UNSPECIFIED;
} /* fl_setpup_entercb */
static char s_fl_setpup_leavecb[]="fl_setpup_leavecb";
SCM nfl_setpup_leavecb(SCM _i, SCM _proc, SCM _v)
{
  ASSERT(INUMP(_i), _i, ARG1, s_fl_setpup_leavecb);
  ASSERT(BOOL_T==procedurep(_proc), _proc, ARG2, s_fl_setpup_leavecb);
  /*ASSERT(INUMP(_v), _v, ARG3, s_fl_setpup_leavecb);*/
  SFDEBUGMSG(s_fl_setpup_leavecb);
  cb_add(loc_pup_leave_callback, _i, _proc);
  fl_setpup_leavecb(INUM(_i), cb_pup_leave_callback, (void *)_v);
  return UNSPECIFIED;
} /* fl_setpup_leavecb */
SUBR_void__3int(fl_setpup_pad);
SUBR_int__2int(fl_setpup_cursor);
SUBR_int__int(fl_setpup_maxpup);
SUBR_int__2int(fl_getpup_mode);
SUBR_charptr__2int(fl_getpup_text);
SUBR_void__int(fl_showpup);
SUBR_void__int(fl_hidepup);
SUBR_int__int(fl_getpup_items);
SUBR_int__void(fl_current_pup);
static char s_fl_setpup_itemcb[]="fl_setpup_itemcb";
SCM nfl_setpup_itemcb(SCM _i1, SCM _i2, SCM _proc)
{
  ASSERT(INUMP(_i1), _i1, ARG1, s_fl_setpup_itemcb);
  ASSERT(INUMP(_i2), _i2, ARG2, s_fl_setpup_itemcb);
  ASSERT(BOOL_T==procedurep(_proc), _proc, ARG2, s_fl_setpup_itemcb);
  SFDEBUGMSG(s_fl_setpup_itemcb);
  cb_add(loc_pup_item_callback, _i1, _proc);
  fl_setpup_itemcb(INUM(_i1), INUM(_i2), cb_pup_item_callback);
  return UNSPECIFIED;
} /* fl_setpup_itemcb */
static char s_fl_setpup_menucb[]="fl_setpup_menucb";
SCM nfl_setpup_menucb(SCM _i, SCM _proc)
{
  ASSERT(INUMP(_i), _i, ARG1, s_fl_setpup_menucb);
  ASSERT(BOOL_T==procedurep(_proc), _proc, ARG2, s_fl_setpup_menucb);
  SFDEBUGMSG(s_fl_setpup_menucb);
  cb_add(loc_pup_menu_callback, _i, _proc);
  fl_setpup_menucb(INUM(_i), cb_pup_menu_callback);
  return UNSPECIFIED;
} /* fl_setpup_menucb */
SUBR_void__3int(fl_setpup_submenu);

/* initialize functions */
void s_init_popup_fct()
{
  MSUBR_2(fl_setpup_entries);
  MSUBR_1(fl_newpup);
  MSUBR_L(fl_defpup);
  MSUBR_L(fl_addtopup);
  MSUBR_3(fl_setpup_mode);
  MSUBR_1(fl_freepup);
  MSUBR_1(fl_dopup);
  MSUBR_1(fl_setpup_default_cursor);
  MSUBR_2(fl_setpup_color);
  MSUBR_1(fl_setpup_checkcolor);
  MSUBR_1(fl_setpup_fontsize);
  MSUBR_1(fl_setpup_fontstyle);
  MSUBR_2(fl_setpup_default_bw);
  MSUBR_3(fl_setpup_shortcut);
  MSUBR_2(fl_setpup_position);
  MSUBR_2(fl_setpup_selection);
  MSUBR_2(fl_setpup_shadow);
  MSUBR_2(fl_setpup_softedge);
  MSUBR_2(fl_setpup_bw);
  MSUBR_2(fl_setpup_title);
  MSUBR_3(fl_setpup_entercb);
  MSUBR_3(fl_setpup_leavecb);
  MSUBR_3(fl_setpup_pad);
  MSUBR_2(fl_setpup_cursor);
  MSUBR_1(fl_setpup_maxpup);
  MSUBR_2(fl_getpup_mode);
  MSUBR_2(fl_getpup_text);
  MSUBR_1(fl_showpup);
  MSUBR_1(fl_hidepup);
  MSUBR_1(fl_getpup_items);
  MSUBR_0(fl_current_pup);
  MSUBR_3(fl_setpup_itemcb);
  MSUBR_2(fl_setpup_menucb);
  MSUBR_3(fl_setpup_submenu);
  return;
}

/*
** EOF
*/



