/*
**
** SForms : Interface to the XForms library for SCM
**
** This file is part of the SForms extension for SCM.
** (for XForms version 0.88 & SCM version 5b2 / 5c3)
**
** Author : Johannes Leveling
** eMail  : Johannes.Leveling@Informatik.Uni-Oldenburg.De
** Copyright (C) 1997, 1998 Free Software Foundation, Inc.
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this software; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

/*
** s_smacro.h - functions returning a string
** nov 07 1998
*/

#define SUBR_charptr__void(_func)          \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(){                          \
SFDEBUGMSG(s_ ##_func);                    \
return MAKSTR((char *)_func());}    /* ok */

#define SUBR_charptr__int(_func)           \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i){                    \
ASSERT(INUMP(_i), _i, ARG1, #_func);       \
SFDEBUGMSG(s_ ##_func);                    \
return MAKSTR((char *)_func(INUM(_i)));}   \
                                    /* ok */

#define SUBR_charptr__objectptr(_func)     \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o){                    \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
SFDEBUGMSG(s_ ##_func);                    \
return MAKSTR((char *)_func(OBJ(_o)));}    \
                                    /* ok */

#define SUBR_charptr__objectptr_int(_func) \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _i){            \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(INUMP(_i), _i, ARG2, #_func);       \
SFDEBUGMSG(s_ ##_func);                    \
return MAKSTR((char *)_func(OBJ(_o),       \
              INUM(_i)));}          /* ok */

#define SUBR_charptr__2int(_func)          \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2){          \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
return MAKSTR((char *)_func(INUM(_i1),     \
              INUM(_i2)));}         /* ok */

#define SUBR_charptr__charptr(_func)       \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _s){                    \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                   _s, ARG1, #_func);      \
SFDEBUGMSG(s_ ##_func);                    \
return MAKSTR((char *)_func(CHARS(_s)));}  \
                                    /* ok */

#define SUBR_charptr__2charptr(_func)      \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _s1, SCM _s2){          \
ASSERT(NIMP(_s1) && STRINGP(_s1),          \
                   _s1, ARG1, #_func);     \
ASSERT(NIMP(_s2) && STRINGP(_s2),          \
                   _s2, ARG2, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
return MAKSTR((char *)_func(CHARS(_s1),    \
              CHARS(_s2)));}        /* ok */

#define SUBR_charptr__4charptr(_func)      \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _s1, SCM _s2,           \
               SCM _args){                 \
SCM _s3, _s4;                              \
ASSERT(ilength(_args)==2, _args,           \
       ARG3, #_func);                      \
_s3=CAR(_args); _args=CDR(_args);          \
_s4=CAR(_args);                            \
ASSERT(NIMP(_s1) && STRINGP(_s1),          \
                   _s1, ARG1, #_func);     \
ASSERT(NIMP(_s2) && STRINGP(_s2),          \
                   _s2, ARG2, #_func);     \
ASSERT(NIMP(_s3) && STRINGP(_s3),          \
                   _s3, ARG3, #_func);     \
ASSERT(NIMP(_s4) && STRINGP(_s4),          \
                   _s4, ARG4, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
return MAKSTR((char *)_func(CHARS(_s1),    \
              CHARS(_s2),CHARS(_s3),       \
              CHARS(_s4)));}        /* ok */

/*
** EOF
*/









