/*
**
** SForms : Interface to the XForms library for SCM
**
** This file is part of the SForms extension for SCM.
** (for XForms version 0.88 & SCM version 5b2 / 5c3)
**
** Author : Johannes Leveling
** eMail  : Johannes.Leveling@Informatik.Uni-Oldenburg.De
** Copyright (C) 1997, 1998 Free Software Foundation, Inc.
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this software; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

/*
** s_vmacro.h - function with no return value (UNSPECIFIED)
** nov 07 1998
*/

#define SUBR_void__void(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(){                          \
SFDEBUGMSG(s_ ##_func);                    \
_func();                                   \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__int(_func)              \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1){                   \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1));                          \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__2int(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2){          \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), INUM(_i2));               \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__uint_int(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2){          \
ASSERT(INUMP(_i1)&&INUM(_i1)>=0, _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), INUM(_i2));               \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__int_uint(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2){          \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2)&&INUM(_i2)>=0, _i2, ARG2, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), INUM(_i2));               \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__int_double(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2){          \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(DOUBLEP(_i2), _i2, ARG2, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), DOUBLE(_i2));               \
return UNSPECIFIED;}                /* ok */


#define SUBR_void__3int(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _i3){ \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
ASSERT(INUMP(_i3), _i3, ARG3, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), INUM(_i2), INUM(_i3));    \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__2int_double(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _i3){ \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
ASSERT(DOUBLEP(_i3), _i3, ARG3, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), INUM(_i2), DOUBLE(_i3));    \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__int_2double(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _i3){ \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(DOUBLEP(_i2), _i2, ARG2, #_func);     \
ASSERT(DOUBLEP(_i3), _i3, ARG3, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), DOUBLE(_i2), DOUBLE(_i3));    \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__uint_2double(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _i3){ \
ASSERT(INUMP(_i1)&&INUM(_i1)>=0, _i1, ARG1, #_func);     \
ASSERT(DOUBLEP(_i2), _i2, ARG2, #_func);     \
ASSERT(DOUBLEP(_i3), _i3, ARG3, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), DOUBLE(_i2), DOUBLE(_i3));    \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__2int_uint(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _i3){ \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
ASSERT(INUMP(_i3)&&INUM(_i3)>=0, _i3, ARG3, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), INUM(_i2), INUM(_i3));    \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__3uint(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _i3){ \
ASSERT(INUMP(_i1)&&INUM(_i1)>=0, _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2)&&INUM(_i2)>=0, _i2, ARG2, #_func);     \
ASSERT(INUMP(_i3)&&INUM(_i3)>=0, _i3, ARG3, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), INUM(_i2), INUM(_i3));    \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__uint_double(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _d){ \
ASSERT(INUMP(_i1)&&INUM(_i1)>=0, _i1, ARG1, #_func);     \
ASSERT(DOUBLEP(_d), _d, ARG2, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), DOUBLE(_d));    \
return UNSPECIFIED;}                /* ok */


#define SUBR_void__uint(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1){                   \
ASSERT(INUMP(_i1) && INUM(_i1) >= 0,       \
                   _i1, ARG1, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1));                          \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__double(_func)          \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _d){          \
ASSERT(DOUBLEP(_d), _d, ARG1, #_func);   \
SFDEBUGMSG(s_ ##_func);                    \
_func(DOUBLE(_d));           \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__2double(_func)          \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _d1, SCM _d2){          \
ASSERT(DOUBLEP(_d1), _d1, ARG1, #_func);   \
ASSERT(DOUBLEP(_d2), _d2, ARG2, #_func);   \
SFDEBUGMSG(s_ ##_func);                    \
_func(DOUBLE(_d1), DOUBLE(_d2));           \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__3double(_func)          \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _d1, SCM _d2, SCM _d3){ \
ASSERT(DOUBLEP(_d1), _d1, ARG1, #_func);   \
ASSERT(DOUBLEP(_d2), _d2, ARG2, #_func);   \
ASSERT(DOUBLEP(_d3), _d3, ARG3, #_func);   \
SFDEBUGMSG(s_ ##_func);                    \
_func(DOUBLE(_d1), DOUBLE(_d2),            \
      DOUBLE(_d3));                        \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__int_charptr(_func)      \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i, SCM _s){            \
ASSERT(INUMP(_i), _i, ARG1, #_func);       \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG2, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i), CHARS(_s));                \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__charptr_int(_func)      \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _s, SCM _i){            \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG1, #_func);     \
ASSERT(INUMP(_i), _i, ARG2, #_func);       \
SFDEBUGMSG(s_ ##_func);                    \
_func(CHARS(_s), INUM(_i));                \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__charptr_2int(_func)     \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _s, SCM _i1, SCM _i2){  \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG1, #_func);     \
ASSERT(INUMP(_i1), _i1, ARG2, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG3, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(CHARS(_s), INUM(_i1), INUM(_i2));    \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__charptr(_func)          \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _s){                    \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG1, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(CHARS(_s));                          \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__2charptr(_func)         \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _s1, SCM _s2){          \
ASSERT(NIMP(_s1) && STRINGP(_s1),          \
                   _s1, ARG1, #_func);     \
ASSERT(NIMP(_s2) && STRINGP(_s2),          \
                   _s2, ARG2, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(CHARS(_s1), CHARS(_s2));             \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__3charptr(_func)         \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _s1, SCM _s2, SCM _s3){ \
ASSERT(NIMP(_s1) && STRINGP(_s1),          \
                   _s1, ARG1, #_func);     \
ASSERT(NIMP(_s2) && STRINGP(_s2),          \
                   _s2, ARG2, #_func);     \
ASSERT(NIMP(_s3) && STRINGP(_s3),          \
                   _s3, ARG3, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(CHARS(_s1), CHARS(_s2), CHARS(_s3)); \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__formptr(_func)          \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _f){                    \
ASSERT(FORMP(_f), _f, ARG1, #_func);       \
SFDEBUGMSG(s_ ##_func);                    \
_func(FORM(_f));                           \
return UNSPECIFIED;}           

#define SUBR_void__formptr_int(_func)      \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _f, SCM _i){            \
ASSERT(FORMP(_f), _f, ARG1, #_func);       \
ASSERT(INUMP(_i), _i, ARG2, #_func);       \
SFDEBUGMSG(s_ ##_func);                    \
_func(FORM(_f), INUM(_i));                 \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__formptr_uint(_func)     \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _f, SCM _i){            \
ASSERT(FORMP(_f), _f, ARG1, #_func);       \
ASSERT(INUMP(_i) && INUM(_i) >= 0,         \
                  _i, ARG2, #_func);       \
SFDEBUGMSG(s_ ##_func);                    \
_func(FORM(_f), INUM(_i));                 \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__formptr_charptr(_func)  \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _f, SCM _s){            \
ASSERT(FORMP(_f), _f, ARG1, #_func);       \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG2, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(FORM(_f), CHARS(_s));                \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__formptr_objectptr(_func)\
static char s_ ## _func[]=# _func;         \
SCM n ## _func(SCM _f, SCM _o){            \
ASSERT(FORMP(_f), _f, ARG1, #_func);       \
ASSERT(OBJP(_o), _o, ARG2, #_func);        \
SFDEBUGMSG(s_ ##_func);                    \
_func(FORM(_f), OBJ(_o));                  \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__formptr_2int(_func)     \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _f, SCM _i1, SCM _i2){  \
ASSERT(FORMP(_f), _f, ARG1, #_func);       \
ASSERT(INUMP(_i1), _i1, ARG2, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG3, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(FORM(_f), INUM(_i1), INUM(_i2));     \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__formptr_2double(_func)  \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _f, SCM _d1, SCM _d2){  \
ASSERT(FORMP(_f), _f, ARG1, #_func);       \
ASSERT(DOUBLEP(_d1), _d1, ARG2, #_func);   \
ASSERT(DOUBLEP(_d2), _d2, ARG3, #_func);   \
SFDEBUGMSG(s_ ##_func);                    \
_func(FORM(_f), DOUBLE(_d1), DOUBLE(_d2)); \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr(_func)        \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o){                    \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o));                            \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_charptr(_func)\
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _s){            \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG2, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), CHARS(_s));                 \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_2charptr(_func)\
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _s1, SCM _s2){  \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(NIMP(_s1) && STRINGP(_s1),          \
                   _s1, ARG2, #_func);     \
ASSERT(NIMP(_s2) && STRINGP(_s2),          \
                   _s2, ARG3, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), CHARS(_s1), CHARS(_s2));    \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_int(_func)    \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _i){            \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(INUMP(_i), _i, ARG2, #_func);       \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), INUM(_i));                  \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_uint(_func)   \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _i){            \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(INUMP(_i) && INUM(_i)>=0, _i,       \
       ARG2, #_func);                      \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), INUM(_i));                  \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_2int(_func)   \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _i1, SCM _i2){  \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(INUMP(_i1), _i1, ARG2, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG3, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), INUM(_i1), INUM(_i2));      \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_2uint(_func)  \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _i1, SCM _i2){  \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(INUMP(_i1) && INUM(_i1)>=0, _i1,    \
       ARG2, #_func);                      \
ASSERT(INUMP(_i2) && INUM(_i2)>=0, _i2,    \
       ARG3, #_func);                      \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), INUM(_i1), INUM(_i2));      \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_charptr_int(_func)\
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _s, SCM _i){    \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG2, #_func);     \
ASSERT(INUMP(_i), _i, ARG3, #_func);       \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), CHARS(_s), INUM(_i));       \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_int_charptr(_func)\
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _i, SCM _s){    \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(INUMP(_i), _i, ARG2, #_func);       \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG3, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), INUM(_i), CHARS(_s));       \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_formptr(_func)\
static char s_ ## _func[]=# _func;         \
SCM n ## _func(SCM _o, SCM _f){            \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(FORMP(_f), _f, ARG2, #_func);       \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), FORM(_f));                  \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_double(_func) \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _d){            \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(DOUBLEP(_d), _d, ARG2, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), DOUBLE(_d));                \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_2double(_func)\
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _d1, SCM _d2){  \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(DOUBLEP(_d1), _d1, ARG2, #_func);   \
ASSERT(DOUBLEP(_d2), _d2, ARG3, #_func);   \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), DOUBLE(_d1), DOUBLE(_d2));  \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_int_double(_func)\
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _i, SCM _d){    \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(INUMP(_i), _i, ARG2, #_func);       \
ASSERT(DOUBLEP(_d), _d, ARG3, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), INUM(_i), DOUBLE(_d));      \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__window(_func)           \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _w){                    \
ASSERT(WINP(_w), _w, ARG1, #_func);        \
SFDEBUGMSG(s_ ##_func);                    \
_func(WIN(_w));                            \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__window_int(_func)       \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _w, SCM _i){            \
ASSERT(WINP(_w), _w, ARG1, #_func);        \
ASSERT(INUMP(_i), _i, ARG2, #_func);       \
SFDEBUGMSG(s_ ##_func);                    \
_func(WIN(_w), INUM(_i));                  \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__window_charptr(_func)   \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _w, SCM _s){            \
ASSERT(WINP(_w), _w, ARG1, #_func);        \
ASSERT(NIMP(_s) && STRINGP(_s), _s,        \
       ARG2, #_func);                      \
SFDEBUGMSG(s_ ##_func);                    \
_func(WIN(_w), CHARS(_s));                 \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__uint_charptr(_func)   \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i, SCM _s){            \
ASSERT(INUMP(_i)&&INUM(_i)>=0, _i, ARG1, #_func);        \
ASSERT(NIMP(_s) && STRINGP(_s), _s,        \
       ARG2, #_func);                      \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i), CHARS(_s));                 \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__window_2int(_func)      \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _w, SCM _i1, SCM _i2){  \
ASSERT(WINP(_w), _w, ARG1, #_func);        \
ASSERT(INUMP(_i1), _i1, ARG2, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG3, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(WIN(_w), INUM(_i1), INUM(_i2));      \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__window_4int(_func)      \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _w, SCM _i1, SCM _args){\
SCM _i2, _i3, _i4;                         \
ASSERT(ilength(_args)==3, _args,           \
       ARG3, #_func);                      \
_i2=CAR(_args); _args=CDR(_args);          \
_i3=CAR(_args); _args=CDR(_args);          \
_i4=CAR(_args);                            \
ASSERT(WINP(_w), _w, ARG1, #_func);        \
ASSERT(INUMP(_i1), _i1, ARG2, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG3, #_func);     \
ASSERT(INUMP(_i3), _i3, ARG4, #_func);     \
ASSERT(INUMP(_i4), _i4, ARG5, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(WIN(_w), INUM(_i1), INUM(_i2),       \
      INUM(_i3), INUM(_i4));               \
return UNSPECIFIED;}                /* ok */

/* */
#define SUBR_void__4int(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _args){\
SCM _i3, _i4;                              \
ASSERT(ilength(_args)==2, _args,           \
       ARG3, #_func);                      \
_i3=CAR(_args); _args=CDR(_args);          \
_i4=CAR(_args);                            \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
ASSERT(INUMP(_i3), _i3, ARG3, #_func);     \
ASSERT(INUMP(_i4), _i4, ARG4, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), INUM(_i2),                \
      INUM(_i3), INUM(_i4));               \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__int_3double(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _args){\
SCM _i3, _i4;                              \
ASSERT(ilength(_args)==2, _args,           \
       ARG3, #_func);                      \
_i3=CAR(_args); _args=CDR(_args);          \
_i4=CAR(_args);                            \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(DOUBLEP(_i2), _i2, ARG2, #_func);     \
ASSERT(DOUBLEP(_i3), _i3, ARG3, #_func);     \
ASSERT(DOUBLEP(_i4), _i4, ARG4, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), DOUBLE(_i2),                \
      DOUBLE(_i3), DOUBLE(_i4));               \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__4uint(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _args){\
SCM _i3, _i4;                              \
ASSERT(ilength(_args)==2, _args,           \
       ARG3, #_func);                      \
_i3=CAR(_args); _args=CDR(_args);          \
_i4=CAR(_args);                            \
ASSERT(INUMP(_i1)&&INUM(_i1)>=0, _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2)&&INUM(_i2)>=0, _i2, ARG2, #_func);     \
ASSERT(INUMP(_i3)&&INUM(_i3)>=0, _i3, ARG3, #_func);     \
ASSERT(INUMP(_i4)&&INUM(_i4)>=0, _i4, ARG4, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), INUM(_i2),                \
      INUM(_i3), INUM(_i4));               \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__uint_3double(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _args){\
SCM _i3, _i4;                              \
ASSERT(ilength(_args)==2, _args,           \
       ARG3, #_func);                      \
_i3=CAR(_args); _args=CDR(_args);          \
_i4=CAR(_args);                            \
ASSERT(INUMP(_i1)&&INUM(_i1)>=0, _i1, ARG1, #_func);     \
ASSERT(DOUBLEP(_i2), _i2, ARG2, #_func);     \
ASSERT(DOUBLEP(_i3), _i3, ARG3, #_func);     \
ASSERT(DOUBLEP(_i4), _i4, ARG4, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), DOUBLE(_i2),                \
      DOUBLE(_i3), DOUBLE(_i4));               \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__5int(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _args){\
SCM _i3, _i4, _i5;                         \
ASSERT(ilength(_args)==3, _args,           \
       ARG3, #_func);                      \
_i3=CAR(_args); _args=CDR(_args);          \
_i4=CAR(_args); _args=CDR(_args);          \
_i5=CAR(_args);                            \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
ASSERT(INUMP(_i3), _i3, ARG3, #_func);     \
ASSERT(INUMP(_i4), _i4, ARG4, #_func);     \
ASSERT(INUMP(_i5), _i5, ARG5, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), INUM(_i2), INUM(_i3),     \
      INUM(_i4), INUM(_i5));               \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__6int(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _args){\
SCM _i3, _i4, _i5, _i6;                    \
ASSERT(ilength(_args)==4, _args,           \
       ARG3, #_func);                      \
_i3=CAR(_args); _args=CDR(_args);          \
_i4=CAR(_args); _args=CDR(_args);          \
_i5=CAR(_args); _args=CDR(_args);          \
_i6=CAR(_args);                            \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
ASSERT(INUMP(_i3), _i3, ARG3, #_func);     \
ASSERT(INUMP(_i4), _i4, ARG4, #_func);     \
ASSERT(INUMP(_i5), _i5, ARG5, #_func);     \
ASSERT(INUMP(_i6), _i6, ARGn, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), INUM(_i2), INUM(_i3),     \
      INUM(_i4), INUM(_i5), INUM(_i6));    \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__int_2double_int_2double(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _args){\
SCM _i3, _i4, _i5, _i6;                    \
ASSERT(ilength(_args)==4, _args,           \
       ARG3, #_func);                      \
_i3=CAR(_args); _args=CDR(_args);          \
_i4=CAR(_args); _args=CDR(_args);          \
_i5=CAR(_args); _args=CDR(_args);          \
_i6=CAR(_args);                            \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(DOUBLEP(_i2), _i2, ARG2, #_func);     \
ASSERT(DOUBLEP(_i3), _i3, ARG3, #_func);     \
ASSERT(INUMP(_i4), _i4, ARG4, #_func);     \
ASSERT(DOUBLEP(_i5), _i5, ARG5, #_func);     \
ASSERT(DOUBLEP(_i6), _i6, ARGn, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), DOUBLE(_i2), DOUBLE(_i3),     \
      INUM(_i4), DOUBLE(_i5), DOUBLE(_i6));    \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__7int(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _args){\
SCM _i3, _i4, _i5, _i6, _i7;               \
ASSERT(ilength(_args)==5, _args,           \
       ARG3, #_func);                      \
_i3=CAR(_args); _args=CDR(_args);          \
_i4=CAR(_args); _args=CDR(_args);          \
_i5=CAR(_args); _args=CDR(_args);          \
_i6=CAR(_args); _args=CDR(_args);          \
_i7=CAR(_args);                            \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
ASSERT(INUMP(_i3), _i3, ARG3, #_func);     \
ASSERT(INUMP(_i4), _i4, ARG4, #_func);     \
ASSERT(INUMP(_i5), _i5, ARG5, #_func);     \
ASSERT(INUMP(_i6), _i6, ARGn, #_func);     \
ASSERT(INUMP(_i7), _i7, ARGn, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), INUM(_i2), INUM(_i3),     \
      INUM(_i4), INUM(_i5), INUM(_i6),     \
      INUM(_i7));                          \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__8int_charptr(_func)     \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _args){\
SCM _i3, _i4, _i5, _i6, _i7, _i8, _s;      \
ASSERT(ilength(_args)==7, _args,           \
       ARG3, #_func);                      \
_i3=CAR(_args); _args=CDR(_args);          \
_i4=CAR(_args); _args=CDR(_args);          \
_i5=CAR(_args); _args=CDR(_args);          \
_i6=CAR(_args); _args=CDR(_args);          \
_i7=CAR(_args); _args=CDR(_args);          \
_i8=CAR(_args); _args=CDR(_args);          \
_s=CAR(_args);                             \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
ASSERT(INUMP(_i3), _i3, ARG3, #_func);     \
ASSERT(INUMP(_i4), _i4, ARG4, #_func);     \
ASSERT(INUMP(_i5), _i5, ARG5, #_func);     \
ASSERT(INUMP(_i6), _i6, ARGn, #_func);     \
ASSERT(INUMP(_i7), _i7, ARGn, #_func);     \
ASSERT(INUMP(_i8), _i8, ARGn, #_func);     \
ASSERT(NIMP(_s) &&STRINGP(_s), _s,         \
       ARGn, #_func);                      \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), INUM(_i2), INUM(_i3),     \
      INUM(_i4), INUM(_i5), INUM(_i6),     \
      INUM(_i7), INUM(_i8), CHARS(_s));    \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__8int(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _args){\
SCM _i3, _i4, _i5, _i6, _i7, _i8;          \
ASSERT(ilength(_args)==6, _args,           \
       ARG3, #_func);                      \
_i3=CAR(_args); _args=CDR(_args);          \
_i4=CAR(_args); _args=CDR(_args);          \
_i5=CAR(_args); _args=CDR(_args);          \
_i6=CAR(_args); _args=CDR(_args);          \
_i7=CAR(_args); _args=CDR(_args);          \
_i8=CAR(_args);                            \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
ASSERT(INUMP(_i3), _i3, ARG3, #_func);     \
ASSERT(INUMP(_i4), _i4, ARG4, #_func);     \
ASSERT(INUMP(_i5), _i5, ARG5, #_func);     \
ASSERT(INUMP(_i6), _i6, ARGn, #_func);     \
ASSERT(INUMP(_i7), _i7, ARGn, #_func);     \
ASSERT(INUMP(_i8), _i8, ARGn, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), INUM(_i2), INUM(_i3),     \
      INUM(_i4), INUM(_i5), INUM(_i6),     \
      INUM(_i7), INUM(_i8));               \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__formptr_2int_charptr(_func)\
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _f, SCM _i1, SCM _args){\
SCM _i2, _s;                               \
ASSERT(ilength(_args)==2, _args,           \
       ARG3, #_func);                      \
_i2=CAR(_args); _args=CDR(_args);          \
_s=CAR(_args);                             \
ASSERT(FORMP(_f), _f, ARG1, #_func);       \
ASSERT(INUMP(_i1), _i1, ARG2, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG3, #_func);     \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG4, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(FORM(_f), INUM(_i1), INUM(_i2),      \
      CHARS(_s));                          \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__2int_charptr(_func)     \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _s){  \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG3, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), INUM(_i2), CHARS(_s));    \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__2int_charptr_int(_func) \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _args){\
SCM _s, _i3;                               \
ASSERT(ilength(_args)==2, _args,           \
       ARG3, #_func);                      \
_s=CAR(_args); _args=CDR(_args);           \
_i3=CAR(_args);                            \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG3, #_func);     \
ASSERT(INUMP(_i3), _i3, ARG4, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(INUM(_i1), INUM(_i2), CHARS(_s),     \
      INUM(_i3));                          \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__formptr_4int(_func)     \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _f, SCM _i1, SCM _args){\
SCM _i2, _i3, _i4;                         \
ASSERT(ilength(_args)==3, _args,           \
       ARG3, #_func);                      \
_i2=CAR(_args); _args=CDR(_args);          \
_i3=CAR(_args); _args=CDR(_args);          \
_i4=CAR(_args);                            \
ASSERT(FORMP(_f), _f, ARG1, #_func);       \
ASSERT(INUMP(_i1), _i1, ARG2, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG3, #_func);     \
ASSERT(INUMP(_i3), _i3, ARG4, #_func);     \
ASSERT(INUMP(_i4), _i4, ARG5, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(FORM(_f), INUM(_i1), INUM(_i2),      \
      INUM(_i3), INUM(_i4));               \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_3int(_func)   \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _i1, SCM _args){\
SCM _i2, _i3;                              \
ASSERT(ilength(_args)==2, _args,           \
       ARG3, #_func);                      \
_i2=CAR(_args); _args=CDR(_args);          \
_i3=CAR(_args);                            \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(INUMP(_i1), _i1, ARG2, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG3, #_func);     \
ASSERT(INUMP(_i3), _i3, ARG4, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), INUM(_i1), INUM(_i2),       \
      INUM(_i3));                          \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_int_charptr_int(_func)\
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _i1, SCM _args){\
SCM _s, _i2;                               \
ASSERT(ilength(_args)==2, _args,           \
       ARG3, #_func);                      \
_s=CAR(_args); _args=CDR(_args);           \
_i2=CAR(_args);                            \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(INUMP(_i1), _i1, ARG2, #_func);     \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG3, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG4, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), INUM(_i1), CHARS(_s),       \
      INUM(_i2));                          \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_double_charptr_int(_func)\
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _d, SCM _args){ \
SCM _s, _i;                                \
ASSERT(ilength(_args)==2, _args,           \
       ARG3, #_func);                      \
_s=CAR(_args); _args=CDR(_args);           \
_i=CAR(_args);                             \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(DOUBLEP(_d), _d, ARG2, #_func);     \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG3, #_func);     \
ASSERT(INUMP(_i), _i, ARG4, #_func);       \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), DOUBLE(_d), CHARS(_s),      \
      INUM(_i));                           \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_2int_double(_func)\
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _i1, SCM _args){\
SCM _i2, _d;                               \
ASSERT(ilength(_args)==2, _args,           \
       ARG3, #_func);                      \
_i2=CAR(_args); _args=CDR(_args);          \
_d=CAR(_args);                             \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(INUMP(_i1), _i1, ARG2, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG3, #_func);     \
ASSERT(DOUBLEP(_d), _d, ARG4, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), INUM(_i1), INUM(_i2),       \
      DOUBLE(_d));                         \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_int_2double(_func)\
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _i1, SCM _args){\
SCM _d1, _d2;                              \
ASSERT(ilength(_args)==2, _args,           \
       ARG3, #_func);                      \
_d1=CAR(_args); _args=CDR(_args);          \
_d2=CAR(_args);                            \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(INUMP(_i1), _i1, ARG2, #_func);     \
ASSERT(DOUBLEP(_d1), _d1, ARG3, #_func);   \
ASSERT(DOUBLEP(_d2), _d2, ARG4, #_func);   \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), INUM(_i1), DOUBLE(_d1),     \
      DOUBLE(_d2));                        \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_2double_int(_func)\
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _d1, SCM _args){\
SCM _d2, _i;                               \
ASSERT(ilength(_args)==2, _args,           \
       ARG3, #_func);                      \
_d2=CAR(_args); _args=CDR(_args);          \
_i=CAR(_args);                             \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(DOUBLEP(_d1), _d1, ARG2, #_func);   \
ASSERT(DOUBLEP(_d2), _d2, ARG3, #_func);   \
ASSERT(INUMP(_i), _i, ARG4, #_func);       \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), DOUBLE(_d1), DOUBLE(_d2),   \
      INUM(_i));                           \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_2int_charptr(_func)\
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _i1, SCM _args){\
SCM _i2, _s;                               \
ASSERT(ilength(_args)==2, _args,           \
       ARG3, #_func);                      \
_i2=CAR(_args); _args=CDR(_args);          \
_s=CAR(_args);                             \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(INUMP(_i1), _i1, ARG2, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG3, #_func);     \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG4, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), INUM(_i1), INUM(_i2),       \
      CHARS(_s));                          \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_2int_charptr_int(_func)\
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _i1, SCM _args){\
SCM _i2, _s, _i3;                          \
ASSERT(ilength(_args)==2, _args,           \
       ARG3, #_func);                      \
_i2=CAR(_args); _args=CDR(_args);          \
_s=CAR(_args); _args=CDR(_args);           \
_i3=CAR(_args);                            \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(INUMP(_i1), _i1, ARG2, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG3, #_func);     \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG4, #_func);     \
ASSERT(INUMP(_i3), _i3, ARG5, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), INUM(_i1), INUM(_i2),       \
      CHARS(_s), INUM(_i3));               \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_int_double_charptr_int(_func)\
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _i1, SCM _args){\
SCM _d, _s, _i2;                           \
ASSERT(ilength(_args)==3, _args,           \
       ARG3, #_func);                      \
_d=CAR(_args); _args=CDR(_args);           \
_s=CAR(_args); _args=CDR(_args);           \
_i2=CAR(_args);                            \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(INUMP(_i1), _i1, ARG2, #_func);     \
ASSERT(DOUBLEP(_d), _d, ARG3, #_func);     \
ASSERT(NIMP(_s) && STRINGP(_s),            \
                    _s, ARG4, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG5, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), INUM(_i1), DOUBLE(_d),      \
      CHARS(_s), INUM(_i2));               \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__objectptr_4int(_func)   \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _o, SCM _i1, SCM _args){\
SCM _i2, _i3, _i4;                         \
ASSERT(ilength(_args)==3, _args,           \
       ARG3, #_func);                      \
_i2=CAR(_args); _args=CDR(_args);          \
_i3=CAR(_args); _args=CDR(_args);          \
_i4=CAR(_args);                            \
ASSERT(OBJP(_o), _o, ARG1, #_func);        \
ASSERT(INUMP(_i1), _i1, ARG2, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG3, #_func);     \
ASSERT(INUMP(_i3), _i3, ARG4, #_func);     \
ASSERT(INUMP(_i4), _i4, ARG5, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(OBJ(_o), INUM(_i1), INUM(_i2),       \
      INUM(_i3), INUM(_i4));               \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__3charptr_int(_func)     \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _s1, SCM _s2, SCM _args){\
SCM _s3, _i;                               \
ASSERT(ilength(_args)==2, _args,           \
       ARG3, #_func);                      \
_s3=CAR(_args); _args=CDR(_args);          \
_i=CAR(_args);                             \
ASSERT(NIMP(_s1) && STRINGP(_s1),          \
                   _s1, ARG1, #_func);     \
ASSERT(NIMP(_s2) && STRINGP(_s2),          \
                   _s2, ARG2, #_func);     \
ASSERT(NIMP(_s3) && STRINGP(_s3),          \
                   _s3, ARG3, #_func);     \
ASSERT(INUMP(_i), _i, ARG4, #_func);       \
SFDEBUGMSG(s_ ##_func);                    \
_func(CHARS(_s1), CHARS(_s2), CHARS(_s3),  \
      INUM(_i));                           \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__points_2int(_func)      \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _p, SCM _i1, SCM _i2){  \
int i, c;                                  \
ASSERT(NIMP(_p) && CONSP(_p), _p,          \
       ARG1, #_func);                      \
ASSERT(INUMP(_i1), _i1, ARG2, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG3, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
i=INUM(_i1);                               \
{ SCM cp, px, py; FL_POINT p[i+1];         \
ASSERT(ilength(_p)==i, _p, ARG1, #_func);  \
for (c=0; c<i; c++, _p=CDR(_p)) {          \
ASSERT(NIMP(CAR(_p)) &&                    \
    CONSP(CAR(_p)), _p, ARG1, #_func);     \
cp=CAR(_p);                                \
ASSERT(ilength(cp)==2, cp, ARG1, #_func);  \
px=CAR(cp); cp=CDR(cp);                    \
py=CAR(cp);                                \
ASSERT(INUMP(px), px, ARG1, #_func);       \
ASSERT(INUMP(py), py, ARG1, #_func);       \
p[c].x=INUM(px); p[c].y=INUM(py); }        \
p[c+1].x=0; p[c+1].y=0;                    \
_func(&p[0], INUM(_i1), INUM(_i2));        \
} return UNSPECIFIED;}              /* ok */


#define SUBR_void__4double(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _args){\
SCM _i3, _i4;                              \
ASSERT(ilength(_args)==2, _args,           \
       ARG3, #_func);                      \
_i3=CAR(_args); _args=CDR(_args);          \
_i4=CAR(_args);                            \
ASSERT(DOUBLEP(_i1), _i1, ARG1, #_func);     \
ASSERT(DOUBLEP(_i2), _i2, ARG2, #_func);     \
ASSERT(DOUBLEP(_i3), _i3, ARG3, #_func);     \
ASSERT(DOUBLEP(_i4), _i4, ARG4, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(DOUBLE(_i1), DOUBLE(_i2),                \
      DOUBLE(_i3), DOUBLE(_i4));               \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__5double(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _args){\
SCM _i3, _i4, _i5;                         \
ASSERT(ilength(_args)==3, _args,           \
       ARG3, #_func);                      \
_i3=CAR(_args); _args=CDR(_args);          \
_i4=CAR(_args); _args=CDR(_args);          \
_i5=CAR(_args);                            \
ASSERT(DOUBLEP(_i1), _i1, ARG1, #_func);     \
ASSERT(DOUBLEP(_i2), _i2, ARG2, #_func);     \
ASSERT(DOUBLEP(_i3), _i3, ARG3, #_func);     \
ASSERT(DOUBLEP(_i4), _i4, ARG4, #_func);     \
ASSERT(DOUBLEP(_i5), _i5, ARG5, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(DOUBLE(_i1), DOUBLE(_i2), DOUBLE(_i3),     \
      DOUBLE(_i4), DOUBLE(_i5));               \
return UNSPECIFIED;}                /* ok */

#define SUBR_void__6double(_func)             \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _args){\
SCM _i3, _i4, _i5, _i6;                    \
ASSERT(ilength(_args)==4, _args,           \
       ARG3, #_func);                      \
_i3=CAR(_args); _args=CDR(_args);          \
_i4=CAR(_args); _args=CDR(_args);          \
_i5=CAR(_args); _args=CDR(_args);          \
_i6=CAR(_args);                            \
ASSERT(DOUBLEP(_i1), _i1, ARG1, #_func);     \
ASSERT(DOUBLEP(_i2), _i2, ARG2, #_func);     \
ASSERT(DOUBLEP(_i3), _i3, ARG3, #_func);     \
ASSERT(DOUBLEP(_i4), _i4, ARG4, #_func);     \
ASSERT(DOUBLEP(_i5), _i5, ARG5, #_func);     \
ASSERT(DOUBLEP(_i6), _i6, ARGn, #_func);     \
SFDEBUGMSG(s_ ##_func);                    \
_func(DOUBLE(_i1), DOUBLE(_i2), DOUBLE(_i3),     \
      DOUBLE(_i4), DOUBLE(_i5), DOUBLE(_i6));    \
return UNSPECIFIED;}                /* ok */


#define SUBR_void__2int_intlist(_func) \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _l){  \
int i, c;                                  \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
ASSERT(NIMP(_l) && CONSP(_l), _l,          \
       ARG3, #_func);                      \
SFDEBUGMSG(s_ ##_func);                    \
i=ilength(_l);                         \
{ GLint p[i]; \
for (c=0; c<i; c++, _l=CDR(_l)) {          \
ASSERT(INUMP(CAR(_l)), CAR(_l), ARG1, #_func);       \
p[c]=INUM(CAR(_l));} \
_func(INUM(_i1), INUM(_i2), &p[0]);      \
} return UNSPECIFIED;}              /* ok */


#define SUBR_void__2int_doublelist(_func) \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _i2, SCM _l){  \
int i, c;                                  \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(INUMP(_i2), _i2, ARG2, #_func);     \
ASSERT(NIMP(_l) && CONSP(_l), _l,          \
       ARG3, #_func);                      \
SFDEBUGMSG(s_ ##_func);                    \
i=ilength(_l);                         \
{ GLfloat p[i];                             \
for (c=0; c<i; c++, _l=CDR(_l)) {          \
ASSERT(DOUBLEP(CAR(_l)), CAR(_l), ARG1, #_func); \
p[c]=DOUBLE(CAR(_l)); }                   \
_func(INUM(_i1), INUM(_i2), &p[0]);      \
} return UNSPECIFIED;}              /* ok */

#define SUBR_void__int_intlist(_func) \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _l){  \
int i, c;                                  \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(NIMP(_l) && CONSP(_l), _l,          \
       ARG2, #_func);                      \
SFDEBUGMSG(s_ ##_func);                    \
i=ilength(_l);                         \
{ GLint p[i]; \
for (c=0; c<i; c++, _l=CDR(_l)) {          \
ASSERT(INUMP(CAR(_l)), CAR(_l), ARG1, #_func);       \
p[c]=INUM(CAR(_l));} \
_func(INUM(_i1), &p[0]);      \
} return UNSPECIFIED;}              /* ok */


#define SUBR_void__int_doublelist(_func) \
static char s_ ## _func[]=#_func;          \
SCM n ## _func(SCM _i1, SCM _l){  \
int i, c;                                  \
ASSERT(INUMP(_i1), _i1, ARG1, #_func);     \
ASSERT(NIMP(_l) && CONSP(_l), _l,          \
       ARG2, #_func);                      \
SFDEBUGMSG(s_ ##_func);                    \
i=ilength(_l);                         \
{ GLfloat p[i];                             \
for (c=0; c<i; c++, _l=CDR(_l)) {          \
ASSERT(DOUBLEP(CAR(_l)), CAR(_l), ARG1, #_func); \
p[c]=DOUBLE(CAR(_l)); }                   \
_func(INUM(_i1), &p[0]);      \
} return UNSPECIFIED;}              /* ok */



/*
** EOF 
*/



