/*
**
** SForms : Interface to the XForms library for SCM
**
** This file is part of the SForms extension for SCM.
** (for XForms version 0.88 & SCM version 5b2 / 5c3)
**
** Author : Johannes Leveling
** eMail  : Johannes.Leveling@Informatik.Uni-Oldenburg.De
** Copyright (C) 1997, 1998 Free Software Foundation, Inc.
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this software; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

/*
** s_xyplot.c
** oct 25 1998
*/
#include "forms.h"
#include "s_macro.h"

/* initialize symbols */
void s_init_xyplot_sym()
{
  /* types */
  sysintern("fl_normal_xyplot",     MAKINUM(FL_NORMAL_XYPLOT));
  sysintern("fl_square_xyplot",     MAKINUM(FL_SQUARE_XYPLOT));
  sysintern("fl_circle_xyplot",     MAKINUM(FL_CIRCLE_XYPLOT));
  sysintern("fl_fill_xyplot",       MAKINUM(FL_FILL_XYPLOT));  
  sysintern("fl_points_xyplot",     MAKINUM(FL_POINTS_XYPLOT));
  sysintern("fl_dashed_xyplot",     MAKINUM(FL_DASHED_XYPLOT));
  sysintern("fl_impulse_xyplot",    MAKINUM(FL_IMPULSE_XYPLOT));
  sysintern("fl_active_xyplot",     MAKINUM(FL_ACTIVE_XYPLOT));
  sysintern("fl_empty_xyplot",      MAKINUM(FL_EMPTY_XYPLOT));
  sysintern("fl_dotted_xyplot",     MAKINUM(FL_DOTTED_XYPLOT));
  sysintern("fl_dotdashed_xyplot",  MAKINUM(FL_DOTDASHED_XYPLOT));
  sysintern("fl_longdashed_xyplot", MAKINUM(FL_LONGDASHED_XYPLOT));
  sysintern("fl_linepoints_xyplot", MAKINUM(FL_LINEPOINTS_XYPLOT));
  /* axis */
  sysintern("fl_linear",            MAKINUM(FL_LINEAR));
  sysintern("fl_log",               MAKINUM(FL_LOG));
  /* grid */
  sysintern("fl_grid_none",         MAKINUM(FL_GRID_NONE));
  sysintern("fl_grid_major",        MAKINUM(FL_GRID_MAJOR));
  sysintern("fl_grid_minor",        MAKINUM(FL_GRID_MINOR));
  /* defaults */
  sysintern("fl_max_xyplotoverlay", MAKINUM(FL_MAX_XYPLOTOVERLAY));
  /* */
  return;
}

/* routines */
SUBR_objectptr__5int_charptr(fl_create_xyplot);
SUBR_objectptr__5int_charptr(fl_add_xyplot);
/* */
static char s_fl_set_xyplot_data[]="fl_set_xyplot_data";
SCM nfl_set_xyplot_data(SCM _o, SCM _l1, SCM _args)
{
  int i, c;
  SCM _l2, _i1, _s1, _s2, _s3;
  /* _l1, _l2 list of numbers */
  ASSERT(ilength(_args)==5, _args, ARG3, s_fl_set_xyplot_data);
  _l2=CAR(_args); _args=CDR(_args);     
  _i1=CAR(_args); _args=CDR(_args);     
  _s1=CAR(_args); _args=CDR(_args);     
  _s2=CAR(_args); _args=CDR(_args);
  _s3=CAR(_args);
  ASSERT(OBJP(_o), _o, ARG1, s_fl_set_xyplot_data);
  ASSERT(NIMP(_l1) && CONSP(_l1), _l1, ARG2, s_fl_set_xyplot_data);
  ASSERT(NIMP(_l2) && CONSP(_l2), _l2, ARG3, s_fl_set_xyplot_data);
  ASSERT(INUMP(_i1), _i1, ARG4, s_fl_set_xyplot_data);
  ASSERT(ilength(_l1)>=INUM(_i1), _i1, ARG4, s_fl_set_xyplot_data);
  ASSERT(ilength(_l2)>=INUM(_i1), _i1, ARG4, s_fl_set_xyplot_data);
  ASSERT(NIMP(_s1) && STRINGP(_s1), _s1, ARG5, s_fl_set_xyplot_data);
  ASSERT(NIMP(_s2) && STRINGP(_s2), _s2, ARGn, s_fl_set_xyplot_data);
  ASSERT(NIMP(_s3) && STRINGP(_s3), _s3, ARGn, s_fl_set_xyplot_data);
  SFDEBUGMSG(s_fl_set_xyplot_data);
  i=ilength(_l1);
  { 
    float fx[i], fy[i];
    for (c=0; c<i; c++, _l1=CDR(_l1))
      {
	ASSERT(DOUBLEP(CAR(_l1)), CAR(_l1), ARG2, s_fl_set_xyplot_data);
	fx[c]=DOUBLE(CAR(_l1)); 
      }
    for (c=0; c<i; c++, _l2=CDR(_l2))
      {
	ASSERT(DOUBLEP(CAR(_l2)), CAR(_l2), ARG3, s_fl_set_xyplot_data);
	fy[c]=DOUBLE(CAR(_l2)); 
      }
    fl_set_xyplot_data(OBJ(_o), &fx[0], &fy[0], INUM(_i1), 
		       CHARS(_s1), CHARS(_s2), CHARS(_s3));
  }
  return UNSPECIFIED;
} /* fl_set_xyplot_data */
static char s_fl_set_xyplot_file[]="fl_set_xyplot_file";
SCM nfl_set_xyplot_file(SCM _o, SCM _s1, SCM _args)
{
  SCM _s2, _s3, _s4;
  ASSERT(ilength(_args)==3, _args, ARG3, s_fl_set_xyplot_file);
  _s2=CAR(_args); _args=CDR(_args);     
  _s3=CAR(_args); _args=CDR(_args);     
  _s4=CAR(_args);
  ASSERT(OBJP(_o), _o, ARG1, s_fl_set_xyplot_file);
  ASSERT(NIMP(_s1) && STRINGP(_s1), _s1, ARG2, s_fl_set_xyplot_file);
  ASSERT(NIMP(_s2) && STRINGP(_s2), _s2, ARG3, s_fl_set_xyplot_file);
  ASSERT(NIMP(_s3) && STRINGP(_s3), _s3, ARG4, s_fl_set_xyplot_file);
  ASSERT(NIMP(_s4) && STRINGP(_s4), _s4, ARG5, s_fl_set_xyplot_file);
  SFDEBUGMSG(s_fl_set_xyplot_file);
  return MAKINUM(fl_set_xyplot_file(OBJ(_o), CHARS(_s1), CHARS(_s2), 
				    CHARS(_s3), CHARS(_s4)));
} /* fl_set_xyplot_file */
static char s_fl_add_xyplot_text[]="fl_add_xyplot_text";
SCM nfl_add_xyplot_text(SCM _o, SCM _d1, SCM _args)
{
  SCM _d2, _s, _i1, _i2;
  ASSERT(ilength(_args)==4, _args, ARG3, s_fl_add_xyplot_text);
  _d2=CAR(_args); _args=CDR(_args);
  _s=CAR(_args); _args=CDR(_args);
  _i1=CAR(_args); _args=CDR(_args);
  _i2=CAR(_args); 
  ASSERT(OBJP(_o), _o, ARG1, s_fl_add_xyplot_text);
  ASSERT(NUMBERP(_d1), _d1, ARG2, s_fl_add_xyplot_text);
  ASSERT(NUMBERP(_d2), _d2, ARG3, s_fl_add_xyplot_text);
  ASSERT(NIMP(_s) && STRINGP(_s), _s, ARG4, s_fl_add_xyplot_text);
  ASSERT(INUMP(_i1), _i1, ARG5, s_fl_add_xyplot_text);
  ASSERT(INUMP(_i2), _i2, ARGn, s_fl_add_xyplot_text);
  SFDEBUGMSG(s_fl_add_xyplot_text);
  fl_add_xyplot_text(OBJ(_o), DOUBLE(_d1), DOUBLE(_d2), 
		     CHARS(_s), INUM(_i1), INUM(_i2));
  return UNSPECIFIED;
} /* fl_add_xyplot_text */
SUBR_void__objectptr_charptr(fl_delete_xyplot_text);
SUBR_int__objectptr_int(fl_set_xyplot_maxoverlays);
static char s_fl_add_xyplot_overlay[]="fl_add_xyplot_overlay";
SCM nfl_add_xyplot_overlay(SCM _o, SCM _i1, SCM _args)
{
  int i, c;
  SCM _l1, _l2, _i2, _i3;
  /* _l1, _l2 list of numbers */
  ASSERT(ilength(_args)==4, _args, ARG3, s_fl_add_xyplot_overlay);
  _l1=CAR(_args); _args=CDR(_args);     
  _l2=CAR(_args); _args=CDR(_args);     
  _i2=CAR(_args); _args=CDR(_args);     
  _i3=CAR(_args); 
  ASSERT(OBJP(_o), _o, ARG1, s_fl_add_xyplot_overlay);
  ASSERT(INUMP(_i1), _i1, ARG2, s_fl_add_xyplot_overlay);
  ASSERT(NIMP(_l1) && CONSP(_l1), _l1, ARG3, s_fl_add_xyplot_overlay);
  ASSERT(NIMP(_l2) && CONSP(_l2), _l2, ARG4, s_fl_add_xyplot_overlay);
  ASSERT(INUMP(_i2), _i2, ARG5, s_fl_add_xyplot_overlay);
  ASSERT(ilength(_l1)>=INUM(_i2), _i2, ARG2, s_fl_add_xyplot_overlay); 
  ASSERT(ilength(_l2)>=INUM(_i2), _i2, ARG2, s_fl_add_xyplot_overlay); 
  ASSERT(INUMP(_i3), _i3, ARGn, s_fl_add_xyplot_overlay);
  SFDEBUGMSG(s_fl_add_xyplot_overlay);
  i=ilength(_l1);
  { 
    float fx[i], fy[i];
    for (c=0; c<i; c++, _l1=CDR(_l1))
      {
	ASSERT(DOUBLEP(CAR(_l1)), CAR(_l1), ARG2, s_fl_add_xyplot_overlay);
	fx[c]=DOUBLE(CAR(_l1)); 
      }
    for (c=0; c<i; c++, _l2=CDR(_l2))
      {
	ASSERT(DOUBLEP(CAR(_l2)), CAR(_l2), ARG3, s_fl_add_xyplot_overlay);
	fy[c]=DOUBLE(CAR(_l2)); 
      }
    fl_add_xyplot_overlay(OBJ(_o), INUM(_i1), 
			  &fx[0], &fy[0], INUM(_i2), INUM(_i3));
  }
  return UNSPECIFIED;
} /* fl_add_xyplot_overlay */
SUBR_void__objectptr_int_charptr_int(fl_add_xyplot_overlay_file);
SUBR_void__objectptr_int(fl_set_xyplot_return);
SUBR_void__objectptr_2int(fl_set_xyplot_xtics);
SUBR_void__objectptr_2int(fl_set_xyplot_ytics);
SUBR_void__objectptr_2double(fl_set_xyplot_xbounds);
SUBR_void__objectptr_2double(fl_set_xyplot_ybounds);
SUBR_2float__objectptr(fl_get_xyplot_xbounds);
SUBR_2float__objectptr(fl_get_xyplot_ybounds);
SUBR_2float_int__objectptr(fl_get_xyplot);
static char s_fl_get_xyplot_data[]="fl_get_xyplot_data";
SCM nfl_get_xyplot_data(SCM _o)
{
  int i, c;                                      
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_xyplot_data);         
  SFDEBUGMSG(s_fl_get_xyplot_data);                     
  i=fl_get_xyplot_numdata(OBJ(_o), 0); /* find out size */
  {
    SCM sf1=EOL, sf2=EOL;
    float f1[i], f2[i];
    fl_get_xyplot_data(OBJ(_o), &f1[0], &f2[0], &i);
    for (c=0; c<i; c++)
      { 
	sf1=cons(makdbl((double)f1[c], 0.0), sf1);
	sf2=cons(makdbl((double)f2[c], 0.0), sf2);
      }
    return cons(sf1, 
	      cons(sf2,
		   cons(MAKINUM(i),           
			EOL)));
  }
} /* fl_get_xyplot_data */
static char s_fl_get_xyplot_overlay_data[]="fl_get_xyplot_overlay_data";
SCM nfl_get_xyplot_overlay_data(SCM _o, SCM _i1)
{ 
  int i, c;                                      
  ASSERT(OBJP(_o), _o, ARG1, s_fl_get_xyplot_overlay_data);         
  SFDEBUGMSG(s_fl_get_xyplot_overlay_data);                     
  i=fl_get_xyplot_numdata(OBJ(_o), INUM(_i1)); /* find out size */
  {
    SCM sf1=EOL, sf2=EOL;
    float f1[i], f2[i];
    fl_get_xyplot_overlay_data(OBJ(_o), INUM(_i1), &f1[0], &f2[0], &i);
    for (c=0; c<i; c++)
      { 
	sf1=cons(makdbl((double)f1[c], 0.0), sf1);
	sf2=cons(makdbl((double)f2[c], 0.0), sf2);
      }
    return cons(sf1, 
	      cons(sf2,
		   cons(MAKINUM(i),           
			EOL)));
  }
} /* fl_get_xyplot_overlay_data */
SUBR_void__objectptr_2int(fl_set_xyplot_overlay_type);
SUBR_void__objectptr_int(fl_delete_xyplot_overlay);
SUBR_void__objectptr_2int_double(fl_set_xyplot_interpolate);
SUBR_void__objectptr_int(fl_set_xyplot_inspect);
SUBR_void__objectptr_int(fl_set_xyplot_symbolsize);
SUBR_void__objectptr_int_2double(fl_replace_xyplot_point);
SUBR_2float__objectptr(fl_get_xyplot_xmapping);
SUBR_2float__objectptr(fl_get_xyplot_ymapping);
static char s_fl_set_xyplot_keys[]="fl_set_xyplot_keys";
SCM nfl_set_xyplot_keys(SCM _o, SCM _sl, SCM _args)
{
  SCM _f1, _f2, _i1;
  int i;
  ASSERT(ilength(_args)==2, _args, ARG3, s_fl_set_xyplot_keys);
  _f1=CAR(_args); _args=CDR(_args);
  _f2=CAR(_args); _args=CDR(_args);
  _i1=CAR(_args); 
  ASSERT(OBJP(_o), _o, ARG1, s_fl_set_xyplot_keys);
  ASSERT(NIMP(_sl) && CONSP(_sl), _sl, ARG2, s_fl_set_xyplot_keys);
  ASSERT(NUMBERP(_f1), _f1, ARG3, s_fl_set_xyplot_keys);
  ASSERT(NUMBERP(_f2), _f2, ARG4, s_fl_set_xyplot_keys);
  ASSERT(INUMP(_i1), _i1, ARG5, s_fl_set_xyplot_keys);
  SFDEBUGMSG(s_fl_set_xyplot_keys);
  i=ilength(_sl);
  {
    SCM str;
    char *s[i];
    while (_sl!=EOL)
      {
	str=CAR(_sl);
	ASSERT(NIMP(str) && STRINGP(str), str, ARG2, s_fl_set_xyplot_keys);
	s[i]=CHARS(str);
	i++;
      }
    fl_set_xyplot_keys(OBJ(_o), &s[0], DOUBLE(_f1), DOUBLE(_f2), INUM(_i1));
  }
  return UNSPECIFIED;
} /* fl_set_xyplot_keys */
SUBR_void__objectptr_int_charptr(fl_set_xyplot_key);
SUBR_void__objectptr_2double_int(fl_set_xyplot_key_position);
SUBR_void__objectptr_2int(fl_set_xyplot_key_font);
SUBR_int__objectptr_int(fl_get_xyplot_numdata);
/* following two functions will be removed */ /* JL : defined in SForms.scm */
/*SUBR_void__objectptr_int(fl_set_xyplot_fontsize);*/
/*SUBR_void__objectptr_int(fl_set_xyplot_fontstyle);*/
/* */
static char s_fl_xyplot_s2w[]="fl_xyplot_s2w";
SCM nfl_xyplot_s2w(SCM _o, SCM _d1, SCM _d2)
{ /* CHECK */
  float f1, f2;
  ASSERT(OBJP(_o), _o, ARG1, s_fl_xyplot_s2w);
  ASSERT(INUMP(_d1), _d1, ARG2, s_fl_xyplot_s2w);
  ASSERT(INUMP(_d2), _d2, ARG3, s_fl_xyplot_s2w);
  SFDEBUGMSG(s_fl_xyplot_s2w);
  fl_xyplot_s2w(OBJ(_o), INUM(_d1), INUM(_d2), 
		&f1, &f2);
  return cons(makdbl((double)f1, 0.0), 
	      cons(makdbl((double)f2, 0.0), EOL));
} /* fl_xyplot_s2w */
static char s_fl_xyplot_w2s[]="fl_xyplot_w2s";
SCM nfl_xyplot_w2s(SCM _o, SCM _d1, SCM _d2)
{ /* CHECK */
  float f1, f2;
  ASSERT(OBJP(_o), _o, ARG1, s_fl_xyplot_w2s);
  ASSERT(DOUBLEP(_d1), _d1, ARG2, s_fl_xyplot_w2s);
  ASSERT(DOUBLEP(_d2), _d2, ARG3, s_fl_xyplot_w2s);
  SFDEBUGMSG(s_fl_xyplot_w2s);
  fl_xyplot_w2s(OBJ(_o), DOUBLE(_d1), DOUBLE(_d2), 
		&f1, &f2);
  return cons(makdbl(f1, 0.0), 
	      cons(makdbl(f2, 0.0), EOL));
} /* fl_xyplot_w2s */
SUBR_void__objectptr_int_double(fl_set_xyplot_xscale);
SUBR_void__objectptr_int_double(fl_set_xyplot_yscale);
/* */
SUBR_void__objectptr(fl_clear_xyplot);
SUBR_void__objectptr_2int(fl_set_xyplot_linewidth);
SUBR_void__objectptr_int(fl_set_xyplot_xgrid);
SUBR_void__objectptr_int(fl_set_xyplot_ygrid);
SUBR_void__objectptr_2charptr(fl_set_xyplot_alphaxtics);
SUBR_void__objectptr_2charptr(fl_set_xyplot_alphaytics);
SUBR_void__objectptr_2charptr(fl_set_xyplot_fixed_xaxis);
SUBR_void__objectptr_2charptr(fl_set_xyplot_fixed_yaxis);
static char s_fl_interpolate[]="fl_interpolate";
SCM nfl_interpolate(SCM _l1, SCM _l2, SCM _args)
{
  int r, i, c;
  SCM _i1, _d, _i2;
  float f1, f2;
  ASSERT(ilength(_args)==3, _args, ARG3, s_fl_interpolate);
  _i1=CAR(_args); _args=CDR(_args);
  _d=CAR(_args); _args=CDR(_args);
  _i2=CAR(_args); 
  ASSERT(NIMP(_l1) && CONSP(_l1), _l1, ARG1, s_fl_interpolate);
  ASSERT(NIMP(_l2) && CONSP(_l2), _l2, ARG2, s_fl_interpolate);
  ASSERT(INUMP(_i1), _i1, ARG3, s_fl_interpolate);  
  ASSERT(ilength(_l1)>=INUM(_i1), _i1, ARG4, s_fl_interpolate); 
  ASSERT(ilength(_l2)>=INUM(_i1), _i1, ARG4, s_fl_interpolate); 
  ASSERT(NUMBERP(_d), _d, ARG4, s_fl_interpolate);
  ASSERT(INUMP(_i2), _i2, ARG5, s_fl_interpolate);
  i=ilength(_l1);
  { 
    float fx[i], fy[i];
    for (c=0; c<i; c++, _l1=CDR(_l1))
      {
	ASSERT(DOUBLEP(CAR(_l1)), CAR(_l1), ARG2, s_fl_set_xyplot_data);
	fx[c]=DOUBLE(CAR(_l1)); 
      }
    for (c=0; c<i; c++, _l2=CDR(_l2))
      {
	ASSERT(DOUBLEP(CAR(_l2)), CAR(_l2), ARG3, s_fl_set_xyplot_data);
	fy[c]=DOUBLE(CAR(_l2)); 
      }
    r=fl_interpolate(&fx[0], &fy[0], 
		     INUM(_i1), &f1, &f2, DOUBLE(_d), INUM(_i2));
    return cons(MAKINUM(r), 
		cons(makdbl(f1, 0.0), 
		     cons(makdbl(f2, 0.0), EOL)));
  }
} /* fl_interpolate */
static char s_fl_set_xyplot_symbol[]="fl_set_xyplot_symbol";
SCM nfl_set_xyplot_symbol(SCM _o, SCM _i, SCM _proc)
{
  ASSERT(OBJP(_o), _o, ARG1, s_fl_set_xyplot_symbol);
  ASSERT(INUMP(_i), _i, ARG2, s_fl_set_xyplot_symbol);
  ASSERT(BOOL_T==procedurep(_proc), _proc, ARG3, s_fl_set_xyplot_symbol);
  SFDEBUGMSG(s_fl_set_xyplot_symbol);
  cb_add(loc_xyplot_symbol_handler, _o, _proc);
  fl_set_xyplot_symbol(OBJ(_o), INUM(_i), cb_xyplot_symbol_handler);
  return UNSPECIFIED;
} /* fl_set_xyplot_symbol */

/* initialize functions */
void s_init_xyplot_fct()
{
  MSUBR_L(fl_create_xyplot);
  MSUBR_L(fl_add_xyplot);
  MSUBR_L(fl_set_xyplot_data);
  MSUBR_L(fl_set_xyplot_file);
  MSUBR_L(fl_add_xyplot_text);
  MSUBR_2(fl_delete_xyplot_text);
  MSUBR_2(fl_set_xyplot_maxoverlays);
  MSUBR_L(fl_add_xyplot_overlay);
  MSUBR_L(fl_add_xyplot_overlay_file);
  MSUBR_2(fl_set_xyplot_return);
  MSUBR_3(fl_set_xyplot_xtics);
  MSUBR_3(fl_set_xyplot_ytics);
  MSUBR_3(fl_set_xyplot_xbounds);
  MSUBR_3(fl_set_xyplot_ybounds);
  MSUBR_1(fl_get_xyplot_xbounds);
  MSUBR_1(fl_get_xyplot_ybounds);
  MSUBR_1(fl_get_xyplot);
  MSUBR_1(fl_get_xyplot_data);
  MSUBR_2(fl_get_xyplot_overlay_data);
  MSUBR_3(fl_set_xyplot_overlay_type);
  MSUBR_2(fl_delete_xyplot_overlay);
  MSUBR_L(fl_set_xyplot_interpolate);
  MSUBR_2(fl_set_xyplot_inspect);
  MSUBR_2(fl_set_xyplot_symbolsize);
  MSUBR_L(fl_replace_xyplot_point);
  MSUBR_1(fl_get_xyplot_xmapping);
  MSUBR_1(fl_get_xyplot_ymapping);
  MSUBR_L(fl_set_xyplot_keys);
  MSUBR_3(fl_set_xyplot_key);
  MSUBR_L(fl_set_xyplot_key_position);
  MSUBR_L(fl_set_xyplot_key_font);
  MSUBR_2(fl_get_xyplot_numdata);
  MSUBR_3(fl_xyplot_s2w);
  MSUBR_3(fl_xyplot_w2s);
  MSUBR_3(fl_set_xyplot_xscale);
  MSUBR_3(fl_set_xyplot_yscale);
  MSUBR_1(fl_clear_xyplot);
  MSUBR_3(fl_set_xyplot_linewidth);
  MSUBR_2(fl_set_xyplot_xgrid);
  MSUBR_2(fl_set_xyplot_ygrid);
  MSUBR_3(fl_set_xyplot_alphaxtics);
  MSUBR_3(fl_set_xyplot_alphaytics);
  MSUBR_3(fl_set_xyplot_fixed_xaxis);
  MSUBR_3(fl_set_xyplot_fixed_yaxis);
  MSUBR_L(fl_interpolate);
  MSUBR_3(fl_set_xyplot_symbol);
  return;
}

/*
** EOF
*/









